/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#ifndef __LLIB2D_H_
#define __LLIB2D_H_

#include <libPs2.h>

/*
 * 2D Image System
 */

typedef struct {
    int pivots;     // numero de pivotes
    int last;       // ultimo pivote indexado
    int *ox,*oy;    // coordenadas originales a pivotar
    int *fx,*fy;    // coordenadas resultantes
    } llib2D_Pivot;


//
// 2D Sprites Functions
//
/**
  * este metodo resetea toda la informacion referente al Sprite, ser necesario
  * cada vez que queramos dibujar un sprite nuevo.
  */
void spriteReset();
/**
  * Especifica la textura que usa este sprite
  * @param indexTexture id de la textura que queremos usar
  */
void spriteSetTexture(int indexTexture);
/**
  * Especifica el tamao del Sprite en pixels
  * @param x tamao en x
  * @param y tamao en y
  */
void spriteSetSize(int x, int y);
/**
  * Especifica el color del Sprite, por defecto el color es (0x80,0x80,0x80), 
  * los colores por debajo o por encima aplican el tint
  * @param r color Red (0x00 - 0xFF)
  * @param g color Green (0x00 - 0xFF)
  * @param b color Blue (0x00 - 0xFF)
  */
void spriteSetColor(u_char r,u_char g,u_char b);
/**
  * Especifica el valor de transparencia del sprite
  * @param a alpha Value 
  */
void spriteSetAlpha(u_char a);
/**
  * Traslada el sprite los pixels indicados
  * @param x traslate en x
  * @param y traslate en y
  */
void spriteTranslate(int x,int y);
/**
  * escala el sprite
  * @param x size en x (
  * @param y size en y
  */
void spriteScale(float x,float y);
/**
  * rota el sprite el angulo indicado (en radianes)
  * @param angle angulo a rotar en Radianes
  */
void spriteRotate(float angle);
/**
  * rota el sprite el angulo indicado, el angulo se indica en un byte 
  * @param angle angulo a rotar en hex (0x00 -> 0grados 0xFF-> 2PI Radianes)
  */
void spriteRotateHex(u_char angle);
/**
  * Este metodo envia el sprite al GS
  */
void spriteRender();

//
// 2D Pivots Functions
//
/**
  * crea una estructura del tipo llib2D_Pivot indicando cuantos pivotes queremos
  * controlar, luego esa estructura deberemos enlazarla con el sprite para que 
  * se le apliquen todas sus modificaciones. 
  * @param n numero de pivotes a controlar
  */
llib2D_Pivot *spritePivotCreate(int n);
/**
  * asocia la estructura para pivotar al sprite con el que se est trabajando
  * @param pivot estructura para guardar los pivotes
  */
void spriteSetPivot(llib2D_Pivot *pivot);

/**
  * aade un pivote a la estructura indicada
  * @param pivot estructura para guardar los pivotes
  * @param x coordenada x del punto que queremos pivotar
  * @param y coordenada y del punto que queremos pivotar
  * @return id identificador del punto pivotado
  */
int spriteAddPivot(llib2D_Pivot *pivot,int x,int y);
/**
  * nos devuelve la coordenada X del punto indicado
  * @param pivot estructura para guardar los pivotes
  * @param i identificador del pivote que queremos
  * @return coordenada X del pivote una vez aplicadas las modificaciones
  *         del sprite renderizado
  */
int spriteGetPivotX(llib2D_Pivot *pivot,int i);
/**
  * nos devuelve la coordenada Y del punto indicado
  * @param pivot estructura para guardar los pivotes
  * @param i identificador del pivote que queremos
  * @return coordenada Y del pivote una vez aplicadas las modificaciones
  *         del sprite renderizado
  */
int spriteGetPivotY(llib2D_Pivot *pivot,int i);

void spriteSetFrame(int n);
void spriteRenderPal();

#endif


