/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

//
// llib2DText.h - Le damos este nombre tan extrao para diferenciarlo
//                facilmente de "llibtex".
//                Ponemos el 2DTEXT porque esta lib cuelga directamente del 2d
//

#ifndef __LLIB2DTEXT_H_
#define __LLIB2DTEXT_H_

#include <libPs2.h>

/*
 * 2D TEXT Functions
 *
 */

int myStringLen(char* str);

/**
  * este metodo inicializa la funciones de texto, indica a la libreria cual
  * es la textura que tiene el juego de caracteres.
  * @param n id de la Textura que contiene el juego de caracteres
  * @param nFonts numero de fuentes dentro de la imagen
  * @param charSizeX tamao horizontal en pixels de lo que ocupa cada caracter 
  * @param charSizeY tamao vertical en pixels de lo que ocupa cada caracter 
  */
void textSetFont(unsigned int n, int nFonts, int charSizeX, int charSizeY);
/**
  * establece el padding que se va a usar para cada fuente, el padding es la distancia
  * en pixels que la libreria usa para separar los caracteres al imprimirlos. Por
  * defecto al iniciar estas funciones el sistema aplica un padding igual a CharSizeX * 0.8f
  * @param index identificador de la fuente
  * @param padding distancia en pixels del caracter
  */
void textSetPadding(int index,int padding);
/**
  * establece el factor de escalado para la fuente, el sistema dibuja los caracteres
  * segun los pixels que ocupa cada fuente (osea 1x1), mediante esta funcion podremos
  * aumentar el tamao de la fuente
  * @param index identificador de la fuente
  * @param factor factor de escalado
  */
void textSetSize(int index,float factor);


void textPrintScaleChar(float x,float y);

/**
  * establece la fuente activa, unicamente en el caso de que tengamos varias fuentes
  * en la imagen que estamos usando
  * @param index identificador de la fuente
  */
void textSetFontActive(int index);
/**
  * especificamos el color del texto
  * @param r Valor Red (0x00 - 0xFF)
  * @param g Valor Green (0x00 - 0xFF)
  * @param b Valor Blue (0x00 - 0xFF)
  */
void textSetColor(u_char r,u_char g,u_char b);
/**
  * especificamos el valor de transparencia de la fuente
  * @param alpha Valor de transparencia (0x00 - 0xFF)
  */
void textSetAlpha(u_char alpha);

/**
  * devuelve la altura en pixels de la fuente que tenemos activa, en este calculo se
  * tiene en cuenta tambin el factor de escalado.
  * @return altura en pixels de la fuente
  */
int textGetHeightForFont(int n);
int textGetHeight();
/**
  * devuelve la longitud de una cadena en pixels para la fuente que tengamos activa
  * en estos momentos, este metodo tiene en cuenta el factor de escalado y el padding
  * @param str cadena de texto que queremos medir
  * @return longitud en pixels del texto
  */
int textGetWidthForFont(int n, char *str);
int textGetWidth(char *str);


void textPrintChar(int x,int y,char chr);

void textPrintPrepareTexture();
void textPrintPrepareColor(u_char r,u_char g,u_char b);
void textPrintPrepareAlpha(u_char alpha);
void textPrintPreparedChar(int x,int y,unsigned char chr);



/**
  * imprime un texto en pantalla dandole las coordenadas X Y donde imprimirlo
  * @param x coordenada x para imprimir el Texto
  * @param y coordenada y para imprimir el Texto
  * @param str cadena que queremos imprimir
  */
void textPrintXY(int x, int y, char *str);
/**
  * imprime un texto centrado en pantalla
  * @param y coordenada y para imprimir el Texto
  * @param str cadena que queremos imprimir
  */
void textPrintCenter(int y, char *str);

void textPrintJustify(int x1, int x2, int y, char *str);

#endif

