/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#ifndef __LLIB3D_H
#define __LLIB3D_H

#include <libVu0.h>

#include "smf.h"

// Render Keys
#define RENDER_KEY_GEOM     0x01
#define RENDER_KEY_PLANE 	0x02
#define RENDER_KEY_VERTEX   0x04
#define RENDER_KEY_ST    	0x08

// Render Modes
#define RENDER_POINT     	0x01
#define RENDER_WIREFRAME 	0x02
#define RENDER_FLAT      	0x04
#define RENDER_GORAUD    	0x08
#define RENDER_GORAUD3   	0x10

// Render Maps
#define RENDER_ENV       	0x20

// Render Colors
#define RENDER_COLOR     	0x02
#define RENDER_TEXTURE   	0x04

// Light Status
#define LIGHT_OFF           0x00
#define LIGHT_ON            0x01

// Render Lights
#define LIGHT_DIRECTIONAL   0x01


// Max Lights
#define MAXLIGHTS           3

/* 
 * ObjectCamera - Una camara
 */
typedef struct
    {
    libVu0FMatrix *cameraToScreen;
    libVu0FMatrix *worldToCamera;
	
	libVu0FVector p;

    libVu0FVector posicion; 
    libVu0FVector rotacion; 
	unsigned int minz;
	unsigned int maxz;
    // libVu0FMatrix *worldToScreen; 
    } ObjectCamera;

/* 
 * ObjectLight - Definicion de las luces
 */
typedef struct
    {
    libVu0FVector posicion;    // posicion del foco de luz
    libVu0FVector vector;      // 

    // Valores para calcular la rampa de color
    libVu0FVector ambient;    
    libVu0FVector diffuse;
    libVu0FVector specular;
    libVu0FVector cspecular;    
    libVu0FVector specularNode;
    libVu0FVector cspecularNode;

    // rampas calculadas
    unsigned char colorRed[256];
    unsigned char colorGreen[256];
    unsigned char colorBlue[256];

    int status;                // ON | OFF
    } ObjectLight;


/* Model3DM : Modelo3D basdo en el formato SMF: Simple Model Format v1
 *
 *    Objeto original en formato 3DM para cargar directamente desde
 *    disco
 */
typedef struct 
    {
    unsigned int NUMVERTEX;   // numero de vertices
    unsigned int NUMFACES;    // numero de caras
    unsigned int NUMTVERTEX;  // numero de Tvertices (texturas)
    unsigned int NUMTVFACES;  // numero de Tcaras (texturas)

    // Para recuperar el content al leer el modelo
    // y saber que modos de render permite
    unsigned char content[8];

    // Informacion referencia a la geometria del objeto
    libVu0FVector *vertices;
    int *faces;

    // normales para calcular incidencia de la luz    
    libVu0FVector *normalfaces;
    libVu0FVector *normalvertex;
    libVu0FVector *normalSvertex;

    // informacion sobre el texturado del objeto
    libVu0FVector *tvertices;
    int *Tfaces;
    
    } Model3DM;


/* 
 * Object3D : representacion del objeto 3dm para manipular
 *    
 */
 
 
typedef struct 
    {
    Model3DM *generator;               // Modelo inicial

    libVu0FVector *vertices;           // vertices proyectados
    
    libVu0FVector *normalfaces;        // normales 
    libVu0FVector *normalvertex;       // normales 
    libVu0FVector *normalSvertex;      // normales 

    libVu0FVector *color;              // color de los vertices
    
    // ----------------------------------------------------------
    libVu0FVector *rotation;            // rotacion de este Objeto
    libVu0FVector *translation;         // posisicion del Objeto
    libVu0FMatrix *localToWorld;        // matriz para transformar el objecto al mundo
	libVu0FMatrix *localRotation;        // matriz para transformar el objecto al mundo
    libVu0FMatrix *localToWorldRotation;        // matriz para transformar el objecto al mundo
    libVu0FMatrix *localToScreen;       // matriz para transformar el objeto a la pantalla
    libVu0FMatrix *localToScreenInverse;// Matriz inversa para las luces 
    // ----------------------------------------------------------

    float translateX,translateY;
    float scaleX,scaleY;

    unsigned int RenderMode;  // modo de render   : Point - Wired - Flat - Goraud
    unsigned int RenderColor; // Color del render : Color - Texture - Light
    unsigned int LightMode;   // Directional || full
    unsigned int RenderMap;   // mapeado fx       : env 

    libVu0FVector colorMaterial;
    libVu0FVector colorEnvironment;
    libVu0FVector colorMaterialLight;

    int textureMaterial;
    int textureEnvironment;
	
	libVu0FVector rotacion; 

    ObjectLight   *light[MAXLIGHTS];
    libVu0FVector *vlight;
    	
    } Object3D;

/*
 * CAMERA
 */
ObjectCamera *actualCamera;
ObjectCamera *cameraCreate();
ObjectCamera *cameraCreateZ(int factorZ);
ObjectCamera *cameraCreateForLayer(int factorZ,int layer);
void camTurnHex(ObjectCamera *obj,u_char x,u_char y,u_char z);
void camRotateHex(ObjectCamera *obj,u_char x,u_char y,u_char z);
void camTurn(ObjectCamera *cam, float x, float y, float z);
void camRotate(ObjectCamera *cam, float x, float y, float z);
void camMove(ObjectCamera *cam, float x, float y, float z);
void camPoint(ObjectCamera *cam, float x, float y, float z);
void camPosition(ObjectCamera *cam, float x, float y, float z);
void camReset(ObjectCamera *cam);
void camRender(ObjectCamera *cam);
ObjectCamera *cameraCreateZRange(int factorZ,unsigned int minz, unsigned int maxz);
void cameraFree(ObjectCamera *camera);
void cameraSetActive(ObjectCamera *camera);
/*
 * 3DM
 */
Model3DM *LoadModel3DM(char * filename);
void model3DMFree(Model3DM *obj);
void model3DMScale(Model3DM *obj,float x,float y,float z);
void model3DMScaleTo(Model3DM *modelo3DM,float x,float y,float z);

int model3DMHasKey(Model3DM *obj, int key);
int model3DMHasRender(Model3DM *obj, int renderMode);

/*
 * OBJECTS
 */

Object3D *ObjectCreate(Model3DM *modelo3DM);
void objectFree(Object3D *obj);

// Configure Object
void objectSetRenderMode(Object3D *obj, unsigned int mode);
void objectSetRenderColor(Object3D *obj, unsigned int mode);
void objectSetRenderMap(Object3D *obj, unsigned int mode);

// Color and texture
void objectSetColor(Object3D *obj,unsigned char r,unsigned char g,unsigned char b);
void objectSetAlpha(Object3D *obj,unsigned char alpha);
void objectSetTexture(Object3D *obj,int textureIndex);
void objectSetTextureMap(Object3D *obj,int textureIndex);
void objectSetEnvColor(Object3D *obj,
        unsigned char r,unsigned char g,unsigned char b,unsigned char a);

// Light Mode and lights
void objectSetLightMode(Object3D *obj, unsigned int mode);
void objectSetLight(Object3D *obj,int index, ObjectLight *luz);

// Operate
void objectReset(Object3D *obj);
void objectTranslate(Object3D *obj,float x,float y,float z);
void objectRotate(Object3D *obj,float x,float y,float z);
void objectRotateHex(Object3D *obj,u_char x,u_char y,u_char z);
void objectMove(Object3D *obj,float x,float y,float z);
void objectTurn(Object3D *obj,float x,float y,float z);
void objectTurnHex(Object3D *obj,u_char x,u_char y,u_char z);
void objectTranslate2D(Object3D *obj,float x,float y);
void objectScale2D(Object3D *obj,float x,float y);

// Draw
void objectRender(Object3D *obj);
void objectRender_Calc(Object3D *obj);
void objectRender_Draw(Object3D *obj);
void dibujarcubo(float x, float y, float z, int tam);

/*
 * LIGHTS
 */

ObjectLight *lightCreate(float x,float y,float z);
ObjectLight *LoadLight(char *filename);
void lightFree(ObjectLight *luz);

void lightSetActive(ObjectLight *luz,int status);
void lightReset(ObjectLight *luz);
void lightTranslate(ObjectLight *luz,float x,float y,float z);
void lightUpdateVector(ObjectLight *luz);
void lightSetColor(ObjectLight *luz,float r,float g,float b);

void lightSetAmbient(ObjectLight *luz,float r,float g,float b,float intensity);
void lightSetDiffuse(ObjectLight *luz,float r,float g,float b,float intensity);
void lightSetSpecular(ObjectLight *luz,float r,float g,float b,float intensity);
void lightSetCSpecular(ObjectLight *luz,float r,float g,float b,float intensity);
void lightSetSpecularNode(ObjectLight *luz,float r,float g,float b);
void lightSetCSpecularNode(ObjectLight *luz,float r,float g,float b);

void lightCreateColorRamp(ObjectLight *luz);
void clipRender(libVu0FVector v1, libVu0FVector v2, libVu0FVector v3);
#endif

libVu0FMatrix tmp;
libVu0FVector p, zd, yd;

Object3D *crearcaja(int escalax, int escalay, int escalaz);
