/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#ifndef __LLIBTEX_H_
#define __LLIBTEX_H_



/**
 * Mediante esta estructura tendremos indexadas todas las texturas que carguemos
 * en el sistema
 */
typedef struct
    {
    short used;      /**< Used 0 | 1 */
    int ImageIndex;  /**< identificador de la imagen */
    float S[4];      /**< coordenadas S del recorte de la textura */
    float T[4];      /**< coordenadas T del recorte de la textura */
    } llibtex_Texture;

/**
 * llibtex_Image:
 *    Mediante esta estructura vamos a tener una imagen en memoria, por una imagen
 *    vamos a entender a la correspondencia con un archivo grafico, independientemente
 *    de lo que tenga, ya sea una unica textura, o varios recortes de texturas
 *
 */
typedef struct
    {
    short used;      // Used 0 | 1
    int sizeX;       // resolucion en X de la imagen
    int sizeY;       // resolucion en Y de la imagen

    short flush;     // flush 1 | 0
    short tbp0;      // direccion de la textura
    short tbw;       // with de la textura / 64
    short psm;       // pixel storage mode
    short w,h;       // width y height
    short tfx;       // effect
    short cbp, cpsm; // clut y Storage mode del clut
    short csm;       // clut storage mode
    short csa;
    short cld;       // a saber que es esto
    short filter;    // filtro (for image)

    unsigned char gif; // GIF animated -> 1 | 0
    short frames;      // number of frames
    short frametbw;    // size of frame in tbw (for add to view each frame)
    short currentFrame;


    // CLUT Info
    int clutId;
    
    unsigned char *data;
    // -------------------

    } llibtex_Image;


/**
 * llibtex_Info:
 *    Estructura para guardar la info necesaria
 *
 */
typedef struct {
    unsigned long videoDir_Base;
    unsigned long videoDir_Pointer;

    unsigned int numImages;
    llibtex_Image *listImages;

    unsigned int numTextures;
    llibtex_Texture *listTextures;

    unsigned int fixedNumImages;
    unsigned int fixedNumTextures;
    int textureActive;
    } llibtex_Info;

extern llibtex_Info texInfo;


void initVideoBaseDir();

/**
  * indicamos el modo de pantalla que estamos usando para que la libreria
  * prepare la direccion base a partir de la cual dira subiendo las texturas.
  * @param w Resolucion X de la pantalla
  * @param h Resolucion y de la pantalla
  * @param ztest Ztest
  * @param osm Pixel Storage Mode
  */
void setVideoBaseDir(int w,int h,int ztest,int psm);

/**
  * indicamos cuantas imagenes y cuantas texturas vamos a utilizar, la carga
  * de texturas no es dinamica por ello necesitamos desde un principio dar
  * tamao a los arrays.
  * @param NumImages Numero de Imagenes
  * @param NumTextures Numero de Texturas
  */
void texturePrepareArray(int NumImages, int NumTextures);
/**
  * este metodo nos desplaza la direccion base que ha calculado desde un 
  * principio a la direccion actual, de esa forma podemos cargar una textura
  * y dejarla fuera de la direccion base para que no se libere en ningun momento
  */
void textureFixDir();
/**
  * reinicia la direccion base al inicio (o a la direccion indicada por el
  * metodo fix()), marca todas las imagenes y texturas para que puedan ser 
  * usadas de nuevo.
  */
void textureReset();
/**
  * carga una imagen en formato raw
  * @param filename ruta (relativa) del archivo
  * @param w width de la imagen
  * @param h height de la imagen
  * @param psm pixel storage mode de la imagen
  * @return id de la imagen dentro del array o -1 en caso de no tener espacio disponible
  */
int imageLoadRaw(char *filename,int w,int h,int psm);
int imageLoadIIF(char *filename);
int imageLoadRawC(char *filename,int w,int h,int psm);
int imageLoadIIFC(char *filename, int size);

unsigned char *imageGetClut(int id);
unsigned char *imageGetPixels(int id);

int imageGetWidth(int nImage);
int imageGetHeight(int nImage);


/**
  * establece el parametro TFX de una imagen
  * @param nImage id de la imagen
  * @param tfx tfxMode
  */
void imageSetTfx(unsigned int nImage,int tfx);
    // #define LIB_TFX_MODULATE   0
    // #define LIB_TFX_DECAL      1
    // #define LIB_TFX_HILIGHT    2
    // #define LIB_TFX_GHLIGHT2   3
/**
  * establece el parametro FILTER de una imagen
  * @param nImage id de la imagen
  * @param filter Filter
  */
void imageSetFilter(unsigned int nImage,int filter);
    // #define LIB_FLT_NEAREST                0
    // #define LIB_FLT_LINEAR                 1
    // #define LIB_FLT_NEAREST_MIPMAP_NEAREST 2
    // #define LIB_FLT_NEAREST_MIPMAP_LENEAR  3
    // #define LIB_FLT_LINEAR_MIPMAP_NEAREST  4
    // #define LIB_FLT_LINEAR_MIPMAP_LINEAR   5
    

/**
  * crea una textura a partir de una imagen indicandole el cuadrado que queremos
  * recortar dentro de la imagen, es muy importante indicar que este metodo
  * trabaja con la imagen en Pixels. Este metodo genera una estructura llibtex_Texture
  * que luego podremos recuperar por si queremos dibujar algo con esta textura.
  * @param nImage id de la imagen
  * @param x1 Coordenada x1 de la textura 
  * @param y1 Coordenada y1 de la textura 
  * @param x2 Coordenada x2 de la textura 
  * @param y2 Coordenada y2 de la textura 
  * @return id de la textura dentro del array o -1 en caso de no tener espacio disponible
  */
int textureCreate(unsigned int nImage,int x1,int y1,int x2,int y2);
/**
  * crea una textura a partir de una imagen indicandole el cuadrado que queremos
  * recortar dentro de la imagen, este metodo trabaja con las coordenatas en el
  * fotmato ST (0.0f - 1.0f). Este metodo genera una estructura llibtex_Texture
  * que luego podremos recuperar por si queremos dibujar algo con esta textura.
  * @param nImage id de la imagen
  * @param x1 Coordenada x1 de la textura (float 0.0f - 1.0f)
  * @param y1 Coordenada y1 de la textura (float 0.0f - 1.0f)
  * @param x2 Coordenada x2 de la textura (float 0.0f - 1.0f)
  * @param y2 Coordenada y2 de la textura (float 0.0f - 1.0f)
  * @return id de la textura dentro del array o -1 en caso de no tener espacio disponible
  */
int textureCreateST(unsigned int nImage,float S1,float T1,float S2,float T2);

/**
  * Activa una textura para dibujar
  * @param n identificador de la textura
  */

void imageResetFrame(int n);
void imageSetFrame(int n,int frame);

void textureUseFrame(int n,int frame);
void textureUse(int n);


/**
  * nos devuelve una estructura llibtex_Texture. En esta estructura se encuentra
  * la informacion sobre el recorte de la imagen para poder enviarlo al GS
  * @param n id de la textura
  * @return llibtex_Texture informacion necesaria de esta textura
  */
llibtex_Texture textureGetBox(int n);

void imageUpdateClut(int id);
#endif

