/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#ifndef __LLIBGS_H_
#define __LLIBGS_H_

#include <libPs2.h>

typedef struct
    {
    unsigned char r;
    unsigned char g;
    unsigned char b;
    unsigned char a;
    } gfxRGBA;


    #define GFX_PAL   1 // 640 x 512
    #define GFX_NTSC  2 // 640 x 448
    #define GFX_AUTO  3 // Autodetect PAL vs NTSC




/*
 * Constantes para el GS
 */
#define GSCENTERX 2048
#define GSCENTERY 2048
#define MAXLAYERZ 16

#define GS_BLACK  0x00,0x00,0x00
#define GS_WHITE  0xFF,0xFF,0xFF
#define GS_RED    0xFF,0x00,0x00
#define GS_GREEN  0x00,0xFF,0x00
#define GS_BLUE   0x00,0x00,0xFF
#define GS_YELLOW 0xFF,0xFF,0x00
#define GS_CYAN   0x00,0xFF,0xFF
#define GS_FUSHIA 0xFF,0x00,0xFF


extern int GSSCREEN_W;
extern int GSSCREEN_H;
extern int GSSCREEN_CW;
extern int GSSCREEN_CH;
extern int GSSCREEN_interMode;
extern int GSSCREEN_PSMCT;
extern int GSSCREEN_PSMZ;


extern unsigned int GSLayerZ;
extern qUnion *gsPacket;       // GSPacket Pointer
extern u_int gsPacketCount;    // and Packet count
extern int buffer;             // buffer actual
extern libGsDBuff db;          // doble buffer
extern u_char gfxABE;


/*
 * Funciones basicas para generar los Tags y enviarlos al gs
 */
__inline__ void addXYZ2(u_short x1, u_short y1, u_int z1);
__inline__ void addRGBAQ(u_char R, u_char G, u_char B, u_char A, float Q);
__inline__ void addST(float S, float T);
__inline__ void addUV(u_int U, u_int V);
__inline__ void addPRIM(u_char prim, u_char iip, u_char tme, u_char fge, u_char abe, u_char aa1, u_char fst, u_char ctxt, u_char fix);
__inline__ void addSCISSOR_1(u_short SCAX0, u_short SCAX1, u_short SCAY0, u_short SCAY1);
__inline__ void addALPHA_1(u_char A, u_char B, u_char C, u_char D, u_char FIX);
__inline__ void addTEST_1(u_char ATE, u_char ATST, u_char AREF, u_char AFAIL, u_char DATE, u_char DATM, u_char ZTE, u_char ZTST);


/*
 * Funciones generales
 *
 * La funcion gfxClear() se usa solo para indicarle que empezamos con un
 * frame nuevo para que se encargue de resetear todo 
 */ 
__inline__ void setGsPacketsize(int kbytes);
void gfxInitVideo(int mode);
__inline__ void gfxInitVideoMode(int resx,int resy,short interMode);
__inline__ void gfxSetClearColor(u_char r,u_char g, u_char b);
__inline__ void gfxSyncV();
__inline__ void gfxSwapBuffers();
__inline__ void gfxRender();
__inline__ void gfxClear();
__inline__ void gfxSetAlphaBlending(int status);
__inline__ int gfxGetAlphaBlending();
int gfxNTSCy(int y);

__inline__ void gfxSetScissor(int x1,int y1,int x2,int y2);
__inline__ void gfxClearScissor();
__inline__ void gfxPushScissor();
__inline__ void gfxPopScissor();


/*
 * Funciones para controlar el LayerZ
 *
 */
void setGSLayerZ(unsigned int v);

void layerZInit(int n);
void layerZStartAt(int n,int minvalue);
void layerZReset();
void layerZActive(int n);
unsigned int layerZGetActive();
unsigned int layerZGetMin(int n);
unsigned int layerZGetMax(int n);


/*
 * Tipicas funciones para dibujar primitivas en 2D, Todas las primitivas
 * las dibujo en solido (si quieres el contorno te lo curras a base de lines)
 *
 * Point - Line - Triangle - TriangleGoraund - Box - BoxGoraund - Circle - Oval
 *
 * Para seguir con la logica del LayerZ ninguna primitiva tiene coordenada Z
 * y todas toman como centro de coordenadas (2048-2048)
 */
void gfxClearPrimTag();
void gfxSetColor(u_char r, u_char g, u_char b,u_char alpha);
__inline__ void gfxDrawPoint(int x,int y);
__inline__ void gfxDrawLine(int x1,int y1,int x2,int y2);
__inline__ void gfxDrawTriangle(int x1,int y1,int x2,int y2,int x3,int y3);
__inline__ void gfxDrawBox(int x1,int y1,int x2,int y2);
__inline__ void drawBoxLined(int x1,int y1,int x2,int y2);
__inline__ void drawBoxLined2(int x1,int y1,int x2,int y2, int px);
__inline__ void gfxDrawTriangleGoraud(int x1,int y1,u_char r1, u_char g1, u_char b1,
                            int x2,int y2,u_char r2, u_char g2, u_char b2,
                            int x3,int y3,u_char r3, u_char g3, u_char b3);
__inline__ void gfxDrawBoxGoraud(int x1,int y1,u_char r1, u_char g1, u_char b1,
                       int x2,int y2,u_char r2, u_char g2, u_char b2,
                       int x3,int y3,u_char r3, u_char g3, u_char b3,
                       int x4,int y4,u_char r4, u_char g4, u_char b4);
__inline__ void gfxDrawCircle(int x,int y,int radius,int step);
void gfxDrawOval(int x,int y,int radiusX,int radiusY,int step);

// Added LlibPlanar 2.0
void gfxDrawPolygon(int x,int y,libVu0FVector *v,int n, int filled);

#endif


