/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/


#ifndef __LOADFILE_H_
#define __LOADFILE_H_


// MEDIA MODES
//
#define MODE_HDD    0
#define MODE_CD     1
#define MODE_MC0    2
#define MODE_MC1    3
#define MODE_HOST   4
#define MODE_MASS   5
#define MODE_VHOST  6
#define MODE_PCSX2  6

void findMediaMode(char* archivo1);
void setMediaMode(int media);
int getMediaMode(int media);

void getMediaName(char *fname, char *filename);
void getMediaNameFor(char *fname, char *filename,int media);

int openFile(char *filename, int mode);
void closeFile(int handle);
int readFile(int handle, void *buffer, int size);
unsigned char *readFileBlock(int handle, int size);
int seekFile(int handle, int pos, int rel);

void vopenFile(char *source,int size);
void vcloseFile();
char* vreadFile(int size);
char* vreadFileBlock(int size);
int vseekFile(int pos, int rel);

/*
 * - - - R A W - - - 
 *
 * use:
 *      char *data = RAW_ReadFile("image.raw"); 
 * 
 * you also can read a file with multiple images:
 *
 *      RAW_OpenFile("image.raw");
 *      char *data1 = RAW_readImage(64*64); // read 64x64 bytes
 *		char *data2 = RAW_readImage(64*64*3); // read 64x64 bytes RGB
 *      RAW_CloseFile();
 *
 *
 */

void RAW_OpenFile(char* filename);
void RAW_CloseFile();
unsigned char *RAW_readImage(int size);
unsigned char *RAW_ReadFile(char* filename);
void RAW_seek(int pos, int from);
/*
 * - - - I I F - - - 
 *
 * Use:
 *      IIF_HEADER *iif = IIF_OpenFile("image.iif");
 *      char *clut  = IIF_readClut(iif); 
 *      char *image = IIF_readImage(iif); 
 *      IIF_CloseFile();
 *
 * Please, note that IIF_HEADER use a especific PSM codification
 *         you will check this psm to know if you need load the clut data
 *
 *
 */

#define IIF_RGBA32			0
#define IIF_RGB24			1
#define IIF_RGBA16			2
#define IIF_CLUT8_RGBA32	3
#define IIF_CLUT8_RGBA16	4
#define IIF_CLUT4_RGBA32	5
#define IIF_CLUT4_RGBA16	6

// neofar . addef Gif animated to iif format ;)

#define IIF_GIF256          7
#define IIF_GIF16           8

// IIF HEADER

typedef struct{
	char ID[4]; // 'IIF1' - IIFv (v = version of iif)
	unsigned int width;
	unsigned int height;
	unsigned int psm;
} IIF_HEADER;


IIF_HEADER *IIF_OpenFile(char* filename);
void IIF_CloseFile();
int IIF_isAnimated(IIF_HEADER *iif_file);
int IIF_countFrames(IIF_HEADER *iif_file);
int IIF_hasMoreData(IIF_HEADER *iif_file);
unsigned char *IIF_readClut(IIF_HEADER *iif_file);
unsigned char *IIF_readImage(IIF_HEADER *iif_file);

IIF_HEADER *IIF_vOpenFile(char *filename, int size);
void IIF_vCloseFile();
int IIF_vcountFrames(IIF_HEADER *iif_file);
unsigned char *IIF_vreadClut(IIF_HEADER *iif_file);
unsigned char *IIF_vreadImage(IIF_HEADER *iif_file);


/*
 * - - - G S I - - - 
 *
 * Use:
 *      GSI_FILE *gsi = GSI_OpenFile("image.gsi");
 *      char *clut  = GSI_readClut(gsi); 
 *      char *image = GSI_readImage(gsi); 
 *      GSI_CloseFile();
 *
 *
 */
/*
typedef struct
	{
	unsigned char id[4];				// 'GSI0' 
	unsigned int struct_size;			// = sizeof(GSI_HEAD)
	unsigned int image_struct_ofset;	// offset of GSI_IMAGE
	unsigned int clut_struct_ofset;     // offset of GSI_CLUT
	} GSI_HEAD;

typedef struct
	{
	unsigned int image_psm;				// Pixel Storage Mode
	unsigned int image_width;			// width
	unsigned int image_height;			// height
	unsigned int image_pixel_offset;	// offset of image
	} GSI_IMAGE;

typedef struct
	{
	unsigned int clut_psm;				// clut psm
	unsigned int clut_csm;				// clut csm
	unsigned int clut_width;			// clut width
	unsigned int clut_height;			// clit height
	unsigned int clut_pixel_offset;		// offset
	unsigned int pad1;
	unsigned int pad2[2];
	} GSI_CLUT;

typedef struct
	{
	GSI_HEAD *head;
	GSI_CLUT *clut;
	GSI_IMAGE *image;
	} GSI_FILE;


GSI_FILE *GSI_OpenFile(char* filename);
void GSI_CloseFile();
unsigned char *GSI_readClut(GSI_FILE *gsi_file);
unsigned char *GSI_readImage(GSI_FILE *gsi_file);
*/
void delay(int count);
char *IIF_OpenFileC(char *filename, int seek);
int IIF_countFramesC(IIF_HEADER *iif_file, char *filename);
unsigned char *IIF_readClutC(char *filename, int seek);
unsigned char *IIF_readImageC(IIF_HEADER *iif_file,int seek);
void loadusb();

#endif

