//----------------------------------------------------------------------
//  This program renders a 3D model of a monkey on the screen.
//  The monkey's name is Suzanne and is Blender's mascot.
// 
//  Author: Carlos Neto                            Date: 09-21-2004
//
//  license: GNU Library General Public License Version 2 
//----------------------------------------------------------------------
 
#include <stdlib.h>
#include <stdio.h>
#include "ps2gs.h"
#include "ps2vpu.h"
#include "libvu0.h"
#include "ps2dma.h"

#include "../common/gswrap.h"
#include "../common/gif.h"
#include "../common/camera.h"
#include "../models/monkey.h"

//----------------------------------------------------------------------
ps2_gs_dbuff g_db;
ps2_gs_finish g_finish;

int g_inter, g_out_mode, g_ff_mode, g_resolution;
int g_refresh_rate, g_psm, g_zpsm, g_zbits;

int g_fd_gs;
ps2_vpu *g_vpu0;

#define OFFSET  140

ps2_vu0_fvector modelVerticesD[nVertices];
ps2_vu0_fvector modelFacesNormalsD[nFaces];
ps2_vu0_fvector modelVerticesNormalsD[nVertices]; 
   
ps2_vu0_fvector rotation;
ps2_vu0_fvector translation;
ps2_vu0_fvector l_vector={0.0f,0.0f,-300.0f,0.0f};  // light vector

// global matrices
ps2_vu0_fmatrix localToWorld, localToScreen, worldToScreen;
ps2_vu0_fmatrix cameraToScreen, worldToCamera;

struct rawCamera c1=
{
    {0.0f,0.0f,0.0f,1.0f},    // rot
    {0.0f,0.0f,-400.0f,1.0f}, // p
    {0.0f,0.0f,1.0f,1.0f},    // zd
    {0.0f,-1.0f,0.0f,1.0f},   // yd
};

void init(void)
{
    createCamera(worldToCamera, &c1);
    ps2_vu0_view_screen_matrix(cameraToScreen,512.0f,
        70.0f,70.0f,      // scale factor x,y
        2048.0f,2048.0f,  // center of screen x,y
        1.0f,
        16777215.0f,
        64.0f,
        65536.0f );

    ps2_vu0_mul_matrix(worldToScreen, cameraToScreen, worldToCamera);
    
    // initialize vectors
    translation[0] = 0.0f;
    translation[1] = 0.0f;
    translation[2] = 0.0f;
    translation[3] = 0.0f;

    rotation[0] = 0.0f;
    rotation[1] = 0.0f;
    rotation[2] = 0.0f;
    rotation[3] = 0.0f;
 
    // normalize light vector
    ps2_vu0_normalize(l_vector,l_vector);
}

//----------------------------------------------------------------------
int main(int argc, char *argv[])
{
    int frame, odev;
    ps2_gs_gparam *gp = ps2_gs_get_gparam();
    int i;
    float angle = 0.02f;
    float PI    = 3.141592f;
    float PI_2  = PI * 2.0f;
    ps2_vu0_fmatrix work;
    
    ps2_giftag *giftag;
    ps2_dmatag *dmatag;
    
    typedef struct {
       QWdata gsPacket[16 * 1024];
    } dmaChain;

    dmaChain dmaBuffer[3];
    int dmaCount;
    
    g_vpu0 = ps2_vpu_open(0);
    
    gs_pre_reset();
   
    ps2_gs_reset(0, g_inter, g_out_mode, g_ff_mode, g_resolution,
                    g_refresh_rate);   
    
    ps2_gs_set_dbuff(&g_db, g_psm, gp->width, gp->height,
                      (g_zbits == 0) ? 0 : PS2_GS_ZGREATER, g_zpsm, 1);

    ps2_gs_put_drawenv(&g_db.giftag1);

    ps2_gs_set_finish(&g_finish);
    ps2_gs_wait_finish(&g_finish);

    odev = !ps2_gs_sync_v(0);
    
    ps2_gs_start_display(1);

    ps2_gs_vc_enablevcswitch(acquire, release);
    
    init();
 
    frame = 0;
    for (; ;)
    {       
        ps2_gs_vc_lock();   
                
         // Rotate the planes around the axis.
        rotation[0] += angle; 
        rotation[1] = 0; 
        rotation[2] = PI; 
        if(rotation[0] > PI) rotation[0] -= PI_2;
        if(rotation[1] > PI) rotation[1] -= PI_2;
        if(rotation[2] > PI) rotation[2] -= PI_2;
        
        ps2_vu0_unit_matrix(work);
        ps2_vu0_rot_matrix(work, work, rotation);
        ps2_vu0_trans_matrix(localToWorld, work, translation);

        // Create the final Local->Screen matrix
        ps2_vu0_mul_matrix(localToScreen, worldToScreen, localToWorld);
        
        // transform all vertices
        for(i=0;i<nVertices;i++) 
        {
            float q;
            ps2_vu0_apply_matrix(modelVerticesD[i], localToScreen,
                                                           modelVertices[i]);
            q = 1.0f / modelVerticesD[i][3];

            modelVerticesD[i][0] *= q;
            modelVerticesD[i][1] *= q;
            modelVerticesD[i][2] *= q;
            modelVerticesD[i][3]  = 0.0f;
        }
        
        // transform face normals
        for(i=0;i<nFaces;i++) 
        {
            float q;

            ps2_vu0_apply_matrix(modelFacesNormalsD[i], localToWorld, 
                                                        modelFacesNormals[i]);
            q = 1.0f / modelFacesNormalsD[i][3];
            modelFacesNormalsD[i][0] *= q;
            modelFacesNormalsD[i][1] *= q;
            modelFacesNormalsD[i][2] *= q;
            modelFacesNormalsD[i][3] = 0.0f;
        }
        
        // Transform Vertex normals
        for(i=0;i<nVertices;i++) 
        {
            float q;

            ps2_vu0_apply_matrix(modelVerticesNormalsD[i], localToWorld,
                                 modelVerticesNormals[i]);
            q = 1.0f / modelVerticesNormalsD[i][3];
            modelVerticesNormalsD[i][0] *= q;
            modelVerticesNormalsD[i][1] *= q;
            modelVerticesNormalsD[i][2] *= q;
            modelVerticesNormalsD[i][3] = 0.0f;
        }
        
        dmaCount=0;
        
        // -----------------------------------------------------------------
        // Paint background
        
        gsPacketCount=2; // reserve two qwords for dmatag and giftag

        addPRIM(PS2_GS_PRIM_PRIM_TRIANGLE,
                                    1,     // -> active gouraud 
                                    0,0,0,0,0,0,0);

        addRGBAQ(0xE0, 0xD0, 0x30, 0x80, 1.0f);
        addXYZ2(2048-320,2048-240,0xFFFF);
        addRGBAQ(0xE0, 0xD0, 0x30, 0x80, 1.0f);
        addXYZ2(2048+320,2048-240,0xFFFF);
        addRGBAQ(0xF7, 0x00, 0x38, 0x80, 1.0f);
        addXYZ2(2048-320,2048,0xFFFF);

        addRGBAQ(0xE0, 0xD0, 0x30, 0x80, 1.0f);
        addXYZ2(2048+320,2048-240,0xFFFF);
        addRGBAQ(0xF7, 0x00, 0x38, 0x80, 1.0f);
        addXYZ2(2048-320,2048,0xFFFF);
        addRGBAQ(0xF7, 0x00, 0x38, 0x80, 1.0f);
        addXYZ2(2048+320,2048,0xFFFF);
        
        addRGBAQ(0xDB, 0xFA, 0xD0, 0x80, 1.0f);
        addXYZ2(2048-320,2048+240,0xFFFF);
        addRGBAQ(0xDB, 0xFA, 0xD0, 0x80, 1.0f);
        addXYZ2(2048+320,2048+240,0xFFFF);
        addRGBAQ(0x44, 0x20, 0x10, 0x80, 1.0f);
        addXYZ2(2048-320,2048,0xFFFF);

        addRGBAQ(0xDB, 0xFA, 0xD0, 0x80, 1.0f);
        addXYZ2(2048+320,2048+240,0xFFFF);
        addRGBAQ(0x44, 0x20, 0x10, 0x80, 1.0f);
        addXYZ2(2048-320,2048,0xFFFF);
        addRGBAQ(0x44, 0x20, 0x10, 0x80, 1.0f);
        addXYZ2(2048+320,2048,0xFFFF);

        // set the dmatag to determine the end of the chain
        dmatag = (ps2_dmatag *)&gsPacket[0].ul128;
        dmatag->ID = PS2_DMATAG_END;
        dmatag->QWC = (gsPacketCount - 1); 
        
        // set the giftag to determine the end of the gspacket
        giftag = (ps2_giftag *)&gsPacket[1].ul128;
        giftag->NLOOP = (gsPacketCount - 2); // (gsPacketCount - 2) / NREG
        giftag->EOP = 1;
        giftag->PRE = 0;
        giftag->PRIM = 0;
        giftag->FLG = PS2_GIFTAG_FLG_PACKED;
        giftag->NREG = 1;
        giftag->REGS0 = PS2_GIFTAG_REGS_AD;
                
        // copy gs packet to a temporary dma buffer
        for (i=0; i <= dmatag->QWC; i++)
        {
            dmaBuffer[dmaCount].gsPacket[i].ul128 = gsPacket[i].ul128;   
        }
    
        dmaCount++;
        
        // -----------------------------------------------------------------
        // FLAT MODE

        gsPacketCount=2;    // reserve two qwords for dmatag and giftag
        
        for(i=0;i<nFaces;i++)
        {
            int v1,v2,v3;
            float d_prod;
            v1 = modelFaces[i][0];
            v2 = modelFaces[i][1];
            v3 = modelFaces[i][2];

            // calcuate the inner product also known as dot product
            d_prod = ps2_vu0_inner_product(modelFacesNormalsD[i],l_vector);
            if (d_prod<0.0f) continue;  // backface culling
            
            add_PK_RGBA(140*d_prod,70*d_prod,35*d_prod,0xFF);

            add_PK_XYZ2(modelVerticesD[v1][0]-OFFSET, modelVerticesD[v1][1],
                                                        modelVerticesD[v1][2]);
            add_PK_XYZ2(modelVerticesD[v2][0]-OFFSET, modelVerticesD[v2][1],
                                                        modelVerticesD[v2][2]);
            add_PK_XYZ2(modelVerticesD[v3][0]-OFFSET, modelVerticesD[v3][1],
                                                        modelVerticesD[v3][2]);
        }
        
        // set the dmatag to determine the end of the chain
        dmatag = (ps2_dmatag *)&gsPacket[0].ul128;
        dmatag->ID = PS2_DMATAG_END;
        dmatag->QWC = (gsPacketCount - 1); 
        
        // set the giftag to determine the end of the gspacket
        giftag = (ps2_giftag *)&gsPacket[1].ul128;
        giftag->NLOOP = (gsPacketCount - 2)/4; // (gsPacketCount - 2) / NREG
        giftag->EOP = 1;
        giftag->PRE = 1;
        giftag->PRIM = PS2_GS_SETREG_PRIM(PS2_GS_PRIM_PRIM_TRIANGLE,
                                                          0,0,0,0,0,0,0,0);
        giftag->FLG = PS2_GIFTAG_FLG_PACKED;
        giftag->NREG = 4;
        giftag->REGS0 = PS2_GIFTAG_REGS_RGBAQ;
        giftag->REGS1 = PS2_GIFTAG_REGS_XYZ2;
        giftag->REGS2 = PS2_GIFTAG_REGS_XYZ2;
        giftag->REGS3 = PS2_GIFTAG_REGS_XYZ2;
        
        // copy gs packet to a temporary dma buffer
        for (i=0; i <= dmatag->QWC; i++)
        {
            dmaBuffer[dmaCount].gsPacket[i].ul128 = gsPacket[i].ul128;   
        }
    
        dmaCount++;
        
        // -----------------------------------------------------------------
        // GORAUD MODE

        gsPacketCount=2;    // reserve two qwords for dmatag and giftag
        
        for(i=0;i<nFaces;i++)
        {
            int v1,v2,v3;
            float d_prod[3];
            v1 = modelFaces[i][0];
            v2 = modelFaces[i][1];
            v3 = modelFaces[i][2];

            // calculate dot products
            d_prod[0] = ps2_vu0_inner_product(modelVerticesNormalsD[v1],l_vector);
            if (d_prod[0]<0.0f) d_prod[0]=0.0f;
            d_prod[1] = ps2_vu0_inner_product(modelVerticesNormalsD[v2],l_vector);
            if (d_prod[1]<0.0f) d_prod[1]=0.0f;
            d_prod[2] = ps2_vu0_inner_product(modelVerticesNormalsD[v3],l_vector);
            if (d_prod[2]<0.0f) d_prod[2]=0.0f;
                                    
            // vertex 1
            add_PK_RGBA(140*d_prod[0], 70*d_prod[0], 35*d_prod[0], 0x80);
            add_PK_XYZ2(modelVerticesD[v1][0]+OFFSET, modelVerticesD[v1][1],
                                                       modelVerticesD[v1][2]);
            // vertex 2
            add_PK_RGBA(140*d_prod[1], 70*d_prod[1], 35*d_prod[1], 0x80);
            add_PK_XYZ2(modelVerticesD[v2][0]+OFFSET, modelVerticesD[v2][1],
                                                       modelVerticesD[v2][2]);
            // vertex 3
            add_PK_RGBA(140*d_prod[2], 70*d_prod[2], 35*d_prod[2], 0x80);
            add_PK_XYZ2(modelVerticesD[v3][0]+OFFSET, modelVerticesD[v3][1],
                                                       modelVerticesD[v3][2]);
        }
        
        // set the dmatag to determine the end of the chain
        dmatag = (ps2_dmatag *)&gsPacket[0].ul128;
        dmatag->ID = PS2_DMATAG_END;
        dmatag->QWC = (gsPacketCount - 1);
        
        // set the giftag to determine the end of the gspacket
        giftag = (ps2_giftag *)&gsPacket[1].ul128;
        giftag->NLOOP = (gsPacketCount - 2)/2; // (gsPacketCount - 2) / NREG
        giftag->EOP = 1;
        giftag->PRE = 1;
        giftag->PRIM = PS2_GS_SETREG_PRIM(PS2_GS_PRIM_PRIM_TRIANGLE,
                                                          1,0,0,0,0,0,0,0);
        giftag->FLG = PS2_GIFTAG_FLG_PACKED;
        giftag->NREG = 2;
        giftag->REGS0 = PS2_GIFTAG_REGS_RGBAQ;
        giftag->REGS1 = PS2_GIFTAG_REGS_XYZ2;
               
        // copy gs packet to a temporary dma buffer
        for (i=0; i <= dmatag->QWC; i++)
        {
            dmaBuffer[dmaCount].gsPacket[i].ul128 = gsPacket[i].ul128;   
        }
        
        dmaCount++;
        
        // -----------------------------------------------------------------
        odev = !ps2_gs_sync_v(0);
        
        ps2_gs_set_half_offset((frame & 1) ? &g_db.draw1 : &g_db.draw0, odev);
        ps2_gs_swap_dbuff(&g_db, frame);
        
        // send dma chain to GS
        for (i=0; i < dmaCount; i++)
        {
            ps2_dma_start(g_fd_gs, NULL, (ps2_dmatag *)&dmaBuffer[i].gsPacket);   
        }
        // -----------------------------------------------------------------
       
        ps2_gs_vc_unlock();       
        frame++;
    }
    return 0;
}
