#include <llibPlanar.h>

int main(void){
    float x = 0.0f;
    float y = 0.0f;
    float z = 0.0f;

    int idImagen;
    int idTextura;

    Model3DM *modelo3DM;
    Object3D *object;
    ObjectLight *white, *red, *blue;

    setGsPacketsize(16);
    gfxInitVideo(GFX_AUTO);

    // Inicializamos LlibTex
    texturePrepareArray(2, 2);

    // Cargamos una Imagen, y generamos una Textura de esa imagen
    // ambos metodos devuelven el indice, pero al ser un unico registro
    // sabemos que es el 0
    idImagen  = imageLoadRaw("../../resources/images/chip_64.raw", 64, 64, LIB_GS_PSMCT32);
    imageSetFilter(idImagen,LIB_FLT_LINEAR);
    idTextura = textureCreateST(idImagen,0.0f,0.0f,1.0f,1.0f);
            

    // creamos la camara, al crear una camara esta se activa por defecto
    // por lo que no necesitamos ahora coger la referencia
    ObjectCamera *cam = cameraCreateZ(-180);
    
    // leemos un modelo 3DM
    modelo3DM = LoadModel3DM("../../resources/models/toro.3dm");
    
    // creamos una instancia para manipular el objeto
    object = ObjectCreate(modelo3DM);
    objectSetRenderMode(object, RENDER_FLAT);
    objectSetRenderColor(object, RENDER_TEXTURE);
    objectSetTexture(object, 0);
    objectSetColor(object,0xFF,0xFF,0xFF);
    
    // LIGHT WHITE
    white = lightCreate(10,5,-5);
    lightSetAmbient(white,0.1f,0.1f,0.1f,1.0f);
    lightSetDiffuse(white,0.3f,0.3f,0.3f,1.0f);
    lightCreateColorRamp(white);

    // LIGHT RED
    red = lightCreate(-10,-5,-20);
    lightSetAmbient(red,0.1f,0.1f,0.1f,1.0f);
    lightSetDiffuse(red,0.5f,0.0f,0.0f,1.0f);
    lightCreateColorRamp(red);

    // LIGHT BLUE
    blue = lightCreate(-10,-5,-20);
    lightSetAmbient(blue,0.1f,0.1f,0.1f,1.0f);
    lightSetDiffuse(blue,0.0f,0.0f,0.5f,1.0f);
    lightCreateColorRamp(blue);


    // enlazamos las luces
    objectSetLight(object,0, white);
    objectSetLight(object,1, red);
    objectSetLight(object,2, blue);

    FlushCache(0);

	while (1) {
		x+=0.015f;
		y+=0.012f;
		z+=0.013f;

		gfxSyncV();        // Esperamos al Retrazo
		gfxSwapBuffers();  // cambiamos el buffer de dibujado
		gfxClear();        // y limpiamos la pantalla

		// empezamos con un degradado para el fondo
		gfxDrawBoxGoraud(
		-320,-240, 0x00, 0x00, 0x80,
		320,-240, 0x00, 0x00, 0x80,
		-320, 240, 0x10, 0x10, 0x10,
		320, 240, 0x10, 0x10, 0x10);

		// Toro 1 Flat
		lightSetActive(red,LIGHT_ON);
		lightSetActive(blue,LIGHT_OFF);

		objectSetRenderMode(object, RENDER_FLAT);
		objectRotate(object,x,y,z);
		objectTranslate(object,-50,0,0);
		objectRender(object);


		// Toro 2 Goraud
		lightSetActive(red,LIGHT_OFF);
		lightSetActive(blue,LIGHT_ON);

		objectSetRenderMode(object, RENDER_GORAUD);
		objectRotate(object,x,y,z);
		objectTranslate(object,50,0,0);
		objectRender(object);

		camRender(cam);
		gfxRender();       // enviamos todo el flujo al GS
	}

    return 0;
    }
