

#include <llibPlanar.h>
#include <math.h>

#define POINTS 10
#define STARS 20

typedef struct
    {
    int px,py;
    int fx,fy;
    
    int alpha;
    float angle;
    } nodeStar;
    
nodeStar star[STARS];

void fixStar(int id);
void steepStar(int id);
void drawStar(int id);


int main(void){
    int i;
    int tick=0;

    float angle1 = 0.0f;
    float angle2 = 0.0f;


    setGsPacketsize(4);
    gfxInitVideo(GFX_AUTO);

    // 1 - Make a Star Geometry
    //
    libVu0FVector *nodes;
    nodes = (libVu0FVector *)malloc(sizeof(libVu0FVector)*POINTS);
    
    for(i=0;i<POINTS;i++)
        {
        float angulo = (360.0f / POINTS) * i;
        float angle  = angulo * 3.1416f / 180.0f;

        int radius = 80.0f;
        if(i%2) radius = 120.0f;
        
        nodes[i][0] = sinf(angle) * radius;
        nodes[i][1] = cosf(angle) * radius;
        nodes[i][2] = 0.0f; // Z
        nodes[i][3] = 1.0f; // W
        }
        
    // 2 - Make Geometry structure
    //
    geometryCreate(nodes, POINTS);

    while (1) {
        gfxSyncV();
        gfxSwapBuffers();
        gfxClear();

        // dibujamos un fondo degradado
        gfxSetAlphaBlending(1);
        gfxDrawBoxGoraud(
          -320,-256, 0x00, 0x00, 0x80,
           320,-256, 0x00, 0x00, 0x80,
          -320, 256, 0x40, 0x40, 0x20,
           320, 256, 0x40, 0x40, 0x20);
        
        angle1 +=0.02f;
        angle2 +=0.03f;

        // la primera estrella va de parranda
        star[0].px = sinf(angle1) * 300.0f;
        star[0].py = cosf(angle1) * (sinf(angle2)*300.0f);

        // la segunda sigue a la primera a intervalos 
        if(tick++%50==0) fixStar(1);

        // y de la tercera hacia alante todas se siguen
        for(i=2;i<STARS;i++) fixStar(i);
        
        // Stepeamos y dibujamos
        for(i=0;i<STARS;i++) steepStar(i);
        for(i=1;i<STARS;i++) drawStar(STARS-i);

        gfxRender();
        }

    return 0;
    }


void fixStar(int id)
    {
    star[id].fx = star[id-1].px;
    star[id].fy = star[id-1].py;
    }
    
void steepStar(int id)
    {
    float diferenciaX = star[id].px - star[id].fx;
    float diferenciaY = star[id].py - star[id].fy;

    star[id].px -= (diferenciaX*0.1f);
    star[id].py -= (diferenciaY*0.1f);
    
    star[id].angle=(diferenciaX*0.05f)+ (diferenciaY*0.05f);
    star[id].angle *=0.9f;
    }
    
    
void drawStar(int id)
    {
    
    geometryIdentity();
    geometryRotate(0.0f,0.0f,star[id].angle);
    geometryScale(0.3f-id*0.01f,0.3f-id*0.01f,1.0f);
    geometryTranslate2D(star[id].px,star[id].py);
    geometryMatrixVU0Apply();
    
    // Solid Yellow
    gfxSetAlphaBlending(1);
    gfxSetColor(0xFF,0xFF,0x00,0x80-id*4);
    geometryDrawPolygon(1);

    // Black Line
    gfxSetAlphaBlending(0);
    gfxSetColor(0x00,0x00,0x00,0x80);
    geometryDrawPolygon(0);

    }
