#include <llibPlanar.h>
#include "math.h"



typedef struct
    {
    u_char status;      // 0 | 1 particula en uso
    
    float px,py;        // Posicion
    float vx,vy;        // Velocidad
    int type;
    
    int tick;
    int frame;

    } particle;
    
#define MAXPARTICLES 256

particle part[MAXPARTICLES];

void InicializaParticulas();
void CrearParticula(int x,int y);
void StepParticulas();
void DrawParticulas();

int flare1,flare2;
int explosion;


int main(void){
    int idImagen;
    int cont=0;
    float angle1=0,angle2=0,angle3=0;

    // inicializa el modo grafico    
    setGsPacketsize(16);
    gfxInitVideo(GFX_AUTO); // Detect Pal||NTSC

    // Inicializamos LlibTex
    texturePrepareArray(32, 32);
    
    // Cargamos una imagen con los Flares
    idImagen = imageLoadRaw("../../resources/images/flare.raw", 64, 32, LIB_GS_PSMCT32);
    imageSetFilter(idImagen,LIB_FLT_LINEAR);
    flare1 = textureCreateST(idImagen,0.0f,0.0f,0.5f,1.0f);
    flare2 = textureCreateST(idImagen,0.5f,0.0f,1.0f,1.0f);

    // Cargamos la animacion de la explosion (13 frames)
    idImagen = imageLoadIIF("../../resources/images/explosion.iif");
    explosion = textureCreateST(idImagen,0.0f,0.0f,1.0f,1.0f);


    InicializaParticulas();

    FlushCache(0);

    while (1) {

       gfxSyncV();        // Esperamos al Retrazo
       gfxSwapBuffers();  // cambiamos el buffer de dibujado
       gfxClear();        // y limpiamos la pantalla

       // algunas rotaciones para el punto
       angle1 += (sinf(angle3) + 1.0f) * 0.02f;
       angle2 += 0.02f;
       angle3 += 0.01f;

       int px = sinf(angle1)*128;
       int py = cosf(angle2)*128;


       gfxSetColor(0x20,0x20,0x20,0x80);
       gfxDrawBox(-200,-200,200,200);
       gfxSetColor(0xFF,0xFF,0x00,0x80);
       gfxDrawBox(px-4,py-4,px+4,py+4);

       if (cont++%6==0)
          CrearParticula(px,py);

       StepParticulas();
       DrawParticulas();

        
       gfxRender();
       }

    return 0;
    }


void InicializaParticulas()
    {
    int i;

    for(i=0;i<MAXPARTICLES;i++)
        part[i].status = 0;
    }
    
    
void CrearParticula(int x,int y)
    {
    int i;
    int id = -1;

    // buscamos un indice libre
    for(i=0;i<MAXPARTICLES;i++)
        if(part[i].status == 0) 
            {
            id = i;
            break;
            }
    // si no hemos encontrado id salimos
    if (id==-1) return;

    // creamos la particula
    part[id].status = 1;
    part[id].px = x;
    part[id].py = y;

    part[id].vx = randFloat() * 2.0f - 1.0f;
    part[id].vy = randFloat() * 2.0f - 4.0f;
    part[id].type = 0;
    part[id].tick = 0;
    part[id].frame = 0;

    }

void StepParticulas()
    {
    int i;

    // para cada particula que esta viva vamos incrementando
    // su tamao y bajando el alpha
    // buscamos un indice libre
    for(i=0;i<MAXPARTICLES;i++)
    if(part[i].status == 1)
        {
        if(part[i].type == 0)
            {
            part[i].px += part[i].vx;
            part[i].py += part[i].vy;
            part[i].vy +=0.1f;

            if(part[i].px<-200 || part[i].px>200) part[i].type = 1;
            if(part[i].py<-200 || part[i].py>200) part[i].type = 1;
            }
        else
            {
            part[i].tick++;
            if(part[i].tick%5==0)
                part[i].frame++;
                        
            if(part[i].frame>12) part[i].status = 0;

            }
        }

    }

void DrawParticulas()
    {
    int i;

    for(i=0;i<MAXPARTICLES;i++)
        if(part[i].status == 1 && part[i].type == 0)
           {
           spriteReset();
           spriteSetTexture(flare2);
           spriteSetSize(64,64);
           spriteTranslate(part[i].px,part[i].py);
           spriteSetColor(0xFF,0xFF,0x00);
           spriteSetAlpha(0x60);
           spriteRender();

           spriteReset();
           spriteSetTexture(flare1);
           spriteSetSize(16,16);
           spriteTranslate(part[i].px,part[i].py);
           spriteSetColor(0xFF,0x80,0x00);
           spriteSetAlpha(0x80);
           spriteRender();
           }

    for(i=0;i<MAXPARTICLES;i++)
        if(part[i].status == 1 && part[i].type == 1)
           {
           spriteReset();
           spriteSetTexture(explosion);
           spriteSetSize(64+part[i].frame*4,64+part[i].frame*4);
           spriteRotate(i);
           spriteTranslate(part[i].px,part[i].py);
           spriteSetColor(0xFF,0xFF,0xFF);
           spriteSetAlpha(0x80-part[i].frame*4);
           spriteSetFrame(part[i].frame);
           spriteRender();
           }

    }


