#include <llibPlanar.h>
#include "math.h"



typedef struct
    {
    u_char status;      // 0 | 1 particula en uso
    
    float px,py;        // <- posicion
    u_char r,g,b;       // <- color
    u_char alpha;       // <- transparencia
    float size;         // <- tamao

    float angle;        // angulo
    float incAngle;     // incremento para el angulo
    
    } particle;
    
#define MAXPARTICLES 128

particle part[MAXPARTICLES];

void InicializaParticulas();
void CrearParticula(int x,int y);
void StepParticulas();
void DrawParticulas();


int main(void){
    int cont=0;
    float angle1 = 0.0f;
    float angle2 = 0.0f;
    float angle3 = 0.0f;

    // inicializa el modo grafico    
    setGsPacketsize(4);     // Make 4kb for the GSPACKET
    gfxInitVideo(GFX_AUTO); // Detect Pal||NTSC

    // Inicializamos LlibTex
    texturePrepareArray(1, 16);
    
    // Cargamos una Imagen, y generamos una Textura de esa imagen
    imageLoadRaw("../../resources/images/pokemon.raw", 128, 128, LIB_GS_PSMCT32);
    textureCreate(0,0,0,32,32);
    imageSetFilter(0,LIB_FLT_LINEAR);
    
    InicializaParticulas();

    FlushCache(0);

    while (1) {

       gfxSyncV();        // Esperamos al Retrazo
       gfxSwapBuffers();  // cambiamos el buffer de dibujado
       gfxClear();        // y limpiamos la pantalla

       // algunas rotaciones para el punto
       angle1 += (sinf(angle3) + 1.0f) * 0.02f;
       angle2 += 0.02f;
       angle3 += 0.01f;
       
       //if(randInt(0,10) == 0)
       if (cont++%2==0)
          CrearParticula(sinf(angle1)*65,cosf(angle2)*65);
       
       StepParticulas();
       DrawParticulas();
       
        
       gfxRender();       // enviamos todo el flujo al GS
       }

    return 0;
    }


void InicializaParticulas()
    {
    int i;

    for(i=0;i<MAXPARTICLES;i++)
        part[i].status = 0;
    }
    
    
void CrearParticula(int x,int y)
    {
    int i;
    int id = -1;
    // buscamos un indice libre
    for(i=0;i<MAXPARTICLES;i++)
        if(part[i].status == 0) 
            {
            id = i;
            break;
            }
    // si no hemos encontrado id salimos
    if (id==-1) return;


    // creamos la particula
    part[id].status = 1;
    part[id].px = x;
    part[id].py = y;
    part[id].r = randInt(0x80,0xFF);
    part[id].g = randInt(0x80,0xFF);
    part[id].b = randInt(0x80,0xFF);

    part[id].alpha = 0x80;
    part[id].size  = 1.0f;
    part[id].angle = 0;
    part[id].incAngle = randFloat() * 0.1f + 0.1f;
    
    
    }

void StepParticulas()
    {
    int i;

    // para cada particula que esta viva vamos incrementando
    // su tamao y bajando el alpha
    // buscamos un indice libre
    for(i=0;i<MAXPARTICLES;i++)
        if(part[i].status == 1) 
           {
           part[i].size  *= 1.02f;
           part[i].alpha -= 1;
           part[i].angle += part[i].incAngle;
           
           // si la particula se queda Transparente pos entonces
           // la matamos que ya no nos sirve de nada
           if(part[i].alpha == 0) part[i].status = 0;
           }

    }

void DrawParticulas()
    {
    int i;

    for(i=0;i<MAXPARTICLES;i++)
        if(part[i].status == 1) 
           {
           spriteReset();
           spriteSetTexture(0);
           spriteSetSize(64,64);
           spriteRotate(part[i].angle);
           
           spriteTranslate(part[i].px,part[i].py);
           spriteScale(part[i].size,part[i].size);

           spriteSetColor(part[i].r,part[i].g,part[i].b);
           spriteSetAlpha(part[i].alpha);

           
           spriteRender();
           }

    }


