#include <llibPlanar.h>

void drawConsole();


int main(void){
    int i,j;
    float x = 0.0f;
    float y = 0.0f;
    float z = 0.0f;

    int i0,i1,i2,i3;
    int TextureStars,t1,t2,t3;

    Model3DM *modelo3DM;
    Object3D *object;
    ObjectLight *luz;
    
    float angleStars = 0.0f;

    setGsPacketsize(16);
    gfxInitVideo(GFX_AUTO);

    // Inicializamos LlibTex
    texturePrepareArray(6, 6);

    // Cargamos las texturas
    // fondo de Estrellas
    i0 = imageLoadRaw("../../resources/images/stars_128.raw", 128, 128, LIB_GS_PSMCT32);
    imageSetFilter(i0,LIB_FLT_LINEAR);
    TextureStars = textureCreateST(i0,0.0f,0.0f,1.0f,1.0f);

    // ladrillos
    i1 = imageLoadRaw("../../resources/images/brick_64.raw", 64, 64, LIB_GS_PSMCT32);
    t1 = textureCreateST(i1,0.0f,0.0f,1.0f,1.0f);
    imageSetFilter(i1,LIB_FLT_LINEAR);

    // mapa de reflejos
    i2 = imageLoadRaw("../../resources/images/envmetal_64.raw", 64, 64, LIB_GS_PSMCT32);
    t2 = textureCreateST(i2,0.0f,0.0f,1.0f,1.0f);
    imageSetFilter(i2,LIB_FLT_LINEAR);
    imageSetTfx(i2,LIB_TFX_HILIGHT);

    // Cargamos una Imagen, y generamos una Textura de esa imagen
    i3 = imageLoadIIF("../../resources/images/font.iif");
    t3 = textureCreateST(i3,0.0f,0.0f,1.0f,1.0f);
    FlushCache(0);

    // 
    textSetFont(t3, 2, 16, 16); // imagen=0, fonts=2, charsize=(16,16)
    textSetSize(0,1.6f); // fuente[0] -> size 2
    textSetSize(1,2.0f); // fuente[1] -> size 2
    textSetPadding(0,8); // fuente[0] -> padding 8 pixels
    textSetPadding(1,12);// fuente[1] -> padding 12 pixels


    // Setup Console
    consoleCreate();
    consoleSet(0,-200,260,200,0);     // x,y,w,h, font
    consoleSetColor(0xFF,0xFF,0xFF);  // Color for Text
    consoleSetAlpha(0x80);            // Alpha

    // Text
    consolePrintf("LLibPlanar 2.0 \n");
    consolePrintf("By neofar \n\n");
    consolePrintf("Esto ya es un sample complejo... se usa el Layer Z, se usan ");
    consolePrintf("objetos en 3D, Environment Map, un Clut para la textura, ");
    consolePrintf("una consola... y encima, todo esta funcionando a la primera\n");


    // creamos la camara, al crear una camara esta se activa por defecto
    // por lo que no necesitamos ahora coger la referencia
    ObjectCamera *cam = cameraCreateZ(-300);
    
    // leemos un modelo 3DM
    modelo3DM = LoadModel3DM("../../resources/models/hex.3dm");

    // creamos una instancia para manipular el objeto
    object = ObjectCreate(modelo3DM);

    objectSetRenderMode(object, RENDER_FLAT);
    objectSetRenderColor(object, RENDER_TEXTURE);
    objectSetTexture(object,t1);
    objectSetColor(object,0x80,0x80,0x80);

    objectSetRenderMap(object, RENDER_ENV);
    objectSetTextureMap(object,t2);
    objectSetEnvColor(object,0x80,0x80,0x80,0x00);


    // creamos la luz
    luz = lightCreate(-10,-5,-20);
    lightSetAmbient(luz,0.1f,0.1f,0.1f,1.0f);
    lightSetDiffuse(luz,0.5f,0.5f,0.5f,1.0f);
    lightCreateColorRamp(luz);

    // enlazamos la luz al objeto
    objectSetLight(object,0, luz);


    layerZInit(2);

    layerZStartAt(0,0);
    layerZStartAt(1,0xFFFF / 2);


    FlushCache(0);

    while (1) {
		x+=0.015f;
		y+=0.012f;
		z+=0.013f;

		gfxSyncV();        // Esperamos al Retrazo
		gfxSwapBuffers();  // cambiamos el buffer de dibujado
		gfxClear();        // y limpiamos la pantalla


		angleStars+=0.003f;

		// Dibujamos el fondo de estrellitas
		for(i=-1;i<=1;i++)
		for(j=-1;j<=1;j++)
		{
			spriteReset();
			spriteSetTexture(TextureStars);
			spriteSetSize(300,300);
			spriteTranslate(i*300,j*300);
			spriteRotate(angleStars);
			spriteRender();
		}

		// Dibujamos los objetos en 3D
		objectRotate(object,0,0,z);
		objectRotateHex(object,64,0,0);
		objectTranslate(object,-40,0,0);
		objectRotate(object,x,y,z);
		objectRender(object);

		objectRotate(object,0,0,z);
		objectTranslate(object,40,0,0);
		objectRotate(object,x,y,z);
		objectRender(object);


		// Console
		layerZActive(1);
		drawConsole();

		camRender(cam);
		gfxRender();       // enviamos todo el flujo al GS
		}

		return 0;
    }

void drawConsole()
    {
    static float angle =0.0f;
    int x1 =    0;
    int y1 = -200;
    int x2 =  260;
    int y2 =    0;

    angle+=0.008f;
    float factor = cosf(angle)*0.5f-0.5f;

    gfxSetAlphaBlending(1);

    // BackGround
    gfxSetColor(0x00,0x20,0x80,0x30);
    gfxDrawBox(x1,y1,x2,y2);
    gfxDrawBox(x2+12,y1,x2+12+16,y2);

    // Border para la consola
    gfxSetColor(0x80,0x80,0x80,0xFF);
    drawBoxLined2(x1-2,y1-2,x2+2,y2+2,2);

    // ScrollBar
    drawBoxLined2(x2+12,y1-2,x2+12+16,y2+2,2);
    gfxSetColor(0x80,0x80,0x80,0x80);
    gfxDrawBox(x2+12,y1-(factor*80.0f),x2+12+16,y1-(factor*80.0f)+120);
    
    // And render console
    consoleSetView(factor);
    consoleRender();
    }

