#include <stdlib.h>
#include <stdio.h>
#include <sys/fcntl.h>
#include <assert.h>
#include "ps2gs.h"
#include "ps2vpu.h"
#include "gswrap.h"

extern int g_fd_gs;
extern ps2_vpu *g_vpu0;

extern int g_inter, g_out_mode, g_ff_mode, g_resolution;
extern int g_refresh_rate, g_psm, g_zpsm, g_zbits;

//----------------------------------------------------------------------
static int get_out_mode(void)
{
    struct ps2_crtmode current_mode;
        
    ioctl(g_fd_gs, PS2IOC_GCRTMODE, &current_mode);

    return (current_mode.mode);
}

//----------------------------------------------------------------------
void gs_pre_reset(void)
{
    g_inter = PS2_GS_INTERLACE;
    g_ff_mode = PS2_GS_FIELD;
    g_resolution = PS2_GS_640x480;
    g_refresh_rate = PS2_GS_60Hz;
    g_psm = PS2_GS_PSMCT32;
    g_zpsm = PS2_GS_PSMZ24;
    g_zbits = 24;
    
    ps2_gs_vc_graphicsmode();       // set virtual console to graphics mode

    g_fd_gs = ps2_gs_open(-1);      // open GS

    g_out_mode = get_out_mode();    // auto detect output mode
}

//----------------------------------------------------------------------
int release(void)
{
    if (g_fd_gs >= 0)
    {
        ps2_gs_close();             // close GS
        g_fd_gs = -1;
    }
    
    if (g_vpu0)
    {
        ps2_vpu_close(g_vpu0);      // close VU0
    }
    
    return PS2_GS_VC_REL_SUCCESS;
}

//----------------------------------------------------------------------
int acquire(void)
{
    assert(g_fd_gs == -1);
	assert(g_vpu0 == NULL);
    
    g_fd_gs = ps2_gs_open(-1);
    g_vpu0 = ps2_vpu_open(0);       // open VU0
    
    if (g_fd_gs < 0 || g_vpu0 == NULL)
    {
        release();
        return PS2_GS_VC_ACQ_FAILURE;
    }
    
    ps2_vpu_reset(g_vpu0);

    ps2_gs_reset(0, g_inter, g_out_mode, g_ff_mode, g_resolution,
                    g_refresh_rate);
    ps2_gs_start_display(1);

    return PS2_GS_VC_ACQ_SUCCESS;
}
