/*
 *	colorMatrix
 *      Use 4x4 matricies to process color images.
 *      http://www.sgi.com/misc/grafica/matrix/index.html
 *
 *
 *      Paul Haeberli - 1993
 *      Please note that for these operations to be correct, we really must 
 *      operate on linear brightness values. If the input image is in a 
 *      non-linear brightness space RGB colors must be transformed into a 
 *      linear space before these matrix operations are used.
 *
 */

#include <libVu0.h>
#include <math.h>

#define RLUM    (0.3086f)
#define GLUM    (0.6094f)
#define BLUM    (0.0820f)

#define M_PI    (3.14f)



libVu0FMatrix mat; // <- Matriz de trabajo

void _colorMatrixRotateX(float thismat[4][4],float rs,float rc);
void _colorMatrixRotateY(float thismat[4][4],float rs,float rc);
void _colorMatrixRotateZ(float thismat[4][4],float rs,float rc);
void _colorMatrixRotateShearZ(float thismat[4][4],float dx,float dy);


/*
 * Creamos la Matriz unidad 
 */
void colorMatrixIdentity()
	{
    libVu0UnitMatrix(mat);
	}

/* 
 *	xformpnt -	
 *		transform a 3D point using a matrix
 */
void xformpnt(libVu0FMatrix thismat, float x,float y,float z,float *tx,float *ty,float *tz)
	{
    *tx = x*thismat[0][0] + y*thismat[1][0] + z*thismat[2][0] + thismat[3][0];
    *ty = x*thismat[0][1] + y*thismat[1][1] + z*thismat[2][1] + thismat[3][1];
    *tz = x*thismat[0][2] + y*thismat[1][2] + z*thismat[2][2] + thismat[3][2];
    }



/*
 * Aplicamos la matriz
 */
void _colorMatrixVU0Apply(unsigned char *dest,unsigned char *src, int n)
	{
	libVu0FVector v0,v1;

	while(n--) 
 		{
 		v1[0]=src[0];
        v1[1]=src[1];
        v1[2]=src[2];
        v1[3]=src[3];
        src+=4;
        
        libVu0ApplyMatrix(v0, mat, v1);

        dest[0]=v0[0];
        dest[1]=v0[1];
        dest[2]=v0[2];
        dest[3]=v0[3];
        dest+=4;
        }
   }
// Paleta de 256 Colores
void colorMatrixVU0Apply(unsigned char *dest,unsigned char *src)
	{
	_colorMatrixVU0Apply(dest, src, 256);
	
	}

/* 
 *	cscalemat -	
 *		make a color scale marix
 */
void colorMatrixScale(float rscale,float gscale,float bscale)
	{
    libVu0FMatrix mmat;

    mmat[0][0]=rscale;mmat[1][0]=0.0f;  mmat[2][0]=0.0f;   mmat[3][0]=0.0f;
    mmat[0][1]=0.0f;  mmat[1][1]=gscale;mmat[2][1]=0.0f;   mmat[3][1]=0.0f;
    mmat[0][2]=0.0f;  mmat[1][2]=0.0f;  mmat[2][2]=bscale; mmat[3][2]=0.0f;
    mmat[0][3]=0.0f;  mmat[1][3]=0.0f;  mmat[2][3]=0.0f;   mmat[3][3]=1.0f;

    //matrixmult(mmat,mat,mat);
    libVu0MulMatrix(mat, mat, mmat);
    }

/* 
 *	negativemat -	
 *		make a negative marix
 */
void colorMatrixNegative()
	{
    libVu0FMatrix mmat;

    mmat[0][0] = -1.0f; mmat[1][0] =  0.0f; mmat[2][0] =  0.0f; mmat[3][0] = 255.0f;
    mmat[0][1] =  0.0f; mmat[1][1] = -1.0f; mmat[2][1] =  0.0f; mmat[3][1] = 255.0f;
    mmat[0][2] =  0.0f; mmat[1][2] =  0.0f; mmat[2][2] = -1.0f; mmat[3][2] = 255.0f;
    mmat[0][3] =  0.0f; mmat[1][3] =  0.0f; mmat[2][3] =  0.0f; mmat[3][3] =   1.0f;

    libVu0MulMatrix(mat, mat, mmat);
    }

/* 
 *	lummat -	
 *		make a luminance marix
 */
void colorMatrixLum()
	{
    libVu0FMatrix mmat;
    float rwgt, gwgt, bwgt;
    rwgt = RLUM; gwgt = GLUM; bwgt = BLUM;
    
    mmat[0][0] = rwgt; mmat[1][0] = gwgt; mmat[2][0] = bwgt; mmat[3][0] = 0.0f;
    mmat[0][1] = rwgt; mmat[1][1] = gwgt; mmat[2][1] = bwgt; mmat[3][1] = 0.0f;
    mmat[0][2] = rwgt; mmat[1][2] = gwgt; mmat[2][2] = bwgt; mmat[3][2] = 0.0f;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;
    
    libVu0MulMatrix(mat, mat, mmat);
    }

/* 
 *	saturatemat -	
 *		make a saturation marix
 */
void colorMatrixSaturate(float sat)
	{
    libVu0FMatrix mmat;
    float a, b, c, d, e, f, g, h, i;
    float rwgt, gwgt, bwgt;

    rwgt = RLUM; gwgt = GLUM; bwgt = BLUM;

    a = (1.0f-sat)*rwgt + sat; d = (1.0f-sat)*gwgt;       g = (1.0f-sat)*bwgt;
    b = (1.0f-sat)*rwgt;       e = (1.0f-sat)*gwgt + sat; h = (1.0f-sat)*bwgt;
    c = (1.0f-sat)*rwgt;       f = (1.0f-sat)*gwgt;       i = (1.0f-sat)*bwgt + sat;
    
    mmat[0][0] = a;    mmat[1][0] = d;    mmat[2][0] = g;    mmat[3][0] = 0.0f;
    mmat[0][1] = b;    mmat[1][1] = e;    mmat[2][1] = h;    mmat[3][1] = 0.0f;
    mmat[0][2] = c;    mmat[1][2] = f;    mmat[2][2] = i;    mmat[3][2] = 0.0f;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;
    
    libVu0MulMatrix(mat, mat, mmat);
    }

/* 
 *	offsetmat -	
 *		offset r, g, and b
 */
void colorMatrixOffset(float roffset,float goffset,float boffset)
	{
    libVu0FMatrix mmat;

    mmat[0][0] = 1.0f; mmat[1][0] = 0.0f; mmat[2][0] = 0.0f; mmat[3][0] = roffset;
    mmat[0][1] = 0.0f; mmat[1][1] = 1.0f; mmat[2][1] = 0.0f; mmat[3][1] = goffset;
    mmat[0][2] = 0.0f; mmat[1][2] = 0.0f; mmat[2][2] = 1.0f; mmat[3][2] = boffset;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;

    libVu0MulMatrix(mat, mat, mmat);
    }

void colorMatrixRotateX(float rs,float rc)
	{
	_colorMatrixRotateX(mat,rs,rc);
	}    
void colorMatrixRotateY(float rs,float rc)
	{
	_colorMatrixRotateY(mat,rs,rc);
	}    
void colorMatrixRotateZ(float rs,float rc)
	{
	_colorMatrixRotateZ(mat,rs,rc);
	}    


/* 
 *	xrotate -	
 *		rotate about the x (red) axis
 */
void _colorMatrixRotateX(libVu0FMatrix thismat, float rs,float rc)
	{
    libVu0FMatrix mmat;

    mmat[0][0] = 1.0f; mmat[1][0] = 0.0f; mmat[2][0] = 0.0f; mmat[3][0] = 0.0f;
    mmat[0][1] = 0.0f; mmat[1][1] = rc;   mmat[2][1] = -rs;  mmat[3][1] = 0.0f;
    mmat[0][2] = 0.0f; mmat[1][2] = rs;   mmat[2][2] = rc;   mmat[3][2] = 0.0f;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;
    
    libVu0MulMatrix(thismat, thismat, mmat);
    }

/* 
 *	yrotate -	
 *		rotate about the y (green) axis
 */
void _colorMatrixRotateY(libVu0FMatrix thismat, float rs,float rc)
	{
    libVu0FMatrix mmat;

    mmat[0][0] = rc;   mmat[1][0] = 0.0f; mmat[2][0] = rs;   mmat[3][0] = 0.0f;
    mmat[0][1] = 0.0f; mmat[1][1] = 1.0f; mmat[2][1] = 0.0f; mmat[3][1] = 0.0f;
    mmat[0][2] = -rs;  mmat[1][2] = 0.0f; mmat[2][2] = rc;   mmat[3][2] = 0.0f;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;
    
    libVu0MulMatrix(thismat, thismat, mmat);
    }

/* 
 *	zrotate -	
 *		rotate about the z (blue) axis
 */
void _colorMatrixRotateZ(libVu0FMatrix thismat, float rs,float rc)
	{
    libVu0FMatrix mmat;

    mmat[0][0] = rc;   mmat[1][0] = -rs;  mmat[2][0] = 0.0f; mmat[3][0] = 0.0f;
    mmat[0][1] = rs;   mmat[1][1] = rc;   mmat[2][1] = 0.0f; mmat[3][1] = 0.0f;
    mmat[0][2] = 0.0f; mmat[1][2] = 0.0f; mmat[2][2] = 1.0f; mmat[3][2] = 0.0f;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;
    
    libVu0MulMatrix(thismat, thismat, mmat);
    }

/* 
 *	zshear -	
 *		shear z usinfg x and y.
 */
void colorMatrixRotateShearZ(float dx,float dy)
	{
	_colorMatrixRotateShearZ(mat,dx,dy);
 	}

void _colorMatrixRotateShearZ(libVu0FMatrix thismat,float dx,float dy)
	{
    libVu0FMatrix mmat;

    mmat[0][0] = 1.0f; mmat[1][0] = 0.0f; mmat[2][0] = 0.0f; mmat[3][0] = 0.0f;
    mmat[0][1] = 0.0f; mmat[1][1] = 1.0f; mmat[2][1] = 0.0f; mmat[3][1] = 0.0f;
    mmat[0][2] = dx;   mmat[1][2] = dy;   mmat[2][2] = 1.0f; mmat[3][2] = 0.0f;
    mmat[0][3] = 0.0f; mmat[1][3] = 0.0f; mmat[2][3] = 0.0f; mmat[3][3] = 1.0f;
    
    libVu0MulMatrix(thismat, thismat, mmat);
    }


/* 
 *	simplehuerotatemat -	
 *		simple hue rotation. This changes luminance 
 */
void colorMatrixRotateHueNOLUM(float rot)
	{
    float mag;
    float xrs, xrc;
    float yrs, yrc;
    float zrs, zrc;

    /* rotate the grey vector into positive Z */
        mag = sqrtf(2.0f);
        xrs = 1.0f/mag;
        xrc = 1.0f/mag;
        _colorMatrixRotateX(mat,xrs,xrc);
    
        mag = sqrtf(3.0f);
        yrs = -1.0f/mag;
        yrc = sqrtf(2.0f)/mag;
        _colorMatrixRotateY(mat,yrs,yrc);

    /* rotate the hue */
        zrs = sinf(rot*M_PI/180.0f);
        zrc = cosf(rot*M_PI/180.0f);
        _colorMatrixRotateZ(mat,zrs,zrc);

    /* rotate the grey vector back into place */
        _colorMatrixRotateY(mat,-yrs,yrc);
        _colorMatrixRotateX(mat,-xrs,xrc);
    }

/* 
 *	huerotatemat -	
 *		rotate the hue, while maintaining luminance.
 */
void colorMatrixRotateHueLUM(float rot)
	{
    libVu0FMatrix mmat;
    float mag;
    float lx, ly, lz;
    float xrs, xrc;
    float yrs, yrc;
    float zrs, zrc;
    float zsx, zsy;
    
    // por mi
    // libVu0FVector lv, RV={RLUM,GLUM,BLUM,1.0f};
    // libVu0FVector rot2={0.0f, 0.0f, 0.0f, 0.0f};

    libVu0UnitMatrix(mmat);

    /* rotate the grey vector into positive Z */
        mag = sqrtf(2.0f);
        xrs = 1.0f/mag;
        xrc = 1.0f/mag;
        _colorMatrixRotateX(mmat,xrs,xrc);
        mag = sqrtf(3.0f);
        yrs = -1.0f/mag;
        yrc = sqrtf(2.0f)/mag;
        _colorMatrixRotateY(mmat,yrs,yrc);

    /* shear the space to make the luminance plane horizontal */
    	xformpnt(mmat,RLUM,GLUM,BLUM,&lx,&ly,&lz);
    	
        //void  libVu0ApplyMatrix(libVu0FVector v0, libVu0FMatrix m, libVu0FVector v1);
        //libVu0ApplyMatrix(lv, (libVu0FMatrix *) mmat, RV);
        zsx = lx/lz;
        zsy = ly/lz;
        //zsx=lv[0]/lv[2];
        //zsy=lv[1]/lv[2];
    	_colorMatrixRotateShearZ(mmat,zsx,zsy);

    /* rotate the hue */
        zrs = sinf(rot*M_PI/180.0f);
        zrc = cosf(rot*M_PI/180.0f);
        _colorMatrixRotateZ(mmat,zrs,zrc);
        //void  libVu0RotMatrixZ(libVu0FMatrix m0, libVu0FMatrix m1, float rz);
        //libVu0RotMatrixZ(mmat, mmat, rot*M_PI/180.0f);

    /* unshear the space to put the luminance plane back */
    	_colorMatrixRotateShearZ(mmat,-zsx,-zsy);

    /* rotate the grey vector back into place */
        _colorMatrixRotateY(mmat,-yrs,yrc);
        _colorMatrixRotateX(mmat,-xrs,xrc);

    libVu0MulMatrix(mat, mat, mmat);
    }

/* 
 *	simplehuerotatemat -	
 *		simple hue rotation. This changes luminance 
 */
void colorMatrixRotateHue(float rot,int lum)
	{
	if(lum) 
 		 colorMatrixRotateHueLUM(rot);
	else colorMatrixRotateHueNOLUM(rot);
	}    


/*
main(argc,argv)
int argc;
char **argv;
{
    unsigned long *lbuf;
    int xsize, ysize;
    float mat[4][4];

    if(argc<2) {
	fprintf(stderr,"usage: matrix in.rgb\n");
	exit(1);
    }
    sizeofimage(argv[1],&xsize,&ysize);
    lbuf = (unsigned long*)longimagedata(argv[1]);
    prefsize(xsize,ysize);
    winopen("matrix color");
    RGBmode();
    gconfig();
    lrectwrite(0,0,xsize-1,ysize-1,lbuf);

    identmat(mat);

    offsetmat(mat,-50.0,-50.0,-50.0);	// offset color 
    cscalemat(mat,1.4,1.5,1.6);		// scale the colors 
    saturatemat(mat,1.7);		// saturate by 2.0 
    huerotatemat(mat,10.0);		// rotate the hue 10 
    printmat(mat);
    applymatrix(lbuf,mat,xsize*ysize);

    lrectwrite(0,0,xsize-1,ysize-1,lbuf);
    sleep(10);
    exit(0);
}
*/
