/*
 *	geometryMatrix
 *      Use 4x4 matricies to process color images.
 *      http://www.sgi.com/misc/grafica/matrix/index.html
 *
 *
 *      Paul Haeberli - 1993
 *      Please note that for these operations to be correct, we really must 
 *      operate on linear brightness values. If the input image is in a 
 *      non-linear brightness space RGB colors must be transformed into a 
 *      linear space before these matrix operations are used.
 *
 */

#include <libVu0.h>

#include "llibPlanar.h"
#include "math.h"
#include "vu0.h"


typedef struct
    {
    libVu0FMatrix mat;
    libVu0FVector *nodes;
    libVu0FVector *transform;

    int translateX;
    int translateY;

    int size;
    } geometryInfo;
    
geometryInfo geometryinfo;


    
void geometryCreate(libVu0FVector *nodes, int size)
    {
    geometryFree();
    
    geometryinfo.nodes = nodes;
    geometryinfo.size  = size;
    geometryinfo.transform = (libVu0FVector *)malloc(sizeof(libVu0FVector)*geometryinfo.size);
    }

void geometrySetSize(int size)
    {
    geometryinfo.size  = size;
    }

void geometryFree()
    {
    if(geometryinfo.transform != NULL)
        free(geometryinfo.transform);
    }


/*
 * Aplicamos la matriz
 */
void geometryMatrixVU0Apply()
	{
    int i;
	libVu0FVector v0,v1;

	for(i=0;i<geometryinfo.size;i++)
 		{
 		v1[0]=geometryinfo.nodes[i][0];
        v1[1]=geometryinfo.nodes[i][1];
        v1[2]=geometryinfo.nodes[i][2];
        v1[3]=geometryinfo.nodes[i][3];
        
        libVu0ApplyMatrix(v0, geometryinfo.mat, v1);

        geometryinfo.transform[i][0]=v0[0];
        geometryinfo.transform[i][1]=v0[1];
        geometryinfo.transform[i][2]=v0[2];
        geometryinfo.transform[i][3]=v0[3];
        }
    }

void geometryIdentity()
	{
    libVu0UnitMatrix(geometryinfo.mat);
    geometryinfo.translateX = 0;
    geometryinfo.translateY = 0;

	}


void geometryRotate(float x,float y,float z)
	{
    libVu0FVector rotate;
    
    rotate[0] = x;
    rotate[1] = y;
    rotate[2] = z;
    rotate[3] = 1.0f;

    // cuidado con pasarnos por encima
    while(rotate[0] > PI) rotate[0]  -= PI_2;
    while(rotate[1] > PI) rotate[1]  -= PI_2;
    while(rotate[2] > PI) rotate[2]  -= PI_2;
    
    while(rotate[0] < -PI) rotate[0] += PI_2;
    while(rotate[1] < -PI) rotate[1] += PI_2;
    while(rotate[2] < -PI) rotate[2] += PI_2;

    libVu0RotMatrix(geometryinfo.mat, geometryinfo.mat, rotate);
	}

void geometryRotateHex(u_char x,u_char y,u_char z)
    {
    geometryRotate(
                HexToRadians(x),
                HexToRadians(y),
                HexToRadians(z));
    }

void geometryTranslate(float x,float y,float z)
    {
    libVu0FVector translate;

    translate[0] = x;
    translate[1] = y;
    translate[2] = z;
    translate[3] = 1.0f;

    // aplicamos el translate a la matriz
    libVu0TransMatrix(geometryinfo.mat, geometryinfo.mat, translate);
    }

void geometryTranslate2D(int x,int y)
    {
    geometryinfo.translateX += x;
    geometryinfo.translateY += y;
    }


void geometryScale(float scalex,float scaley,float scalez)
	{
    libVu0FMatrix mmat;

    mmat[0][0]=scalex;mmat[1][0]=0.0f;  mmat[2][0]=0.0f;   mmat[3][0]=0.0f;
    mmat[0][1]=0.0f;  mmat[1][1]=scaley;mmat[2][1]=0.0f;   mmat[3][1]=0.0f;
    mmat[0][2]=0.0f;  mmat[1][2]=0.0f;  mmat[2][2]=scalez; mmat[3][2]=0.0f;
    mmat[0][3]=0.0f;  mmat[1][3]=0.0f;  mmat[2][3]=0.0f;   mmat[3][3]=1.0f;

    libVu0MulMatrix(geometryinfo.mat, geometryinfo.mat, mmat);
    }


// -----

void geometryDrawPolygon(int filled)
    {
    gfxDrawPolygon(
       geometryinfo.translateX, // X in screen
       geometryinfo.translateY, // Y in screen
       geometryinfo.transform,  // vertex
       geometryinfo.size,       // size
       filled);                 // filled
    }


