/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "llibPlanar.h"
#include "math.h"

typedef struct {
    int texture;
    int x[4],y[4];
    u_char colorRed;
    u_char colorGreen;
    u_char colorBlue;
    u_char colorAlpha;
    u_char rotate; // simple bool to know if is rotated
    short frame;
    
    llib2D_Pivot *pivot;
    } llib2D_Sprite;


llib2D_Sprite sprite;
int TemporalTextura;


void spriteReset()
    {
    spriteSetSize(0, 0);
    spriteSetColor(0x80,0x80,0x80);
    spriteSetAlpha(0x80);
    sprite.rotate = 0;
    sprite.frame = -1;
    sprite.pivot = NULL;
    }


void spriteSetPivot(llib2D_Pivot *pivotInfo)
    {
    int i;
    sprite.pivot = pivotInfo;
    
    for(i=0;i<sprite.pivot->pivots;i++)
        {
        sprite.pivot->fx[i] = sprite.pivot->ox[i];
        sprite.pivot->fy[i] = sprite.pivot->oy[i];
        }
    }


void spriteSetTexture(int indexTexture)
    {
    sprite.texture = indexTexture;
    }
void spriteSetFrame(int n)
    {
    sprite.frame = n;
    }
    
void spriteSetSize(int x, int y)
    {
    //   1 -----2
    //   |    / |
    //   |  /   |
    //   |/     |
    //   3------4
    //
    int midx = x/2;
    int midy = gfxNTSCy(y)/2;

    sprite.x[0] = -midx;  sprite.y[0] = -midy;
    sprite.x[1] = +midx;  sprite.y[1] = -midy;
    sprite.x[2] = -midx;  sprite.y[2] = +midy;
    sprite.x[3] = +midx;  sprite.y[3] = +midy;
    
    }
    
void spriteTranslate(int x,int y)
    {
    int i;
    y=gfxNTSCy(y);
    // Coordenadas del objeto
    for(i=0;i<4;i++)
        {
        sprite.x[i] += x;
        sprite.y[i] += y;
        }

    // Coordenadas del pivote
    if(sprite.pivot != NULL)
    for(i=0;i<sprite.pivot->pivots;i++)
        {
        sprite.pivot->fx[i] += x;
        sprite.pivot->fy[i] += y;
        }
    }

void spriteScale(float x,float y)
    {
    int i;
    y=gfxNTSCy(y);
    // Coordenadas del objeto
    for(i=0;i<4;i++)
        {
        sprite.x[i] *= x;
        sprite.y[i] *= y;
        }

    // Coordenadas del pivote
    if(sprite.pivot != NULL)
    for(i=0;i<sprite.pivot->pivots;i++)
        {
        sprite.pivot->fx[i] *= x;
        sprite.pivot->fy[i] *= y;
        }
        
    }


void spriteRotateHex(u_char z)
    {
    spriteRotate(HexToRadians(z));
    }

void spriteRotate(float z)
    {
    int i;
    int tempx,tempy;

    float seno   = sinf(z);
    float coseno = cosf(z);

    // Coordenadas del objeto
    for(i=0;i<4;i++)
        {
        tempx = sprite.y[i] * seno   + sprite.x[i] * coseno;
        tempy = sprite.y[i] * coseno - sprite.x[i] * seno;
        sprite.x[i] = tempx;
        sprite.y[i] = tempy;
        }

    // Coordenadas del pivote
    if(sprite.pivot != NULL)
    for(i=0;i<sprite.pivot->pivots;i++)
        {
        tempx = sprite.pivot->fy[i] * seno   + sprite.pivot->fx[i] * coseno;
        tempy = sprite.pivot->fy[i] * coseno - sprite.pivot->fx[i] * seno;
        sprite.pivot->fx[i] = tempx;
        sprite.pivot->fy[i] = tempy;
        }
    sprite.rotate = 1;
    }

void spriteSetColor(u_char r,u_char g,u_char b)
    {
    sprite.colorRed   = r;
    sprite.colorGreen = g;
    sprite.colorBlue  = b;
    }

void spriteSetAlpha(u_char a)
    {
    sprite.colorAlpha = a;
    }


void spriteRenderClut(int mode)
    {
    int i;
    
    // sacamos la informacion de la textura para enviar los ST al GS
    llibtex_Texture texture = textureGetBox(sprite.texture);

    // le decimos que queremos usar la textura 
    if(mode==0)
         textureUseFrame(sprite.texture,sprite.frame); // Texture
    else textureUseFrame(sprite.texture-1,0);          // Clut
    
    if(sprite.rotate)
         addPRIM(LIB_PRIM_TSTRIP, 0, 1, 0, 1, 0, 0, 0, 0);
    else addPRIM(LIB_PRIM_SPRITE, 0, 1, 0, 1, 0, 0, 0, 0);
	

    // Tiramos el Color + Alpha + Q (trabajamos en 2D siempre q=1.0f)
    addRGBAQ(sprite.colorRed, 
             sprite.colorGreen, 
             sprite.colorBlue, 
             sprite.colorAlpha, 1.0f);

    if(sprite.rotate)
        for(i=0;i<4;i++)
            {
            // Coordenadas ST para la textura
            addST(texture.S[i],texture.T[i]);

            // Coordenadas XYZ del Sprite
            addXYZ2( GSCENTERX + sprite.x[i],
                     GSCENTERY + sprite.y[i],
                     GSLayerZ);
            }
    else
        for(i=0;i<4;i+=3)
            {
            // Coordenadas ST para la textura
            addST(texture.S[i],texture.T[i]);

            // Coordenadas XYZ del Sprite
            addXYZ2( GSCENTERX + sprite.x[i],
                     GSCENTERY + sprite.y[i],
                     GSLayerZ);
            }

    GSLayerZ++;
    }

void spriteRender()
    {
    spriteRenderClut(0);
    }
void spriteRenderPal()
    {
    spriteRenderClut(1);
    }

//

llib2D_Pivot *spritePivotCreate(int n)
    {
    llib2D_Pivot *pivot;

    pivot = (llib2D_Pivot *)malloc(sizeof(llib2D_Pivot));

    pivot->pivots = n;
    pivot->last   = 0;
    pivot->ox = (int *)malloc(n * sizeof(int));
    pivot->oy = (int *)malloc(n * sizeof(int));
    pivot->fx = (int *)malloc(n * sizeof(int));
    pivot->fy = (int *)malloc(n * sizeof(int));
    return pivot;    
    }

int spriteAddPivot(llib2D_Pivot *pivot,int x,int y)
    {
    int n;
    n = pivot->last;
    pivot->ox[n] = x;
    pivot->oy[n] = y;
    pivot->last++;
    return n;
    }
    
int spriteGetPivotX(llib2D_Pivot *pivot,int i)
    {
    return pivot->fx[i];
    }
int spriteGetPivotY(llib2D_Pivot *pivot,int i)
    {
    return pivot->fy[i];
    }


