/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "llibPlanar.h"

#include <stdio.h>

#define MAXSTRINGSIZE 8192
#define MAXCOLSIZE    0xF
#define MAXCONSOLES   8


typedef struct
    {
    int use;

    int x,y;    // posicion de la consola
    int w,h;    // Tamao
    int font;   // fuente
    int charSizeX, charSizeY;
    int cursorX;

    // texto
    char text[MAXSTRINGSIZE];
    int linesCon, lines;

    // posicion visible
    float factorView;
    int lineView;
    
    // colores
    gfxRGBA col[MAXCOLSIZE];
    } llib2DConsole_Data;

llib2DConsole_Data *consoles = NULL;
llib2DConsole_Data *conActive = NULL;


void consoleActive(int n)
    {
    conActive = &consoles[n];
    }

int consoleCreate()
    {
    int i;

    // miramos si hay que dar memoria
    if(consoles == NULL)
        {
        consoles = (llib2DConsole_Data *)malloc(sizeof(llib2DConsole_Data)* MAXCONSOLES);
        for(i=0;i<MAXCONSOLES;i++)
            {
            consoles[i].use=0;
            }
        }


    // Buscamos una consola libre
    for(i=0;i<MAXCONSOLES;i++)
        {
        if(consoles[i].use==0)
            {
            consoleActive(i);
            consoles[i].use=1;
            return i;
            }
        }

    // No quedan consolas libres
    return -1;
    }
    

void consoleSet(int x,int y,int w,int h,int n)
    {
    consoleSetPosition(x,y);
    consoleSetSize(w,h);
    consoleSetFont(n);
    
    consoleSetColor(0x80,0x80,0x80);
    consoleSetAlpha(0x80);
    }
    
void consoleSetPosition(int x,int y)
    {
    conActive->x = x;
    conActive->y = y;
    }
    
void consoleSetSize(int w,int h)
    {
    conActive->w = w;
    conActive->h = h;
    }

int consoleGetX() { return conActive->x; }
int consoleGetY() { return conActive->y; }
int consoleGetW() { return conActive->w; }
int consoleGetH() { return conActive->h; }


void consoleSetFont(int n)
    {
    // Activate idFont
    conActive->font = n;

    // Font Metrics
    conActive->charSizeX = textGetWidthForFont(conActive->font, " ");
    conActive->charSizeY = textGetHeightForFont(conActive->font);

    // miramos cuantas lineas nos caben en la consola
    conActive->linesCon = (int)(conActive->h / conActive->charSizeY);

    // al cambiar de font limpiamos la consola
    // esto reinicializa el cursor
    consoleClear();
    }


void consoleSetColorId(int id, int r,int g,int b,int a)
    {
    if(id>=0 && id<MAXCOLSIZE)
        {
        conActive->col[id].r = r;
        conActive->col[id].g = g;
        conActive->col[id].b = b;
        conActive->col[id].a = a;
        }
    }

void consoleSetColor(int r,int g,int b)
    {
    conActive->col[0].r = r;
    conActive->col[0].g = g;
    conActive->col[0].b = b;
    }

void consoleSetAlpha(int a)
    {
    conActive->col[0].a = a;
    }
    


void consoleEnable(int flag)
    {
    // EatSpacesAt
    // Colors
    }
void consoleDisable(int flag)
    {
    }

void consoleClear()
    {
    conActive->text[0] = '\0';    // Clear
    conActive->cursorX = 0;       // Reset cursor point
    }

void consolePrint(char *line)
    {
    int n = myStringLen(&conActive->text[0]);
    int i = 0;
    
    // metemos el String a partir de esta posicion
    while(line[i]!='\0')
        {
        // si es un retorno reiniciamos el cursor
        switch(line[i])
            {
            case '\n':
                conActive->cursorX = 0;
                break;
            case '\\':
                // La proxima vuelta vamos a sumar un espacio
                // por el char del color, asi que lo restamos ahora para compensar
                conActive->cursorX -= conActive->charSizeX;
                break;
            default:
                // vamos incrementando la posicion X hasta que no
                // quepa en la consola, entonces forzamos un \n
                conActive->cursorX += conActive->charSizeX;
                if (conActive->cursorX >= (conActive->w-conActive->charSizeX))
                    {
                    conActive->cursorX = 0;
                    conActive->text[n] = '\n';
                    n++;
                    }
            }

        // Una vez filtradas posiciones, metemos el char
        conActive->text[n] = line[i];
        n++;i++;
        }
    conActive->text[n] = '\0';
    
    // Contamos las lineas que tiene el texto
    conActive->lines = 1;

    for(i=0;i<myStringLen(&conActive->text[0]);i++)
        {
        if(conActive->text[i]=='\n')
            conActive->lines++;
        }
    }

void consolePrintln(char *line)
    {
    consolePrint(line);
    consolePrint("\n");
    }

void consolePrintf(const char *format, ...)
    {
    va_list	opt;
    unsigned char buff[2048];

    va_start(opt, format);
    vsnprintf( buff, sizeof(buff), format, opt);
    
    consolePrint(buff);
    }

void consoleSetText(char *text)
    {
    consoleClear();
    consolePrint(text);
    }

int consoleGetSize()
    {
	return 0;
    }

void consoleSetView(float factor)
    {
    // 0.0f - 1.0f
    conActive->factorView = factor;
    conActive->lineView = (conActive->linesCon - conActive->lines) *
                            conActive->charSizeY *
                            conActive->factorView;
    }

int consoleGetLines()
    {
    return conActive->lines;
    }
int consoleGetLinesView()
    {
    return conActive->linesCon;
    }

void consoleRender()
    {
    int i;
    int px=0,py=0;

    gfxClearPrimTag();

    // gfxPushScissor();

    gfxSetScissor(conActive->x,conActive->y,
                  conActive->x + conActive->w,
                  conActive->y + conActive->h);

    // debug
    //gfxSetColor(0x20,0x20,0x00,0x50);
    //gfxDrawBox(conActive->x,conActive->y,
    //           conActive->x + conActive->w,
    //           conActive->y + conActive->h);
    // -------------------

    textSetFontActive(conActive->font);
    textPrintPrepareTexture();
    textPrintPrepareColor(conActive->col[0].r,conActive->col[0].g,conActive->col[0].b);
    textPrintPrepareAlpha(conActive->col[0].a);

    int size = myStringLen(&conActive->text[0]);
    for(i=0;i<size;i++)
        {
        unsigned char c = conActive->text[i];

        switch(c)
            {
            case '\n':
                        px = 0;
                        py+=conActive->charSizeY;
                        break;
            case '\\':
                        i++;
                        unsigned char color = conActive->text[i] - '0';
                        textPrintPrepareColor(conActive->col[color].r,
                                              conActive->col[color].g,
                                              conActive->col[color].b);
                        textPrintPrepareAlpha(conActive->col[color].a);
                        break;
            default:
                        textPrintPreparedChar(conActive->x + px,
                                              conActive->y - conActive->lineView + py ,
                                              c);
                        px+=conActive->charSizeX;
            }
        }
    // gfxPopScissor();
    gfxClearScissor();
    }
    
void consoleRenderHost()
    {
    printf(conActive->text);
    }

