/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "llibPlanar.h"

#define MAXCHARSETS 8

typedef struct
    {
    unsigned int texture;      // textura de la fuente

    unsigned int charSizeX;    // Tamao en pixels del caracter
    unsigned int charSizeY;
    unsigned int numFonts;     // numero de fuentes
    unsigned int fontActive;   // fuente activa

    int padding[MAXCHARSETS]; 
    float factor[MAXCHARSETS];
    
    u_char colorRed; 
    u_char colorGreen;
    u_char colorBlue;
    u_char colorAlpha;
    
    float scaleX;
    float scaleY;
    } llib2DText_Info;

llib2DText_Info textInfo;


void textSetFont(unsigned int n, int nFonts, int charSizeX, int charSizeY)
    {
    int i;

    // no podemos tener mas de 8 juegos de caracteres
    if(nFonts > MAXCHARSETS)
        {
        printf("[llib2DText] error, Only %d charset\n", MAXCHARSETS);
        nFonts = MAXCHARSETS;
        }
    
    // guardamos la info
    textInfo.texture    = n;
    textInfo.numFonts   = nFonts;
    textInfo.charSizeX  = charSizeX;
    textInfo.charSizeY  = charSizeY;

    // Preparamos el array para los paddings
    for(i=0;i<MAXCHARSETS;i++)
        {
        textSetPadding(i,textInfo.charSizeX * 0.8f);
        textSetSize(i, 1.0f);
        }    

    textSetFontActive(0);
    textSetColor(0xFF,0xFF,0xFF);
    textSetAlpha(0x80);
    textPrintScaleChar(1.0f,1.0f);
    }

void textSetPadding(int index,int padding)
    {
    if(index < MAXCHARSETS)
        textInfo.padding[index] = padding;
    }

void textSetFontActive(int index)
    {
    if(index < MAXCHARSETS)
        textInfo.fontActive = index;
    }

void textSetSize(int index, float factor)
    {
    if(index < MAXCHARSETS)
        textInfo.factor[index] = factor;
    }

void textPrintScaleChar(float x,float y)
    {
    textInfo.scaleX = x;
    textInfo.scaleY = y;
    }


void textSetColor(u_char r,u_char g,u_char b)
    {
    textInfo.colorRed   = r; 
    textInfo.colorGreen = g;
    textInfo.colorBlue  = b;
    }
        
void textSetAlpha(u_char alpha)
    {
    textInfo.colorAlpha = alpha;
    }    

int myStringLen(char* str)
	{
	char *char_ptr;
	char_ptr = str;

	while(1)
		{
		if (*char_ptr == '\0') return (char_ptr - str);
		char_ptr++;
		}
	}

int textGetHeightForFont(int n)
    {
    return (textInfo.charSizeY * textInfo.factor[n]);
    }
int textGetHeight()
    {
    return textGetHeightForFont(textInfo.fontActive);
    }

int textGetWidthForFont(int n, char *str)
    {
	int len = myStringLen(str);
    len *= (textInfo.padding[n] * textInfo.factor[n]);
    return len;
    }

int textGetWidth(char *str)
    {
    return textGetWidthForFont(textInfo.fontActive, str);
    }

void textPrintPrepareTexture()
    {
	textureUse(textInfo.texture);
    addPRIM(LIB_PRIM_SPRITE,         // PRIM (Primitive type)
                0,                   // IIP  (Gouraud)
                1,                   // TME  (Textured)
                0,                   // FGE  (Fogging)
                1,                   // ABE  (Alpha Blending)
                0,                   // AA1  (Anti-Aliasing)
                1,                   // FST  (Use ST for texture coords)
                0,                   // CTXT (Context)
                0);                  // FIX  (Fragment control)
    }

void textPrintPrepareColor(u_char r,u_char g,u_char b)
    {
    textSetColor(r,g,b);
    addRGBAQ(textInfo.colorRed,
             textInfo.colorGreen,
             textInfo.colorBlue,
             textInfo.colorAlpha, 1.0f);
    }

void textPrintPrepareAlpha(u_char alpha)
    {
    textSetAlpha(alpha);
    addRGBAQ(textInfo.colorRed,
             textInfo.colorGreen,
             textInfo.colorBlue,
             textInfo.colorAlpha, 1.0f);
    }

void textPrintChar(int x,int y,char chr)
    {
	textureUse(textInfo.texture);

    // Metemos el SPRITE
    addPRIM(LIB_PRIM_SPRITE,         // PRIM (Primitive type)
                0,                   // IIP  (Gouraud)
                1,                   // TME  (Textured)
                0,                   // FGE  (Fogging)
                1,                   // ABE  (Alpha Blending)
                0,                   // AA1  (Anti-Aliasing)
                1,                   // FST  (Use ST for texture coords)
                0,                   // CTXT (Context)
                0);                  // FIX  (Fragment control)

    // Tiramos el Color + Alpha + Q (trabajamos en 2D siempre q=1.0f)
    // tomamos estos valores de CFG
    addRGBAQ(textInfo.colorRed,
             textInfo.colorGreen,
             textInfo.colorBlue,
             textInfo.colorAlpha, 1.0f);
			 
    textPrintPreparedChar(x,y,chr);
    }

void textPrintPreparedChar(int x,int y,unsigned char chr)
    {
	unsigned int textX;
	unsigned int textY;

	// calculamos el offset de la escala
	int offsetx = 0;
	int offsety = 0;
	offsetx = textInfo.charSizeX - (textInfo.charSizeX * textInfo.scaleX);
	offsety = textInfo.charSizeY - (textInfo.charSizeY * textInfo.scaleY);
	// ---
	
	if (chr==191){ chr=129;} //
	
	if (chr==241){ chr=134;} //
	if (chr==209){ chr=135;} //
		
	chr-= 32;
		
	if ((chr>0 && chr<128))
        {
        textX  = (chr % textInfo.charSizeX)*textInfo.charSizeX;
        textY  = (chr / textInfo.charSizeY)*textInfo.charSizeY;
        textY += (textInfo.fontActive * (textInfo.charSizeY * 8));

        addUV(textX,textY);
        addXYZ2(GSCENTERX + x - offsetx,
                GSCENTERY + gfxNTSCy(y) - offsety, GSLayerZ);

        addUV(textX+textInfo.charSizeX,textY+textInfo.charSizeY);
        addXYZ2(GSCENTERX + x + offsetx + (textInfo.charSizeX * textInfo.factor[textInfo.fontActive]),
                GSCENTERY + gfxNTSCy(y) + offsetx + (textInfo.charSizeY * textInfo.factor[textInfo.fontActive]), GSLayerZ);

        GSLayerZ++;
        }
    }



void textPrintXY(int x, int y, char *str)
	{
	int len;
	unsigned int i;

	len = myStringLen(str);

	textPrintPrepareTexture();
	textPrintPrepareAlpha(textInfo.colorAlpha);

	for(i=0;i<len;i++)
	    {
	    textPrintPreparedChar(x,y,str[i]);
        x += (textInfo.padding[textInfo.fontActive] * textInfo.factor[textInfo.fontActive]);
		}
	}

void textPrintJustify(int x1, int x2, int y, char *str)
    {
    int i;
    int sizeText = textGetWidth(str); // tamao en pixels del texto
    int sizeBox  = x2-x1;             // tamao en pixels del bloque
    int len = myStringLen(str);       // caracteres

    float inc = (float)(sizeBox-sizeText) / (float)len; // incremento de cada carater
    float x = x1;

	textPrintPrepareTexture();
	textPrintPrepareAlpha(textInfo.colorAlpha);

	for(i=0;i<len;i++)
	    {
	    textPrintPreparedChar(x,y,str[i]);
        x += (textInfo.padding[textInfo.fontActive] * textInfo.factor[textInfo.fontActive]);
        x += inc;
		}
    }

void textPrintCenter(int y, char *str)
    {
    textPrintXY(((textGetWidth(" ")+textGetWidth(str)) / -2), y, str);
    }
    
