/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include <libVu0.h>
#include <math.h>

#include "llibPlanar.h"
#include "math.h"
#include "vu0.h"

//#define MyMalloc(x) memalign(64,x)
#define MyMalloc(x) malloc(x)

void printMatrix(libVu0FMatrix *m){
    int i;
    for(i=0;i<4;i++)
        printf(" %d \t %d \t %d \t %d \n",
                (int)(m[0][i][0] * 100),
                (int)(m[0][i][1] * 100),
                (int)(m[0][i][2] * 100),
                (int)(m[0][i][3] * 100));
    }

void makefashion(Object3D *obj);

void objectRender_Wireframe(Object3D *obj);
void objectRender_Point(Object3D *obj);
void objectRender_Flat(Object3D *obj);
void objectRender_Goraud(Object3D *obj);
void objectRender_Goraud3(Object3D *obj);
void objectRender_Solid(Object3D *obj); // <- Solido sin luz

void objectRender_EnvironmentMap(Object3D *obj);

void object_ProyectaVertices(Object3D *obj);
void object_ProyectaNormalesCara(Object3D *obj);
void object_ProyectaNormalesVertices(Object3D *obj);
void object_ProyectaNormalesVerticesSingle(Object3D *obj);

void objectClearLight(Object3D *obj);
void objectSetLight_Flat(Object3D *obj,int light);
void objectSetLight_Goraud(Object3D *obj,int light);
void objectSetLight_Goraud3(Object3D *obj,int light);

void Render_AplicaMatriz(libVu0FVector *VerticesOriginales,
                         libVu0FVector *VerticesProyectados,
                         libVu0FMatrix matriz, int nPuntos);
void Render_AplicaMatriz1(libVu0FVector VerticesOriginales,
                         libVu0FVector VerticesProyectados,
                         libVu0FMatrix matriz);
						 
libVu0FVector cazd  = {0.0f,0.0f,1.0f,1.0f};     // zd ojo
libVu0FVector cayd  = {0.0f,1.0f,0.0f,1.0f};    // yd vertical

Object3D *actualobj;
int camrotated=0;
int cammoved=0;

ObjectCamera *cameraCreate(){
    return cameraCreateZRange(0,0x00FF, 0xFF00);
    }

ObjectCamera *cameraCreateZ(int factorZ){
    return cameraCreateZRange(factorZ,0x00FF, 0xFF00);
    }

ObjectCamera *cameraCreateForLayer(int factorZ,int layer){
    return cameraCreateZRange(factorZ,layerZGetMin(layer),layerZGetMax(layer));
    }

void camRotateHex(ObjectCamera *obj,u_char x,u_char y,u_char z){
    camRotate(obj,
                HexToRadians(x),
                HexToRadians(y),
                HexToRadians(z)
                );
    }
		
void camRotate(ObjectCamera *cam, float x, float y, float z){

	libVu0UnitMatrix(tmp);
	
    // cuidado con pasarnos por encima
    while(x > PI) x -= PI_2;
    while(y > PI) y -= PI_2;
    while(z > PI) z -= PI_2;

    // y por debajo
    while(x < -PI) x += PI_2;
    while(y < -PI) y += PI_2;
    while(z < -PI) z += PI_2;

	cam->rotacion[0]=x;
	cam->rotacion[1]=y;
	cam->rotacion[2]=z;
	cam->rotacion[3]=1.0f;
	
	libVu0RotMatrixX(tmp,tmp,-cam->rotacion[0]);	
	libVu0RotMatrixY(tmp,tmp,-cam->rotacion[1]);	
	libVu0RotMatrixZ(tmp,tmp,-cam->rotacion[2]);	
	
	libVu0ApplyMatrix(zd, tmp, cazd);
    libVu0ApplyMatrix(yd, tmp, cayd);		
	
	camrotated=1;

}

void camTurn(ObjectCamera *cam, float x, float y, float z){
	

	libVu0UnitMatrix(tmp);

    // cuidado con pasarnos por encima
	cam->rotacion[0]=cam->rotacion[0]+x;
	cam->rotacion[1]=cam->rotacion[1]+y;
	cam->rotacion[2]=cam->rotacion[2]+z;
	cam->rotacion[3]=1.0f;

    while(cam->rotacion[0] > PI) cam->rotacion[0] -= PI_2;
    while(cam->rotacion[1] > PI) cam->rotacion[1] -= PI_2;
    while(cam->rotacion[2] > PI) cam->rotacion[2] -= PI_2;

    // y por debajo
    while(cam->rotacion[0] < -PI) cam->rotacion[0] += PI_2;
    while(cam->rotacion[1] < -PI) cam->rotacion[1] += PI_2;
    while(cam->rotacion[2] < -PI) cam->rotacion[2] += PI_2;

	libVu0RotMatrixX(tmp,tmp,-cam->rotacion[0]);	
	libVu0RotMatrixY(tmp,tmp,-cam->rotacion[1]);	
	libVu0RotMatrixZ(tmp,tmp,-cam->rotacion[2]);	
	
	libVu0ApplyMatrix(zd, tmp, cazd);
    libVu0ApplyMatrix(yd, tmp, cayd);		
	
	camrotated=1;

}

void camPoint(ObjectCamera *cam, float x, float y, float z){

		float ax=cam->posicion[0];
		float ay=cam->posicion[1];
		float az=cam->posicion[2];
		
		float diffx=cam->posicion[0]-x;
		float diffy=cam->posicion[1]-y;
		float diffz=cam->posicion[2]-z;
				
		camPosition(cam,0,0,0);
		
		cam->rotacion[1]=-(diffx/diffz);
		cam->rotacion[0]=(diffy/diffz);
				
		camPosition(cam,ax,ay,az);
		
}

void camPosition(ObjectCamera *cam, float x, float y, float z){

	libVu0UnitMatrix(tmp);

	cam->posicion[0]=x;
	cam->posicion[1]=y;
	cam->posicion[2]=z;
	cam->posicion[3]=1.0f;
	
	libVu0FVector cap  = {cam->posicion[0],-cam->posicion[1],cam->posicion[2],1.0f};     // zd ojo
    
	libVu0ApplyMatrix(p, tmp, cap);

	cammoved=1;
}

void camMove(ObjectCamera *cam, float x, float y, float z){

	libVu0UnitMatrix(tmp);

	libVu0FMatrix r;

	libVu0FVector cap  = {cam->posicion[0],-cam->posicion[1],cam->posicion[2],1.0f};     // posicion de la camara
	libVu0FVector mov  = {x,y,z,1.0f};     // movimiento
	
	libVu0UnitMatrix(r);
	
	libVu0RotMatrixX(r,r,-cam->rotacion[0]);	
	libVu0RotMatrixY(r,r,-cam->rotacion[1]);	
	libVu0RotMatrixZ(r,r,-cam->rotacion[2]);
		
	libVu0ApplyMatrix(mov, r, mov);
	
	libVu0TransMatrix(tmp,tmp,mov);	

	libVu0ApplyMatrix(p, tmp, cap);
	
	cam->posicion[0]=p[0];
	cam->posicion[1]=-p[1];
	cam->posicion[2]=p[2];
	cam->posicion[3]=1.0f;
			
	cammoved=1;
}

void camRender(ObjectCamera *cam){
	if(camrotated==0){
		camRotate(cam,cam->rotacion[0],cam->rotacion[1],cam->rotacion[2]);
	}
	if(cammoved==0){
		camPosition(cam,cam->posicion[0],cam->posicion[1],cam->posicion[2]);
	}
    libVu0CameraMatrix(cam->worldToCamera[0], p, zd, yd);
	camrotated=0;
	cammoved=0;
}

ObjectCamera *cameraCreateZRange(int factorZ,unsigned int minz, unsigned int maxz){
    ObjectCamera *camera;
	
    // damos memoria
    camera = (ObjectCamera *)malloc(sizeof(ObjectCamera));
    camera->cameraToScreen = (libVu0FMatrix *)memalign(64,1*sizeof(libVu0FMatrix));
    camera->worldToCamera  = (libVu0FMatrix *)memalign(64,1*sizeof(libVu0FMatrix));
	camera->minz=minz;
	camera->maxz=maxz;
	
    // Creamos Screen matrix //
    libVu0ViewScreenMatrix(camera->cameraToScreen[0],
        512.0f,
    	1.0f,1.0f,                          // factor de escalado x,y
    	GSCENTERX * 1.0f ,GSCENTERY * 1.0f, // centro de la pantalla x,y
		//5.0f, 16777000.0f,
        minz * 1.0f, maxz * 1.0f,           // zmin, zmax
        16.0f, 65536.0f );                  // nearz, farz

	libVu0UnitMatrix(tmp);
	camRotate(camera,0,0,0);
	camPosition(camera,0,0,factorZ);

    cameraSetActive(camera);
    return camera;
    }


void cameraFree(ObjectCamera *camera){
    if (actualCamera == camera) actualCamera = NULL;
    free(camera);
    }

void cameraSetActive(ObjectCamera *camera){
    actualCamera = camera;
    }
	
/* Model3DM *LoadModel3DM(char * filename)
 *    Cargamos un modelo 3DM
 */

Model3DM *LoadModel3DM(char * filename){
    Model3DM *obj;
    SMF_Header *ObjHeader;
    int fd;

    obj       = (Model3DM *)  MyMalloc(sizeof(Model3DM));
    ObjHeader = (SMF_Header *)MyMalloc(sizeof(SMF_Header));

    fd = openFile(filename, O_RDONLY);
    readFile(fd, ObjHeader, sizeof(SMF_Header));
	
    if (ObjHeader->id[0] == 'S' && 
        ObjHeader->id[1] == 'M' &&
        ObjHeader->id[2] == 'F')
        {
        /*
        printf("Leyendo objeto : %s version(%c) \n",filename,ObjHeader->id[3]);
        printf("Content : [VFNEST] \n");
        printf("          [");
        printf("%d", ObjHeader->content[SMF_V] );
        printf("%d", ObjHeader->content[SMF_F] );
        printf("%d", ObjHeader->content[SMF_N] );
        printf("%d", ObjHeader->content[SMF_E] );
        printf("%d", ObjHeader->content[SMF_S] );
        printf("%d", ObjHeader->content[SMF_T] );
        printf("] \n");
        printf(" NUMVERTEX   = %d \n",ObjHeader->NUMVERTEX);
        printf(" NUMFACES    = %d \n",ObjHeader->NUMFACES);
        printf(" NUMTVERTEX  = %d \n",ObjHeader->NUMTVERTEX);
        printf(" NUMTVFACES  = %d \n",ObjHeader->NUMTVFACES);
        */
        }
    else
        {
        printf("Error, el archivo no es un SMF : %s\n",filename);
        free(obj);
        free(ObjHeader);        // liberamos el header
      	closeFile(fd);       // cerramos el archivo
        return NULL;            // y devolvemos un error
        }

    obj->NUMVERTEX  = ObjHeader->NUMVERTEX;
    obj->NUMFACES   = ObjHeader->NUMFACES;
    obj->NUMTVERTEX = ObjHeader->NUMTVERTEX;
    obj->NUMTVFACES = ObjHeader->NUMTVFACES;

    // --
    obj->content[SMF_V] = ObjHeader->content[SMF_V];
    obj->content[SMF_F] = ObjHeader->content[SMF_F];
    obj->content[SMF_N] = ObjHeader->content[SMF_N];
    obj->content[SMF_E] = ObjHeader->content[SMF_E];
    obj->content[SMF_S] = ObjHeader->content[SMF_S];
    obj->content[SMF_T] = ObjHeader->content[SMF_T];

    // cargamos los vertices
    if(ObjHeader->content[SMF_V] == 1)
        {
        obj->vertices = (libVu0FVector *)MyMalloc(ObjHeader->NUMVERTEX * sizeof(libVu0FVector));
        readFile(fd, obj->vertices, sizeof(libVu0FVector) * ObjHeader->NUMVERTEX);
        }


    // Cargamos las caras
    if(ObjHeader->content[SMF_F] == 1)
        {
        obj->faces = (int *)MyMalloc(ObjHeader->NUMFACES * 3 * sizeof(int));
        readFile(fd, obj->faces, sizeof(int)* ObjHeader->NUMFACES * 3);
        }


    // las normales de las caras
    if(ObjHeader->content[SMF_N] == 1)
        {
        obj->normalfaces = (libVu0FVector *)MyMalloc(ObjHeader->NUMFACES * sizeof(libVu0FVector));
        readFile(fd, obj->normalfaces, sizeof(libVu0FVector) * ObjHeader->NUMFACES);
        }

    // las normales de los vertices
    if(ObjHeader->content[SMF_E] == 1)
        {
        obj->normalvertex = (libVu0FVector *)MyMalloc(ObjHeader->NUMFACES * 3 * sizeof(libVu0FVector));
        readFile(fd, obj->normalvertex, ObjHeader->NUMFACES * 3 * sizeof(libVu0FVector));
        }

    // las normales de los vertices Unificados
    if(ObjHeader->content[SMF_S] == 1)
        {
        obj->normalSvertex = (libVu0FVector *)MyMalloc(ObjHeader->NUMVERTEX * sizeof(libVu0FVector));
        readFile(fd, obj->normalSvertex, ObjHeader->NUMVERTEX * sizeof(libVu0FVector));
        }

    // UV Texture !!
    if(ObjHeader->content[SMF_T] == 1)
        {
        obj->tvertices = (libVu0FVector *)MyMalloc(ObjHeader->NUMTVERTEX * sizeof(libVu0FVector));
        readFile(fd, obj->tvertices, sizeof(libVu0FVector)* ObjHeader->NUMTVERTEX);

        obj->Tfaces = (int *)MyMalloc(ObjHeader->NUMTVFACES * 3 * sizeof(int));
        readFile(fd, obj->Tfaces, sizeof(int) * ObjHeader->NUMTVFACES * 3);
        }    

    free(ObjHeader);
	closeFile(fd);

	return obj;
    }

void model3DMFree(Model3DM *obj){
    free(obj->vertices);
    free(obj->faces);
    free(obj->normalfaces);
    free(obj->normalvertex);
    free(obj->normalSvertex);
    free(obj->tvertices);
    free(obj->Tfaces);
    }

void model3DMScale(Model3DM *obj,float x,float y,float z){
    int i;
    
    for(i=0;i<obj->NUMVERTEX;i++)
        {
        obj->vertices[i][0] *= x;
        obj->vertices[i][1] *= y;
        obj->vertices[i][2] *= z;
        }
    }
    
void model3DMScaleTo(Model3DM *obj,float x,float y,float z){
    float minx=0.0f,miny=0.0f,minz=0.0f;
    float maxx=0.0f,maxy=0.0f,maxz=0.0f;
    int i;
    float fx,fy,fz;

    // Buscamos las coordenadas maximas y minimas del modelo
    for(i=0;i<obj->NUMVERTEX;i++)
        {
        // min bounding
        if (minx>obj->vertices[i][0]) minx=obj->vertices[i][0];
        if (miny>obj->vertices[i][1]) miny=obj->vertices[i][1];
        if (minz>obj->vertices[i][2]) minz=obj->vertices[i][2];

        // max bounding
        if (maxx<obj->vertices[i][0]) maxx=obj->vertices[i][0];
        if (maxy<obj->vertices[i][1]) maxy=obj->vertices[i][1];
        if (maxz<obj->vertices[i][2]) maxz=obj->vertices[i][2];
        }
    
    // Calculamos Factor de escalado
    if((maxx-minx)==0) fx=1.0f; else fx=(float)x /(float)(maxx-minx);
    if((maxy-miny)==0) fy=1.0f; else fy=(float)y /(float)(maxy-miny);
    if((maxz-minz)==0) fz=1.0f; else fz=(float)z /(float)(maxz-minz);
    
    // scalamos el modelo por ese factor
    model3DMScale(obj,fx,fy,fz);
    }

int model3DMHasKey(Model3DM *obj, int key){
	if(key & RENDER_KEY_GEOM)
		if (!(obj->content[SMF_V] && obj->content[SMF_F])) return 0;

	if(key & RENDER_KEY_PLANE)
		if (!obj->content[SMF_N]) return 0;

	if(key & RENDER_KEY_VERTEX)
		if (!obj->content[SMF_S]) return 0;

	if(key & RENDER_KEY_ST)
		if (!obj->content[SMF_T]) return 0;

    return 1;
	}


int model3DMHasRender(Model3DM *obj, int renderMode){
	if(renderMode & RENDER_POINT) 
 		if (!model3DMHasKey(obj, RENDER_KEY_GEOM)) return 0;

	if(renderMode & RENDER_WIREFRAME) 
 		if (!model3DMHasKey(obj, RENDER_KEY_GEOM)) return 0;

	if(renderMode & RENDER_FLAT)
		if (!(model3DMHasKey(obj, RENDER_KEY_GEOM | RENDER_KEY_PLANE))) return  0;

	if(renderMode & RENDER_GORAUD)
		if (!(model3DMHasKey(obj, RENDER_KEY_GEOM | RENDER_KEY_VERTEX))) return  0;

	if(renderMode & RENDER_TEXTURE)
		if (!model3DMHasKey(obj, RENDER_KEY_ST)) return 0;

	if(renderMode & RENDER_ENV)
		if (!model3DMHasKey(obj, RENDER_KEY_VERTEX)) return  0;

	return 1;
	}    

Object3D *ObjectCreate(Model3DM *modelo3DM){
    int i;  
    Object3D *O3D;
    O3D = (Object3D *) MyMalloc(sizeof(Object3D));

    // Todo lo que vaya a intervenir en el VU tiene que estar alineado
    O3D->rotation             = (libVu0FVector *)memalign(64,sizeof(libVu0FVector));
    O3D->translation          = (libVu0FVector *)memalign(64,sizeof(libVu0FVector));
	O3D->localRotation        = (libVu0FMatrix *)memalign(64,sizeof(libVu0FMatrix));
    O3D->localToWorld         = (libVu0FMatrix *)memalign(64,sizeof(libVu0FMatrix));
    O3D->localToScreen        = (libVu0FMatrix *)memalign(64,sizeof(libVu0FMatrix));
    O3D->localToScreenInverse = (libVu0FMatrix *)memalign(64,sizeof(libVu0FMatrix));

    // primero marcamos la referencia al objeto padre
    O3D->generator = modelo3DM;

    // damos espacio a los vertices proyectados
    O3D->vertices = (libVu0FVector *)memalign(64,modelo3DM->NUMVERTEX * sizeof(libVu0FVector));
    
    // damos espacio a las normales
    O3D->normalfaces   = (libVu0FVector *)MyMalloc(modelo3DM->NUMFACES * sizeof(libVu0FVector));
    O3D->normalvertex  = (libVu0FVector *)MyMalloc(modelo3DM->NUMFACES * 3 * sizeof(libVu0FVector));
    O3D->normalSvertex = (libVu0FVector *)MyMalloc(modelo3DM->NUMVERTEX * sizeof(libVu0FVector));
    
    // damos espacio al color del objeto
    O3D->color = (libVu0FVector *)MyMalloc(modelo3DM->NUMFACES * 3 * sizeof(libVu0FVector));

    // preparamos la referencia a las luces
    for(i=0;i<MAXLIGHTS ;i++) O3D->light[i] = NULL;

    // y damos espacio para el vector de incidencia de la luz
    O3D->vlight = (libVu0FVector *)memalign(64,MAXLIGHTS * sizeof(libVu0FVector));
    for(i=0;i<3;i++)
        {
        O3D->vlight[i][0] = 0.0f;
        O3D->vlight[i][1] = 0.0f;
        O3D->vlight[i][2] = 1.0f;
        O3D->vlight[i][3] = 1.0f;
        }

    // Algunos valores por defecto
    objectSetRenderMode(O3D, RENDER_FLAT);
    objectSetRenderColor(O3D, RENDER_COLOR);
    objectSetColor(O3D,0xFF,0xFF,0xFF);
    objectSetRenderMap(O3D, 0);
    objectSetLightMode(O3D,LIGHT_DIRECTIONAL);

    objectTranslate2D(O3D,0.0f,0.0f); // Translate 0
    objectScale2D(O3D,1.0f,1.0f);     // Scale 1

	O3D->colorMaterial[3]=0x80;
	O3D->textureMaterial=0;
	
	O3D->rotation[0][0]=0;
	O3D->rotation[0][1]=0;
	O3D->rotation[0][2]=0;
	
	libVu0UnitMatrix(O3D->localRotation[0]);
	objectReset(O3D);

    return O3D;
    }
void objectFree(Object3D *obj){
    free(obj->rotation);
    free(obj->translation);
    free(obj->localRotation);
    free(obj->localToWorld);
    free(obj->localToScreen);
    free(obj->localToScreenInverse);
    free(obj->vertices);
    free(obj->normalfaces);
    free(obj->normalvertex);
    free(obj->normalSvertex);
    free(obj->color);
    free(obj->vlight);

    free(obj);
    }

  
    
void objectSetRenderMode(Object3D *obj, unsigned int mode){
	// se supone que podemos activar cualquier modo de render e incluso mezclarlos
	// por ejemplo podemos poner RENDER_WIREFRAME + RENDER_GORAUD
	// pero hay algunas combinaciones un poco inutiles, asi que para optimizar nos 
	// aseguramos que no se usen a la vez

    // No permitimos que Flat y Goraud se activen a la vez
    if ((mode & RENDER_FLAT) && (mode & RENDER_GORAUD))
        {
        mode -= RENDER_FLAT;
        }

    obj->RenderMode = mode;
    }

void objectSetRenderColor(Object3D *obj, unsigned int mode){
    if (mode & RENDER_TEXTURE)
        objectSetColor(obj, 1.0f, 1.0f, 1.0f);

    obj->RenderColor = mode;
    }

void objectSetRenderMap(Object3D *obj, unsigned int mode){
    obj->RenderMap = mode;
    }

void objectSetColor(Object3D *obj,unsigned char r,unsigned char g,unsigned char b){
    // Color para los wired y flat
    obj->colorMaterial[0] = r;
    obj->colorMaterial[1] = g;
    obj->colorMaterial[2] = b;
    // obj->colorMaterial[3] = 0x80; // ya tenemos alpha
    
    // color en float para multiplicar por la incidencia de la luz
    obj->colorMaterialLight[0] = obj->colorMaterial[0] / 256.0f;
    obj->colorMaterialLight[1] = obj->colorMaterial[1] / 256.0f;
    obj->colorMaterialLight[2] = obj->colorMaterial[2] / 256.0f;
    // obj->colorMaterialLight[3] = 0x80;
    }

void objectSetAlpha(Object3D *obj,unsigned char alpha){
    obj->colorMaterial[3] = alpha;
    }


void objectSetEnvColor(Object3D *obj, unsigned char r,unsigned char g,unsigned char b,unsigned char a){
    obj->colorEnvironment[0] = r;
    obj->colorEnvironment[1] = g;
    obj->colorEnvironment[2] = b;
    obj->colorEnvironment[3] = a;
    }

void objectSetLightMode(Object3D *obj, unsigned int mode){
    obj->LightMode = mode;
    }

void objectSetTexture(Object3D *obj,int textureIndex){
    obj->textureMaterial = textureIndex;
    }
    
void objectSetTextureMap(Object3D *obj,int textureIndex){
    obj->textureEnvironment = textureIndex;
    }

void objectReset(Object3D *obj){	
	// hacemos la matrices unitarias
	libVu0UnitMatrix(obj->localToWorld[0]);
	}

    
void objectTranslate(Object3D *obj,float x,float y,float z){
	
    obj->translation[0][0] = x;
    obj->translation[0][1] = y * -1.0f;
    obj->translation[0][2] = z;

    }
	
void objectMove(Object3D *obj,float x,float y,float z){

	libVu0FMatrix r;

	libVu0FVector mov  = {x,y,z,1.0f};     // movimiento
	
	libVu0UnitMatrix(r);
	libVu0CopyMatrix(r, obj->localRotation[0]);
		
	libVu0ApplyMatrix(mov, r, mov);
	
    obj->translation[0][0] = obj->translation[0][0]+mov[0];
    obj->translation[0][1] = obj->translation[0][1]+mov[1];
    obj->translation[0][2] = obj->translation[0][2]+mov[2];
	
	
    }

void objectRotate(Object3D *obj,float x,float y,float z)
    {
	
	libVu0UnitMatrix(obj->localRotation[0]);
	
    obj->rotation[0][0] = -x;
    obj->rotation[0][1] = -y;
    obj->rotation[0][2] = -z;
	
    // cuidado con pasarnos por encima
    while(obj->rotation[0][0] > PI) obj->rotation[0][0] -= PI_2;
    while(obj->rotation[0][1] > PI) obj->rotation[0][1] -= PI_2;
    while(obj->rotation[0][2] > PI) obj->rotation[0][2] -= PI_2;

    // y por debajo
    while(obj->rotation[0][0] < -PI) obj->rotation[0][0] += PI_2;
    while(obj->rotation[0][1] < -PI) obj->rotation[0][1] += PI_2;
    while(obj->rotation[0][2] < -PI) obj->rotation[0][2] += PI_2;

}

void objectTurn(Object3D *obj,float x,float y,float z)
    {
	obj->rotation[0][0] = -x;
    obj->rotation[0][1] = -y;
    obj->rotation[0][2] = -z;
	
    // cuidado con pasarnos por encima
    while(obj->rotation[0][0] > PI) obj->rotation[0][0] -= PI_2;
    while(obj->rotation[0][1] > PI) obj->rotation[0][1] -= PI_2;
    while(obj->rotation[0][2] > PI) obj->rotation[0][2] -= PI_2;

    // y por debajo
    while(obj->rotation[0][0] < -PI) obj->rotation[0][0] += PI_2;
    while(obj->rotation[0][1] < -PI) obj->rotation[0][1] += PI_2;
    while(obj->rotation[0][2] < -PI) obj->rotation[0][2] += PI_2;

    }

void objectRotateHex(Object3D *obj,u_char x,u_char y,u_char z)
    {
    objectRotate(obj,
                HexToRadians(x),
                HexToRadians(y),
                HexToRadians(z)
                );
    }
	
void objectTurnHex(Object3D *obj,u_char x,u_char y,u_char z)
    {
    objectTurn(obj,
                HexToRadians(x),
                HexToRadians(y),
                HexToRadians(z)
                );
    }

void objectTranslate2D(Object3D *obj,float x,float y)
    {
    obj->translateX = x;
    obj->translateY = y;
    }
void objectScale2D(Object3D *obj,float x,float y)
    {
    obj->scaleX = x;
    obj->scaleY = y;
    }

void objectPrepareMatrix(Object3D *obj)
    {
    int i;
    libVu0FVector temp;
    libVu0FMatrix cameraToScreen,worldToScreen;

    // hacemos la inversa de la matriz de rotacion para calcular incidencia
    // con las luces
    libVu0InverseMatrix(obj->localToScreenInverse[0],obj->localRotation[0]);

    // hacemos una copia del cameraToScreen
    libVu0CopyMatrix(cameraToScreen, actualCamera->cameraToScreen[0]);

    // y le aplicamos el Scale y el Rotate
    cameraToScreen[0][0] *= obj->scaleX;         // Scale X
    cameraToScreen[1][1] *= obj->scaleY;         // Scale Y
    cameraToScreen[2][0] += obj->translateX;     // Translate X
    cameraToScreen[2][1] += obj->translateY;     // Translate Y
	

    // ahora con esa Matrix creamos el WorldToScreen que transforma al
    // objeto

    libVu0MulMatrix(
        worldToScreen,
        cameraToScreen,
        actualCamera->worldToCamera[0]);

    // multiplicamos la matriz del objeto por la de la camara activa
    libVu0MulMatrix(obj->localToScreen[0],
                    worldToScreen,
                    obj->localToWorld[0]);

    // Aplicamos las rotaciones a las luces
    for(i=0;i<MAXLIGHTS;i++)
    if(obj->light[i] != NULL)
    if(obj->light[i]->status == LIGHT_ON)
        {
        temp[0] = obj->light[i]->vector[0];
        temp[1] = obj->light[i]->vector[1];
        temp[2] = obj->light[i]->vector[2];
        temp[3] = obj->light[i]->vector[3];

        // y le aplicamos nuestra matriz inversa
        Render_AplicaMatriz1(temp,
                         obj->vlight[i],
                         obj->localToScreenInverse[0]);
        }

    }

void object_AplicaLight(Object3D *obj)
    {
    int i;

    if(obj->LightMode == LIGHT_DIRECTIONAL)
        {
        objectClearLight(obj);

        for(i=0;i<MAXLIGHTS;i++)
            if(obj->light[i] != NULL)
            if(obj->light[i]->status == LIGHT_ON)
                {
                if (obj->RenderMode & RENDER_FLAT)    objectSetLight_Flat(obj,i);
                if (obj->RenderMode & RENDER_GORAUD)  objectSetLight_Goraud(obj,i);
                if (obj->RenderMode & RENDER_GORAUD3) objectSetLight_Goraud3(obj,i);
                }
        }
    else
        {
        // Si el modelo tiene otro tipo de luz
        // deberiamos llamar a este metodo
        //
        // Pero, ademas como no le afecta la luz no es necesario llamarlo
        // en cada render, asi que solo se aplica 1 vez al aplicar este modo
        // de luez y luego nunca se limpia esos indices
        //
        // objectSetLight_ModeOFF(obj);
        }

    }

void objectRender_Calc(Object3D *obj)
    {
    // le decimos que prepare las matrices de este objeto
    // que empieza la marcha !!!!
    objectPrepareMatrix(obj);

    // proyectamos los vertices
    object_ProyectaVertices(obj);

    // calculamos la incidencia de la luz
    object_AplicaLight(obj);

    // Calculamos las normales para el environment
    if (obj->RenderMap == RENDER_ENV)
       object_ProyectaNormalesVerticesSingle(obj);

    }
void objectRender_Draw(Object3D *obj)
    {
    //activamos la textura
    if (obj->RenderMode >= RENDER_FLAT)
    if (obj->RenderColor & RENDER_TEXTURE) textureUse(obj->textureMaterial);

    // Enviamos el Objeto al GS
    if (obj->LightMode == LIGHT_OFF)
    	{
    	if (obj->RenderMode & RENDER_POINT    ) objectRender_Point(obj);
    	if (obj->RenderMode & RENDER_WIREFRAME) objectRender_Wireframe(obj);
    	if (obj->RenderMode & RENDER_FLAT     ) objectRender_Solid(obj);
    	if (obj->RenderMode & RENDER_GORAUD   ) objectRender_Solid(obj);
    	}    
   	else
   		{
        if (obj->RenderMode & RENDER_POINT    ) objectRender_Point(obj);
        if (obj->RenderMode & RENDER_WIREFRAME) objectRender_Wireframe(obj);
        if (obj->RenderMode & RENDER_FLAT     ) objectRender_Flat(obj);
        if (obj->RenderMode & RENDER_GORAUD   ) objectRender_Goraud(obj);
        if (obj->RenderMode & RENDER_GORAUD3  ) objectRender_Goraud3(obj);
        }    

    // Renderizamos el environment
    if (obj->RenderMap == RENDER_ENV)
       {
       textureUse(obj->textureEnvironment);
       objectRender_EnvironmentMap(obj);
       }
    }
	
void objectRender(Object3D *obj)
    {
	
	objectReset(obj);
	
	// aplicamos la rotacion a la matriz
    libVu0RotMatrix(obj->localRotation[0],
                    obj->localRotation[0],
                    obj->rotation[0]);	
					
					
   /* libVu0RotMatrix(obj->localToWorldRotation[0],
                    obj->localToWorldRotation[0],
                    obj->rotation[0]);*/
					
	obj->rotation[0][0]=0;
	obj->rotation[0][1]=0;
	obj->rotation[0][2]=0;
	
	// posicion
	libVu0TransMatrix(obj->localToWorld[0], obj->localRotation[0], obj->translation[0]);
		
    objectRender_Calc(obj);
    objectRender_Draw(obj);
    }
	   
void object_ProyectaVertices(Object3D *obj)
    {
    Render_AplicaMatriz(
                    obj->generator->vertices,
                    obj->vertices,
                    obj->localToScreen[0],
                    obj->generator->NUMVERTEX);
    }
    
void Render_AplicaMatriz1(libVu0FVector VerticesOriginales,
                          libVu0FVector VerticesProyectados,
                          libVu0FMatrix matriz)
    {
    float q;
    libVu0FVector a;
    libVu0FVector b;
            
    b[0] = VerticesOriginales[0];
    b[1] = VerticesOriginales[1];
    b[2] = VerticesOriginales[2];
    b[3] = VerticesOriginales[3];
            
    libVu0ApplyMatrix(a, matriz, b);

    VerticesProyectados[0] = a[0];
    VerticesProyectados[1] = a[1];
    VerticesProyectados[2] = a[2];
    VerticesProyectados[3] = a[3];

    q = 1.0f / VerticesProyectados[3];
    VerticesProyectados[0] *= q;
    VerticesProyectados[1] *= q;
    VerticesProyectados[2] *= q;
    VerticesProyectados[3]  = q;
    }


void Render_AplicaMatriz(libVu0FVector *VerticesOriginales,
                         libVu0FVector *VerticesProyectados,
                         libVu0FMatrix matriz,
                         int nPuntos)
    {
    int i;
    
    for(i=0;i<nPuntos;i++) 
            {
            float q;
            libVu0FVector a;
            libVu0FVector b;
            
            b[0] = VerticesOriginales[i][0];
            b[1] = VerticesOriginales[i][1];
            b[2] = VerticesOriginales[i][2];
            b[3] = VerticesOriginales[i][3];
            
            libVu0ApplyMatrix(a, matriz, b);

            VerticesProyectados[i][0] = a[0];
            VerticesProyectados[i][1] = a[1];
            VerticesProyectados[i][2] = a[2];
            VerticesProyectados[i][3] = a[3];
            
            q = 1.0f / VerticesProyectados[i][3];
            VerticesProyectados[i][0] *= q;
            VerticesProyectados[i][1] *= q;
            VerticesProyectados[i][2] *= q;
            VerticesProyectados[i][3]  = q;
            }
    }

void dibujarcubo(float x, float y, float z, int tam)
    {

        addPRIM(LIB_PRIM_TRIANGLE,0,0,0,1,0,0,0,0);
		
		addRGBAQ(0x80,0x80,0x80, 0x80, 1.0f);

		addXYZ2(x-tam,y-tam,z-tam);
		addXYZ2(x-tam,y-tam,z+tam);
		addXYZ2(x-tam,y+tam,z-tam);
		addXYZ2(x-tam,y+tam,z+tam);
		addXYZ2(x+tam,y+tam,z-tam);
		addXYZ2(x+tam,y+tam,z+tam);
		addXYZ2(x+tam,y-tam,z-tam);
		addXYZ2(x+tam,y-tam,z+tam);
		addXYZ2(x-tam,y-tam,z-tam);
		addXYZ2(x-tam,y-tam,z+tam);
	
    }
    
	
void objectRender_Point(Object3D *obj)
    {
    int i;        

    addPRIM(LIB_PRIM_POINT,0,0,0,0,0,0,0,0);

    addRGBAQ(obj->colorMaterial[0],
             obj->colorMaterial[1],
             obj->colorMaterial[2],
             obj->colorMaterial[3],
             1.0f);

    for(i=0;i<obj->generator->NUMVERTEX;i++) 
        addXYZ2(obj->vertices[i][0],
                obj->vertices[i][1],
                obj->vertices[i][2]);
    }
    
void objectRender_Wireframe(Object3D *obj)
    {
    int v1,v2,v3;
    int i;        

    addRGBAQ(obj->colorMaterial[0],
             obj->colorMaterial[1],
             obj->colorMaterial[2],
             obj->colorMaterial[3],
             1.0f);

    for(i=0;i<obj->generator->NUMFACES;i++) 
        {
        v1 = obj->generator->faces[i*3+0];
        v2 = obj->generator->faces[i*3+1];
        v3 = obj->generator->faces[i*3+2];

        addPRIM(LIB_PRIM_LINESTRIP,0,0,0,0,0,0,0,0);
        addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);
        addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);
        addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
        addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);
        
        }
    }
void objectRender_Solid(Object3D *obj)
    {
    int f1,f2,f3;
    int v1,v2,v3;
    int i;
	int a;
	
	if(obj->colorMaterial[3]<0x80){
		a=1;
	}else{
		a=0;
	}
	
    if (obj->RenderColor & RENDER_TEXTURE)
        addPRIM(LIB_PRIM_TRIANGLE,0,1,0,a,0,0,0,0);
    else
        addPRIM(LIB_PRIM_TRIANGLE,0,0,0,a,0,0,0,0);

    for(i=0;i<obj->generator->NUMFACES;i++) 
        {
        v1 = obj->generator->faces[i*3+0];
        v2 = obj->generator->faces[i*3+1];
        v3 = obj->generator->faces[i*3+2];

        // vertices
        switch(obj->RenderColor)
            {
            case RENDER_TEXTURE:
                f1 = obj->generator->Tfaces[i*3+0];
                f2 = obj->generator->Tfaces[i*3+1];
                f3 = obj->generator->Tfaces[i*3+2];
								
				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
	                addRGBAQ(obj->colorMaterial[0],obj->colorMaterial[1],obj->colorMaterial[2],obj->colorMaterial[3], obj->vertices[v1][3]);
	                addST(obj->generator->tvertices[f1][0] * obj->vertices[v1][3], obj->generator->tvertices[f1][1] * obj->vertices[v1][3]);
	                addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);

	                addRGBAQ(obj->colorMaterial[0],obj->colorMaterial[1],obj->colorMaterial[2],obj->colorMaterial[3], obj->vertices[v2][3]);
	                addST(obj->generator->tvertices[f2][0] * obj->vertices[v2][3], obj->generator->tvertices[f2][1] * obj->vertices[v2][3]);
	                addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);

	                addRGBAQ(obj->colorMaterial[0],obj->colorMaterial[1],obj->colorMaterial[2],obj->colorMaterial[3], obj->vertices[v3][3]);
	                addST(obj->generator->tvertices[f3][0] * obj->vertices[v3][3], obj->generator->tvertices[f3][1] * obj->vertices[v3][3]);
	                addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
				}
                break;

            case RENDER_COLOR:
                addRGBAQ(obj->colorMaterial[0],obj->colorMaterial[1],obj->colorMaterial[2],obj->colorMaterial[3], 1.0f);

				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
	                addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);
	                addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);
	                addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
				}
                break;
            }
        }
    }

void objectRender_Flat(Object3D *obj)
    {
    int f1,f2,f3;
    int v1,v2,v3;
    int i;
	int a;
	
	if(obj->colorMaterial[3]<0x80){
		a=1;
	}else{
		a=0;
	}
	
    if (obj->RenderColor & RENDER_TEXTURE)
        addPRIM(LIB_PRIM_TRIANGLE,0,1,0,a,0,0,0,0);
    else
        addPRIM(LIB_PRIM_TRIANGLE,0,0,0,a,0,0,0,0);

    for(i=0;i<obj->generator->NUMFACES;i++) 
        {
        v1 = obj->generator->faces[i*3+0];
        v2 = obj->generator->faces[i*3+1];
        v3 = obj->generator->faces[i*3+2];
	
        // vertices
        switch(obj->RenderColor)
            {
            case RENDER_TEXTURE:
                f1 = obj->generator->Tfaces[i*3+0];
                f2 = obj->generator->Tfaces[i*3+1];
                f3 = obj->generator->Tfaces[i*3+2];

					if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
		                addRGBAQ(obj->color[i][0],obj->color[i][1],obj->color[i][2], obj->colorMaterial[3], obj->vertices[v1][3]);
		                addST(obj->generator->tvertices[f1][0] * obj->vertices[v1][3], obj->generator->tvertices[f1][1] * obj->vertices[v1][3]);
		                addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);

		                addRGBAQ(obj->color[i][0],obj->color[i][1],obj->color[i][2], obj->colorMaterial[3], obj->vertices[v2][3]);
		                addST(obj->generator->tvertices[f2][0] * obj->vertices[v2][3], obj->generator->tvertices[f2][1] * obj->vertices[v2][3]);
		                addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);

		                addRGBAQ(obj->color[i][0],obj->color[i][1],obj->color[i][2], obj->colorMaterial[3], obj->vertices[v3][3]);
		                addST(obj->generator->tvertices[f3][0] * obj->vertices[v3][3], obj->generator->tvertices[f3][1] * obj->vertices[v3][3]);
		                addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
					}
                break;

            case RENDER_COLOR:
                addRGBAQ(obj->color[i][0],obj->color[i][1],obj->color[i][2], obj->colorMaterial[3], 1.0f);
				
				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
							  
					addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);
					addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);
					addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
					
				}
                break;
            // default:
            //    printf("objectRender_Flat(Object3D *obj) -> Undefined obj->RenderColor \n");
            //    #ifdef DEBUGMODE
            //    
            //    #endif
            //    break;
            }
        }
    }
    
	
void objectRender_EnvironmentMap(Object3D *obj)
    {
    int v1;
    int i;

    addPRIM(LIB_PRIM_TRIANGLE,       // PRIM (Primitive type)
                0,                   // IIP  (Gouraud)
                1,                   // TME  (Textured)
                0,                   // FGE  (Fogging)
                1,                   // ABE  (Alpha Blending)
                0,                   // AA1  (Anti-Aliasing)
                0,                   // FST  (Use ST for texture coords)
                0,                   // CTXT (Context)
                0);                  // FIX  (Fragment control)
    

    addRGBAQ(obj->colorEnvironment[0],
             obj->colorEnvironment[1],
             obj->colorEnvironment[2],
             obj->colorEnvironment[3],
             1.0f);

    /*
    addRGBAQ(0x80,0x80,0x80,0x10,1.0f);
    */

    for(i=0;i<obj->generator->NUMFACES*3;i++) 
        {
        v1 = obj->generator->faces[i];
		if(libVu0ClipScreen(obj->vertices[v1])==0){
	        addST(
	              (obj->normalSvertex[v1][0] * 0.5f) + 0.5f,    // <- S
	              (obj->normalSvertex[v1][1] * 0.5f) + 0.5f);   // <- T
	        addXYZ2(              
	                obj->vertices[v1][0],  // <- X
	                obj->vertices[v1][1],  // <- Y
	                obj->vertices[v1][2]); // <- Z
		}
        }
    }

void objectRender_Goraud(Object3D *obj)
    {
    int v1,v2,v3;
    int i;
	int a;
	
	if(obj->colorMaterial[3]<0x80){
		a=1;
	}else{
		a=0;
	}
	
    if (obj->RenderColor & RENDER_TEXTURE)
        addPRIM(LIB_PRIM_TRIANGLE,1,1,0,a,0,0,0,0);
    else
        addPRIM(LIB_PRIM_TRIANGLE,1,0,0,a,0,0,0,0);
   	
   	
    for(i=0;i<obj->generator->NUMFACES;i++) 
        {
        v1 = obj->generator->faces[i*3+0];
        v2 = obj->generator->faces[i*3+1];
        v3 = obj->generator->faces[i*3+2];

        if (obj->RenderColor & RENDER_TEXTURE)
            {
            int f1 = obj->generator->Tfaces[i*3+0];
            int f2 = obj->generator->Tfaces[i*3+1];
            int f3 = obj->generator->Tfaces[i*3+2];
			
				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
		            addRGBAQ(obj->color[v1][0],obj->color[v1][1],obj->color[v1][2], obj->colorMaterial[3], obj->vertices[v1][3]);
		            addST(obj->generator->tvertices[f1][0] * obj->vertices[v1][3], obj->generator->tvertices[f1][1] * obj->vertices[v1][3]);
		            addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);

		            addRGBAQ(obj->color[v2][0],obj->color[v2][1],obj->color[v2][2], obj->colorMaterial[3],obj->vertices[v2][3]);
		            addST(obj->generator->tvertices[f2][0] * obj->vertices[v2][3], obj->generator->tvertices[f2][1] * obj->vertices[v2][3]);
		            addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);

		            addRGBAQ(obj->color[v3][0],obj->color[v3][1],obj->color[v3][2], obj->colorMaterial[3], obj->vertices[v3][3]);
		            addST(obj->generator->tvertices[f3][0] * obj->vertices[v3][3], obj->generator->tvertices[f3][1] * obj->vertices[v3][3]);
		            addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
				}
            }
        else
            {
			
				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){			
		            addRGBAQ(obj->color[v1][0],obj->color[v1][1],obj->color[v1][2], obj->colorMaterial[3], 1.0f);
		            addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);

		            addRGBAQ(obj->color[v2][0],obj->color[v2][1],obj->color[v2][2], obj->colorMaterial[3], 1.0f);
		            addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);

		            addRGBAQ(obj->color[v3][0],obj->color[v3][1],obj->color[v3][2], obj->colorMaterial[3], 1.0f);
		            addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
				}
          	}
       	}
    }


void objectRender_Goraud3(Object3D *obj)
    {
    int v1,v2,v3;
    int i;    
	int a;
	
	if(obj->colorMaterial[3]<0x80){
		a=1;
	}else{
		a=0;
	}
	
    if (obj->RenderColor & RENDER_TEXTURE)
        addPRIM(LIB_PRIM_TRIANGLE,1,1,0,a,0,0,0,0);
    else
        addPRIM(LIB_PRIM_TRIANGLE,1,0,0,a,0,0,0,0);
   	
   	
    for(i=0;i<obj->generator->NUMFACES;i++) 
        {
        v1 = obj->generator->faces[i*3+0];
        v2 = obj->generator->faces[i*3+1];
        v3 = obj->generator->faces[i*3+2];

        if (obj->RenderColor & RENDER_TEXTURE)
            {
            int f1 = obj->generator->Tfaces[i*3+0];
            int f2 = obj->generator->Tfaces[i*3+1];
            int f3 = obj->generator->Tfaces[i*3+2];
			
				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
		            addRGBAQ(obj->color[i*3+0][0],obj->color[i*3+0][1],obj->color[i*3+0][2], obj->colorMaterial[3], obj->vertices[v1][3]);
		            addST(obj->generator->tvertices[f1][0] * obj->vertices[v1][3], obj->generator->tvertices[f1][1] * obj->vertices[v1][3]);
		            addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);

		            addRGBAQ(obj->color[i*3+1][0],obj->color[i*3+1][1],obj->color[i*3+1][2], obj->colorMaterial[3],obj->vertices[v2][3]);
		            addST(obj->generator->tvertices[f2][0] * obj->vertices[v2][3], obj->generator->tvertices[f2][1] * obj->vertices[v2][3]);
		            addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);

		            addRGBAQ(obj->color[i*3+2][0],obj->color[i*3+2][1],obj->color[i*3+2][2], obj->colorMaterial[3], obj->vertices[v3][3]);
		            addST(obj->generator->tvertices[f3][0] * obj->vertices[v3][3], obj->generator->tvertices[f3][1] * obj->vertices[v3][3]);
		            addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
				}
            }
        else
            {
				if(libVu0ClipScreen3(obj->vertices[v1],obj->vertices[v2],obj->vertices[v3])==0){
		            addRGBAQ(obj->color[i*3+0][0],obj->color[i*3+0][1],obj->color[i*3+0][2], obj->colorMaterial[3], 1.0f);
		            addXYZ2(obj->vertices[v1][0],obj->vertices[v1][1],obj->vertices[v1][2]);

		            addRGBAQ(obj->color[i*3+1][0],obj->color[i*3+1][1],obj->color[i*3+1][2], obj->colorMaterial[3], 1.0f);
		            addXYZ2(obj->vertices[v2][0],obj->vertices[v2][1],obj->vertices[v2][2]);

		            addRGBAQ(obj->color[i*3+2][0],obj->color[i*3+2][1],obj->color[i*3+2][2], obj->colorMaterial[3], 1.0f);
		            addXYZ2(obj->vertices[v3][0],obj->vertices[v3][1],obj->vertices[v3][2]);
				}
          	}
       	}
    }


/*
void Model3DMScale(Object_3DM *obj,float x,float y,float z)
    {
    int i;
    
    for(i=0;i<obj->NUMVERTEX;i++)
        {
        obj->vertices[i][0] *= x;
        obj->vertices[i][1] *= y;
        obj->vertices[i][2] *= z;
        }
    }
*/



void objectSetLight(Object3D *obj,int index, ObjectLight *luz)
    {
    if(index<MAXLIGHTS) obj->light[index] = luz;
    }




void objectSetLight_Flat(Object3D *obj,int light)
    {
    int i;
    float d_prod;    
    unsigned char indice;
   	libVu0FVector b;    // vector temporal
   	libVu0FVector l;    // vector de la luz
   	ObjectLight *luz;

   	luz = obj->light[light];
   	l[0] = obj->vlight[light][0];
   	l[1] = obj->vlight[light][1];
   	l[2] = obj->vlight[light][2];
    l[3] = obj->vlight[light][3];

    for(i=0;i<obj->generator->NUMFACES;i++) 
        {
        b[0] = obj->generator->normalfaces[i][0];
        b[1] = obj->generator->normalfaces[i][1];
        b[2] = obj->generator->normalfaces[i][2];
        b[3] = obj->generator->normalfaces[i][3];

        // intensidad de la luz con este vector
        d_prod = vu0DotProduct(&b,&l);
        if(d_prod<0.0f) d_prod=0.0f;

        // lo pasamos a un char para leer el valor de la tabla precalc
        indice = (unsigned char)(d_prod * 0xFF);


        obj->color[i][0]+= (obj->colorMaterialLight[0] * luz->colorRed[indice]);
        obj->color[i][1]+= (obj->colorMaterialLight[1] * luz->colorGreen[indice]);
        obj->color[i][2]+= (obj->colorMaterialLight[2] * luz->colorBlue[indice]);
/*
        obj->color[i][0]+= luz->colorRed[indice];
        obj->color[i][1]+= luz->colorGreen[indice];
        obj->color[i][2]+= luz->colorBlue[indice];
*/
        // si nos pasamos, nos quedamos en blanco
        if (obj->color[i][0]>0xFF) obj->color[i][0]=0xFF;
        if (obj->color[i][1]>0xFF) obj->color[i][1]=0xFF;
        if (obj->color[i][2]>0xFF) obj->color[i][2]=0xFF;
       	}
    }

void objectSetLight_Goraud(Object3D *obj,int light)
    {
    int i;
    float d_prod;    
    unsigned char indice;
   	libVu0FVector b;    // vector temporal
   	libVu0FVector l;    // vector de la luz
   	ObjectLight *luz;
   	
   	luz = obj->light[light];
   	l[0] = obj->vlight[light][0];
   	l[1] = obj->vlight[light][1];
   	l[2] = obj->vlight[light][2];
    l[3] = obj->vlight[light][3];
    
    for(i=0;i<obj->generator->NUMVERTEX;i++) 
        {
        b[0] = obj->generator->normalSvertex[i][0];
        b[1] = obj->generator->normalSvertex[i][1];
        b[2] = obj->generator->normalSvertex[i][2];
        b[3] = obj->generator->normalSvertex[i][3];

        // intensidad de la luz con este vector
        d_prod = vu0DotProduct(&b,&l);
        if(d_prod<0.0f) d_prod=0.0f;
        indice = (unsigned char)(d_prod * 0xFF);
        obj->color[i][0]+=luz->colorRed[indice]*obj->colorMaterialLight[0];
        obj->color[i][1]+=luz->colorGreen[indice]*obj->colorMaterialLight[1];
        obj->color[i][2]+=luz->colorBlue[indice]*obj->colorMaterialLight[2];
        if (obj->color[i][0]>0xFF) obj->color[i][0] = 0xFF;
        if (obj->color[i][1]>0xFF) obj->color[i][1] = 0xFF;
        if (obj->color[i][2]>0xFF) obj->color[i][2] = 0xFF;
       	}
   }

void objectSetLight_Goraud3(Object3D *obj,int light)
    {
    int i;
    float d_prod;    
    unsigned char indice;
   	libVu0FVector b;    // vector temporal
   	libVu0FVector l;    // vector de la luz
   	ObjectLight *luz;
   	
   	luz = obj->light[light];
   	l[0] = obj->vlight[light][0];
   	l[1] = obj->vlight[light][1];
   	l[2] = obj->vlight[light][2];
    l[3] = obj->vlight[light][3];

    for(i=0;i<obj->generator->NUMFACES * 3;i++) 
        {
        b[0] = obj->generator->normalvertex[i][0];
        b[1] = obj->generator->normalvertex[i][1];
        b[2] = obj->generator->normalvertex[i][2];
        b[3] = obj->generator->normalvertex[i][3];

        // intensidad de la luz con este vector
        d_prod = vu0DotProduct(&b,&l);
        if(d_prod<0.0f) d_prod=0.0f;
        
        // lo multiplicamos por la intensidad de la luz
        // ambiente = 0.2 <- color minimo
        // diffuse  = 0.8 <- Color maximo
        // rampa = (dProd * (diffuse - ambiente)) + ambiente
        // obj->color[i][0]+=(d_prod*(luz->diffuse[0]-luz->ambient[0])+luz->ambient[0])*obj->colorMaterial[0]; 
        // obj->color[i][1]+=(d_prod*(luz->diffuse[1]-luz->ambient[1])+luz->ambient[1])*obj->colorMaterial[1]; 
        // obj->color[i][2]+=(d_prod*(luz->diffuse[2]-luz->ambient[2])+luz->ambient[2])*obj->colorMaterial[2]; 
        indice = (unsigned char)(d_prod * 0xFF);
        obj->color[i][0]+=luz->colorRed[indice]*obj->colorMaterialLight[0];
        obj->color[i][1]+=luz->colorGreen[indice]*obj->colorMaterialLight[1];
        obj->color[i][2]+=luz->colorBlue[indice]*obj->colorMaterialLight[2];
        
        if (obj->color[i][0]>0xFF) obj->color[i][0] = 0xFF;
        if (obj->color[i][1]>0xFF) obj->color[i][1] = 0xFF;
        if (obj->color[i][2]>0xFF) obj->color[i][2] = 0xFF;
        }
   }


void makefashion(Object3D *obj)
    {
    int i,n;
    
    n =  obj->generator->NUMFACES * 3;
    
    for(i=0;i<n;i++)
        {
        obj->color[i][0] = (256.0f / n ) * i;
        obj->color[i][1] = 256 - obj->color[i][0];
        obj->color[i][2] = 128;
        }
    
    }


////////////////
void object_ProyectaNormalesCara(Object3D *obj)
    {
    Render_AplicaMatriz(
                    obj->generator->normalfaces,
                    obj->normalfaces,
                    obj->localToWorld[0],
                    obj->generator->NUMFACES);
    }

void object_ProyectaNormalesVertices(Object3D *obj)
    {
    Render_AplicaMatriz(
                    obj->generator->normalvertex,
                    obj->normalvertex,
                    obj->localToWorld[0],
                    obj->generator->NUMFACES * 3);
    }

void object_ProyectaNormalesVerticesSingle(Object3D *obj)
    {
    Render_AplicaMatriz(
                    obj->generator->normalSvertex,
                    obj->normalSvertex,
                    obj->localToWorld[0],
                    obj->generator->NUMVERTEX);
    }


void objectClearLight(Object3D *obj)
    {
    int i;

    for(i=0;i<obj->generator->NUMFACES * 3;i++)
        {
       	obj->color[i][0] = 0.0f;
       	obj->color[i][1] = 0.0f;
       	obj->color[i][2] = 0.0f;
       	}
    }

