/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include <libVu0.h>

#include "llibPlanar.h"
#include "math.h"
#include "vu0.h"

/* ******* BEZIER ******** */
#define STEPSBEZIER 32

typedef struct 
    {
    float x;
    float y;
    } point;

void makeLinearRamp(unsigned char *array, 
                    float inicialValue, 
                    float inicialPosition, 
                    float finalValue,
                    float finalPosition);

point *setPoint(point *p, float x, float y);

void CubicBezier(point *p, point *p0,point *p1,
                 point *p2,point *p3,
                 float mu);
/* ******* BEZIER ******** */


ObjectLight *lightCreate(float x,float y,float z)
    {
    ObjectLight *luz = (ObjectLight *)malloc(sizeof(ObjectLight));

    lightReset(luz);
    lightTranslate(luz,x,y,z);
    lightSetColor (luz,1.0f,1.0f,1.0f);
    lightSetActive(luz,LIGHT_ON);
    
    lightCreateColorRamp(luz);
    
    return luz;
    }

ObjectLight *LoadLight(char *filename)
    {
    ObjectLight *luz;
    float temp[3];
    char header[4];
    int fd;

    luz = (ObjectLight *)malloc(sizeof(ObjectLight));
    lightReset(luz);

    fd = openFile(filename, O_RDONLY);
      readFile(fd, &header[0], sizeof(char)*4); // el header no nos sirve de nada
      readFile(fd, temp, sizeof(float)*3); lightSetAmbient(luz,temp[0],temp[1],temp[2],1.0f);
      readFile(fd, temp, sizeof(float)*3); lightSetDiffuse(luz,temp[0],temp[1],temp[2],1.0f);
      readFile(fd, temp, sizeof(float)*3); lightSetSpecular(luz,temp[0],temp[1],temp[2],1.0f);
      readFile(fd, temp, sizeof(float)*3); lightSetCSpecular(luz,temp[0],temp[1],temp[2],1.0f);
      readFile(fd, temp, sizeof(float)*3); lightSetSpecularNode(luz,temp[0],temp[1],temp[2]);
      readFile(fd, temp, sizeof(float)*3); lightSetCSpecularNode(luz,temp[0],temp[1],temp[2]);
	closeFile(fd);

    lightSetActive(luz,LIGHT_ON);  // se activa la luz
    lightCreateColorRamp(luz);     // y se calcula la rampa de color

	return luz;
    }


void lightFree(ObjectLight *luz)
    {
    free(luz);
    }

void lightSetActive(ObjectLight *luz,int status)
    {
    luz->status = status;
    }
    
void lightReset(ObjectLight *luz)
    {
    luz->posicion[0] = 0.0f;
    luz->posicion[1] = 0.0f;
    luz->posicion[2] = 1.0f;
    luz->posicion[3] = 1.0f;
    lightUpdateVector(luz);
    }

void lightTranslate(ObjectLight *luz,float x,float y,float z)
    {
    luz->posicion[0] += x;
    luz->posicion[1] += y * -1.0f;
    luz->posicion[2] += z * -1.0f;
    lightUpdateVector(luz);
    }

void lightUpdateVector(ObjectLight *luz)
    {
    libVu0FVector t;
    
    t[0] = 0.0f - luz->posicion[0];
    t[1] = 0.0f - luz->posicion[1];
    t[2] = 0.0f - luz->posicion[2];
    t[3] = 1.0f;
    
    vu0NormalizaVector(&t,&t);

    luz->vector[0] = t[0];
    luz->vector[1] = t[1];
    luz->vector[2] = t[2];
    luz->vector[3] = t[3];
    }

void lightSetColor(ObjectLight *luz,float r,float g,float b)
    {
    lightSetAmbient(luz,r,g,b,0.2f);
    lightSetDiffuse(luz,r,g,b,0.8f);

    lightSetSpecular(luz,r,g,b,0.2f);
    lightSetCSpecular(luz,r,g,b,0.8f);
    lightSetSpecularNode(luz,1.0f,1.0f,1.0f);
    lightSetCSpecularNode(luz,0.0f,0.0f,0.0f);
    }    

void lightSetAmbient(ObjectLight *luz,float r,float g,float b,float intensity)
    {
    luz->ambient[0] = r * intensity;
    luz->ambient[1] = g * intensity;
    luz->ambient[2] = b * intensity;
    luz->ambient[3] = 1.0f;
    }
    
void lightSetDiffuse(ObjectLight *luz,float r,float g,float b,float intensity)
    {
    luz->diffuse[0] = r * intensity;
    luz->diffuse[1] = g * intensity;
    luz->diffuse[2] = b * intensity;
    luz->diffuse[3] = 1.0f;
    }

void lightSetSpecular(ObjectLight *luz,float r,float g,float b,float intensity)
    {
    luz->specular[0] = r * intensity;
    luz->specular[1] = g * intensity;
    luz->specular[2] = b * intensity;
    luz->specular[3] = 1.0f;
    }

void lightSetCSpecular(ObjectLight *luz,float r,float g,float b,float intensity)
    {
    luz->cspecular[0] = r * intensity;
    luz->cspecular[1] = g * intensity;
    luz->cspecular[2] = b * intensity;
    luz->cspecular[3] = 1.0f;
    }

void lightSetSpecularNode(ObjectLight *luz,float r,float g,float b)
    {
    luz->specularNode[0] = r;
    luz->specularNode[1] = g;
    luz->specularNode[2] = b;
    luz->specularNode[3] = 1.0f;
    }

void lightSetCSpecularNode(ObjectLight *luz,float r,float g,float b)
    {
    luz->cspecularNode[0] = r;
    luz->cspecularNode[1] = g;
    luz->cspecularNode[2] = b;
    luz->cspecularNode[3] = 1.0f;
    }



void lightCreateColorRamp(ObjectLight *luz)
    {
    point m1,m2,m3,m4;
    point p1,p2;
    int i,j;
    char *array;

    array = luz->colorRed; // para que no se queje el compilador
    
    for(i=0;i<3;i++)
        {
        switch(i)
           {
           case 0: array = &luz->colorRed[0]; break;
           case 1: array = &luz->colorGreen[0]; break;
           case 2: array = &luz->colorBlue[0]; break;
           }    

        // primero limpiamos el ColorRamp
        for(j=0;j<256;j++) array[j] = 0;

        // Creamos la Rampa Ambient-Diffuse
        makeLinearRamp(array,
                luz->ambient[i],
                0.0f,
                luz->diffuse[i],
                1.0f);


        // Creamos la curva ContraSpecular
        if(luz->cspecularNode[i]>0.0f)
            {
            // preparamos los nodos de control
            setPoint(&m1,0.0f, luz->cspecular[i]);
            setPoint(&m2,0.0f, (m1.y - luz->ambient[i]) / 2.0f + luz->ambient[i]);
            setPoint(&m3,0.0f, luz->ambient[i]);
            setPoint(&m4,luz->cspecularNode[i], (luz->cspecularNode[i] * (luz->diffuse[i] - luz->ambient[i])) + luz->ambient[i]);
    
            // y el bezier
            for(j=0;j<STEPSBEZIER;j++)
                {
                float mu1 = ((float)(j+0)) / ((float)STEPSBEZIER);
                float mu2 = ((float)(j+1)) / ((float)STEPSBEZIER);
                CubicBezier(&p1,&m1,&m2,&m3,&m4,mu1);
                CubicBezier(&p2,&m1,&m2,&m3,&m4,mu2);
                makeLinearRamp(array, p1.y, p1.x, p2.y, p2.x);
                }    
            }

        // Lo mismo para la curva del Specular    
        if(luz->specularNode[i]<1.0f)
            {
            setPoint(&m4,1.0f, luz->specular[i]);
            setPoint(&m3,1.0f, (m4.y - luz->diffuse[i]) / 2.0f + luz->diffuse[i]);
            setPoint(&m2,1.0f, luz->diffuse[i]);
            setPoint(&m1,luz->specularNode[i], (luz->specularNode[i] * (luz->diffuse[i] - luz->ambient[i])) + luz->ambient[i]);
            
            for(j=0;j<=STEPSBEZIER;j++)
                {
                float mu1 = ((float)(j+0)) / ((float)STEPSBEZIER);
                float mu2 = ((float)(j+1)) / ((float)STEPSBEZIER);
                CubicBezier(&p1,&m1,&m2,&m3,&m4,mu1);
                CubicBezier(&p2,&m1,&m2,&m3,&m4,mu2);
                makeLinearRamp(array, p1.y, p1.x, p2.y, p2.x);
                }    
            array[0xFF] = array[0xFF-1];
            }

        }        

    }    

void makeLinearRamp(unsigned char  *array, 
                    float inicialValue, float inicialPosition, 
                    float finalValue,   float finalPosition)
    {
    if(finalPosition<inicialPosition)
            makeLinearRamp(array, finalValue, finalPosition,inicialValue, inicialPosition);

    int i;
    int ipos = (int)(inicialPosition * 256);
    int fpos = (int)(finalPosition * 256);

    for(i=ipos;i<fpos;i++)
        {
        float x = (float)(finalPosition * i) / (float)(finalPosition * 256);
        float t = (x - inicialPosition) / (finalPosition - inicialPosition);
        // float x = (t * (finalPosition - inicialPosition)) + inicialPosition;
        float y = (t * (finalValue - inicialValue)) + inicialValue;
        array[i] = (unsigned char)(y * 256);
        }

    }    


point *setPoint(point *p, float x, float y)
    {
    p->x = x;
    p->y = y;
    return p;
    }    
/*
  Piecewise cubic bezier curve as defined by Adobe in Postscript
  The two end points are p0 and p3
  Their associated control points are p1 and p2
  mu : value 0.0f - 1.0f
  p  : return
*/

void CubicBezier(point *p, point *p0,point *p1,
                 point *p2,point *p3,
                 float mu)
		{
		point a,b,c;

   		c.x = 3 * (p1->x - p0->x);
   		c.y = 3 * (p1->y - p0->y);

   		b.x = 3 * (p2->x - p1->x) - c.x;
   		b.y = 3 * (p2->y - p1->y) - c.y;

   		a.x = p3->x - p0->x - c.x - b.x;
   		a.y = p3->y - p0->y - c.y - b.y;

   		p->x = a.x * mu * mu * mu + b.x * mu * mu + c.x * mu + p0->x;
   		p->y = a.y * mu * mu * mu + b.y * mu * mu + c.y * mu + p0->y;
		}



/*
void objectClearLight(Object3D *obj)
    {
    int i;

    for(i=0;i<obj->generator->NUMFACES * 3;i++) 
        {
       	obj->color[i][0] = 0.0f;
       	obj->color[i][1] = 0.0f;
       	obj->color[i][2] = 0.0f;
       	}    
    }
*/  

