/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "llibplanar.h"
#include "math.h"

int videoMode;
unsigned int GSLayerZ;

/*
 * Estructura para controlar las primitivas GS
 *
 */
typedef struct
    {
    u_char PRIM;
    u_char IIP;
    u_char ABE;
    } GSPrimInfo;

typedef struct
    {
    int x,y;
    int w,h;
    } gfxBox;



GSPrimInfo GSInfo;  // Prim Info
u_char gfxABE;      // Temporal Alpha


gfxBox screenSize;
gfxBox screenScissor;
gfxBox screenScissorTemp;

int GSSCREEN_CW;
int GSSCREEN_CH;




int GSSCREEN_interMode;
int GSSCREEN_PSMCT;
int GSSCREEN_PSMZ;

/** 
  * estructura para controlar las capas que tenemos para dibujar en el
  * LayerZ 
  */
typedef struct
    {
    int nLayers;                    /**< numero de capas */  
    int active;                     /**< capa activa     */  
    unsigned int min[MAXLAYERZ];    /**< valor minimo    */  
    unsigned int max[MAXLAYERZ];    /**< valor maximo    */  
    } GSLayerZInfo;

GSLayerZInfo layerz;



qUnion *gsPacket;       // GSPacket Pointer
u_int gsPacketCount;    // and Packet count
int buffer;       		// buffer actual
libGsDBuff db;    		// doble buffer


__inline__ void setGsPacketsize(int kbytes)
    {
    gsPacket= (qUnion*)memalign(64,kbytes * 1024 * sizeof(qUnion));
    gsPacketCount=1;
    }

__inline__ void addXYZ2(u_short x1, u_short y1, u_int z1)
	{
    gsPacket[gsPacketCount].u64[0]=LIB_SET_XYZ2(x1<<4, y1<<4, z1);
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_XYZ2;
    }

__inline__ void addRGBAQ(u_char R, u_char G, u_char B, u_char A, float Q)
	{
    gsPacket[gsPacketCount].u8[0]=R;
    gsPacket[gsPacketCount].u8[1]=G;
    gsPacket[gsPacketCount].u8[2]=B;
    gsPacket[gsPacketCount].u8[3]=A;
    gsPacket[gsPacketCount].f32[1]=Q;
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_RGBAQ;
    }
    
__inline__ void addST(float S, float T)
	{
    gsPacket[gsPacketCount].f32[0]=S;
    gsPacket[gsPacketCount].f32[1]=T;
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_ST;
    }

__inline__ void addUV(u_int U, u_int V)
    {
    gsPacket[gsPacketCount].u64[0]=LIB_SET_UV(U<<4, V<<4);
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_UV;
    }

__inline__ void addPRIM(u_char prim, u_char iip, u_char tme, u_char fge, u_char abe, u_char aa1, u_char fst, u_char ctxt, u_char fix)
	{
    gsPacket[gsPacketCount].u64[0]=LIB_SET_PRIM(prim, iip, tme, fge, abe, aa1, fst, ctxt, fix);
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_PRIM;
    }

__inline__ void addSCISSOR_1(u_short SCAX0, u_short SCAX1, u_short SCAY0, u_short SCAY1)
	{
    gsPacket[gsPacketCount].u64[0]=LIB_SET_SCISSOR_1(SCAX0,SCAX1,SCAY0,SCAY1);
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_SCISSOR_1;
    }

__inline__ void addALPHA_1(u_char A, u_char B, u_char C, u_char D, u_char FIX)
	{
    gsPacket[gsPacketCount].u64[0]=LIB_SET_ALPHA_1(A,B,C,D,FIX);
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_ALPHA_1;
    }

__inline__ void addTEST_1(u_char ATE, u_char ATST, u_char AREF, u_char AFAIL, u_char DATE, u_char DATM, u_char ZTE, u_char ZTST)
	{
    gsPacket[gsPacketCount].u64[0]=LIB_SET_TEST_1(ATE, ATST, AREF, AFAIL, DATE, DATM, ZTE, ZTST);
    gsPacket[gsPacketCount++].u64[1]=LIB_GSREG_TEST_1;
    }

//

__inline__ void gfxSetScissor(int x1,int y1,int x2,int y2)
    {
    screenScissor.x = x1;
    screenScissor.y = y2;
    screenScissor.w = x2;
    screenScissor.h = y2;
    addSCISSOR_1(x1+GSSCREEN_CW, x2+GSSCREEN_CW, y1+GSSCREEN_CH, y2+GSSCREEN_CH);
    }
__inline__ void gfxClearScissor()
    {
    // addSCISSOR_1(0, screenSize.w-1, 0, GSSCREEN_H-1);
    gfxSetScissor(-GSSCREEN_CW,-GSSCREEN_CH,GSSCREEN_CW,GSSCREEN_CH);
    }
__inline__ void gfxPushScissor()
    {
    screenScissorTemp.x = screenScissor.x;
    screenScissorTemp.y = screenScissor.y;
    screenScissorTemp.w = screenScissor.w;
    screenScissorTemp.h = screenScissor.h;
    }
__inline__ void gfxPopScissor()
    {
    gfxSetScissor(screenScissorTemp.x,screenScissorTemp.y,
                  screenScissorTemp.w,screenScissorTemp.h);
    }



void gfxInitVideo(int mode)
    {
	int videoDirX=0, videoDirY=0;
	int videoResX=0, videoResY=0;
	int videoImode=0;
	int lalibMode =0;
	videoMode=mode;

	// si es auto, detectamos si es PAL o NTSC
	if(mode == GFX_AUTO)
		mode = (LIB_CRT_AUTO==LIB_CRT_PAL)?GFX_PAL:GFX_NTSC;

	// y ponemos la resolucion
 	switch(mode)
 		{
 		case GFX_PAL:
 				videoDirX=640; videoDirY=512;
 				videoResX=640; videoResY=512;
 				videoImode=LIB_CRT_INTERLACE;
 				lalibMode =LIB_CRT_PAL;
 				break;
 		case GFX_NTSC:
 				videoDirX=640; videoDirY=512;
 				videoResX=640; videoResY=448;
 				videoImode=LIB_CRT_INTERLACE;
 				lalibMode =LIB_CRT_NTSC;
 				break;
 		}

    // Init Video Mode
    libGsResetPath();
    libDmaReset(1);
    
    // Arrancamos el modo grafico
    libGsResetGraph(0,videoImode,lalibMode,LIB_CRT_FIELD);
    libGsSetDefDBuff(&db,LIB_GS_PSMCT24,videoResX,videoResY,LIB_TEST_ZGEQUAL,LIB_GS_PSMZ24,LIB_DB_CLEAR);

    // guardamos el interlace , PSM y zbuff (24 bits)
    GSSCREEN_interMode = videoImode;
    GSSCREEN_PSMCT     = LIB_GS_PSMCT24;
    GSSCREEN_PSMZ      = LIB_GS_PSMZ24;

    // guardamos la resolucion de pantalla, de momento esto solo se va
    // a usar para el Scissor
    screenSize.w  = videoResX;
    screenSize.h  = videoResY;
    GSSCREEN_CW = videoResX / 2;
    GSSCREEN_CH = videoResY / 2;

    // Set VRAM Dir
    setVideoBaseDir(videoDirX,videoDirY,LIB_TEST_ZGEQUAL,GSSCREEN_PSMZ);
    gfxSetClearColor(0x0,0x0,0x0);
    FlushCache(0);
    
    //
    screenScissor.x = -GSSCREEN_CW;
    screenScissor.y = -GSSCREEN_CH;
    screenScissor.w =  GSSCREEN_CW;
    screenScissor.h =  GSSCREEN_CW;

    gfxSetScissor(-GSSCREEN_CW,-GSSCREEN_CH,GSSCREEN_CW,GSSCREEN_CH);
    
	}
	
int gfxNTSCy(int y){
	if(videoMode==GFX_NTSC){
		//printf("PAL: %d == NTSC: %d\n",y,((y*480)/512));
		return ((y*448)/512);
	}else{
		return y;
	}
}
	
/*
#define PRIM_ANTIALIAS
__inline__ void gfxEnable(int p)
    {

    }
*/
__inline__ void gfxSetClearColor(u_char r,u_char g, u_char b)
    {
    db.clear0.rgbaq.R = r; db.clear1.rgbaq.R = r;
    db.clear0.rgbaq.G = g; db.clear1.rgbaq.G = g;
    db.clear0.rgbaq.B = b; db.clear1.rgbaq.B = b;
    }

__inline__ void gfxSyncV()
    {
    libGsSyncV(0);
    }

  
__inline__ void gfxInitPacket()    
   {
   gsPacketCount=1; // reserva un qword para el gifTag
   }

__inline__ void gfxSwapBuffers()
    {
    buffer = 1-buffer;
    libGsSwapDBuff(&db, buffer);
    gfxInitPacket();
    }
    
__inline__ void gfxRender()
    {
    // escribimos el tag al final del proceso para decirle cuantos
    // paquetes hemos metido
    gsPacket[0].u64[0]=(u_long)LIB_GIF_SET_TAG(gsPacketCount-1,1,0,0,0,1);
    gsPacket[0].u64[1]=0xEL;
    FlushCache(0);
    // envia el gifTag al GS
    libGsPutDrawEnv((libGifTag *)gsPacket);
    gfxInitPacket();
    }


__inline__ void gfxClear()
    {
    layerZReset();
    setGSLayerZ(0);
    /*
    gfxDrawBoxGoraud(
        -320,-240, 0x00, 0x00, 0x00,
         320,-240, 0x00, 0x00, 0x00,
        -320, 240, 0x00, 0x00, 0x00,
         320, 240, 0x00, 0x00, 0x00);
    */
    gsPacketCount+=libGsSetDefAlphaEnv((libGsAlphaEnv *)(&gsPacket[gsPacketCount]), 0);
    }

void gfxSetAlphaBlending(int status)
    {
    gfxABE = status;
    }
    
int gfxGetAlphaBlending()
    {
    return gfxABE;
    }


// Layer Z

void setGSLayerZ(unsigned int v)
    {
    // reset
    GSInfo.PRIM = -1;
    GSInfo.IIP  = -1;
    GSInfo.ABE  = -1;
    // --
    GSLayerZ = v;
    }

void layerZCalcMax()
    {
    int i;
    for(i=0;i<MAXLAYERZ-1;i++)
        layerz.max[i] = layerz.min[i+1];

    layerz.max[layerz.nLayers-1] = 0xFFFFFF; // 24
    }

void layerZInit(int n)
    {
    int i;
    if(n>MAXLAYERZ) n=MAXLAYERZ;

    layerz.nLayers = n;
    for(i=0;i<MAXLAYERZ;i++) 
        layerz.min[i] = 0;

    layerZCalcMax();
    }

void layerZStartAt(int n,int minvalue)
    {
    if(n>MAXLAYERZ) return;
    layerz.min[n] = minvalue;
    layerZCalcMax();
    }


void layerZReset()
    {
    layerZActive(0); // simplemente activamos la primera capa
    }
    
void layerZActive(int n)
    {
    if(n>MAXLAYERZ) return;

    layerz.active = n;
    setGSLayerZ(layerz.min[n]);
    }
unsigned int layerZGetActive()
    {
    return layerz.active;
    }

unsigned int layerZGetMin(int n)
    {
    if(n>MAXLAYERZ) return 0;
    return layerz.min[n];
    }

unsigned int layerZGetMax(int n)
    {
    if(n>MAXLAYERZ) return 0;
    return layerz.max[n];
    }


// -------------------- //
// BASIC 2D PRIMITIVES //
// ------------------ //
void gfxClearPrimTag()
    {
    GSInfo.PRIM=-1;
    }

void send2DPrimTag(u_char prim,u_char iip)
    {
    if(GSInfo.PRIM!=prim || GSInfo.IIP!=iip || GSInfo.ABE!=gfxABE)
        {
        addPRIM(prim,        // PRIM (Primitive type)
                iip,         // IIP  (Gouraud)
                0,           // TME  (Textured)
                0,           // FGE  (Fogging)
                gfxABE,      // ABE  (Alpha Blending)
                0,           // AA1  (Anti-Aliasing)
                0,           // FST  (Use ST for texture coords)
                0,           // CTXT (Context)
                0);          // FIX  (Fragment control)
        
        GSInfo.PRIM = prim;
        GSInfo.IIP  = iip;
        GSInfo.ABE  = gfxABE;
        }
    }

void gfxSetColor(u_char r, u_char g, u_char b,u_char alpha)
    {
    addRGBAQ(r,g,b,alpha,1.0f);
    }

__inline__ void gfxDrawPoint(int x,int y)
    {
    send2DPrimTag(LIB_PRIM_POINT,0);
    addXYZ2(GSCENTERX + x,GSCENTERY + gfxNTSCy(y), GSLayerZ);
    }
    
__inline__ void gfxDrawLine(int x1,int y1,int x2,int y2)
    {
    send2DPrimTag(LIB_PRIM_LINE,0);
    addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    }

__inline__ void gfxDrawTriangle(int x1,int y1,int x2,int y2,int x3,int y3)
    {
    send2DPrimTag(LIB_PRIM_TRIANGLE,0);
    addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    addXYZ2(GSCENTERX + x3,GSCENTERY + gfxNTSCy(y3),GSLayerZ);
    GSLayerZ++;
    }

__inline__ void gfxDrawBox(int x1,int y1,int x2,int y2)
    {
    send2DPrimTag(LIB_PRIM_TRIANGLE,0);
    addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y2),GSLayerZ);    
    GSLayerZ++;
    }
	
__inline__ void drawBoxLined(int x1,int y1,int x2,int y2)
    {
    gfxDrawLine( x1, (y1), x2, (y1));
    gfxDrawLine( x1, (y2), x2, (y2));
    gfxDrawLine( x1, (y1), x1, (y2));
    gfxDrawLine( x2, (y1), x2, (y2));
    }
    

void drawBoxLined2(int x1,int y1,int x2,int y2, int px)
    {
    gfxDrawBox(x1,y1,x2,y1+px);
    gfxDrawBox(x1,y2,x2,y2-px);
    gfxDrawBox(x1,y1,x1-px,y2);
    gfxDrawBox(x2,y1,x2+px,y2);
    }
	
__inline__ void gfxDrawTriangleGoraud(
                            int x1,int y1,u_char r1, u_char g1, u_char b1,
                            int x2,int y2,u_char r2, u_char g2, u_char b2,
                            int x3,int y3,u_char r3, u_char g3, u_char b3)
    {
    send2DPrimTag(LIB_PRIM_TRIANGLE,1);
    addRGBAQ(r1,g1,b1,0xFF,1.0f); addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addRGBAQ(r2,g2,b2,0xFF,1.0f); addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    addRGBAQ(r3,g3,b3,0xFF,1.0f); addXYZ2(GSCENTERX + x3,GSCENTERY + gfxNTSCy(y3),GSLayerZ);
    GSLayerZ++;
    }
    
__inline__ void gfxDrawBoxGoraud(
                       int x1,int y1,u_char r1, u_char g1, u_char b1,
                       int x2,int y2,u_char r2, u_char g2, u_char b2,
                       int x3,int y3,u_char r3, u_char g3, u_char b3,
                       int x4,int y4,u_char r4, u_char g4, u_char b4)
    {
    send2DPrimTag(LIB_PRIM_TRIANGLE,1);
    addRGBAQ(r1,g1,b1,0xFF,1.0f); addXYZ2(GSCENTERX + x1,GSCENTERY + gfxNTSCy(y1),GSLayerZ);
    addRGBAQ(r2,g2,b2,0xFF,1.0f); addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    addRGBAQ(r3,g3,b3,0xFF,1.0f); addXYZ2(GSCENTERX + x3,GSCENTERY + gfxNTSCy(y3),GSLayerZ);
    addRGBAQ(r2,g2,b2,0xFF,1.0f); addXYZ2(GSCENTERX + x2,GSCENTERY + gfxNTSCy(y2),GSLayerZ);
    addRGBAQ(r3,g3,b3,0xFF,1.0f); addXYZ2(GSCENTERX + x3,GSCENTERY + gfxNTSCy(y3),GSLayerZ);
    addRGBAQ(r4,g4,b4,0xFF,1.0f); addXYZ2(GSCENTERX + x4,GSCENTERY + gfxNTSCy(y4),GSLayerZ);
    GSLayerZ++;
    }

__inline__ void gfxDrawCircle(int x,int y,int radius,int step)
    {
    gfxDrawOval(x,gfxNTSCy(y),radius,radius,step);
    }

void gfxDrawOval(int x,int y,int radiusX,int radiusY,int step)
    {
    int i;

    send2DPrimTag(LIB_PRIM_TFAN,0);
    addXYZ2(GSCENTERX + x,GSCENTERY + gfxNTSCy(y),GSLayerZ);

    for(i=0;i<step+1;i++)
        {
        float angulo = (360.0f / step) * i;
        float angle  = angulo * PI / 180.0f;
        
        addXYZ2(GSCENTERX + x - (sinf(angle) * radiusX),
                GSCENTERY + gfxNTSCy(y) - (cosf(angle) * radiusY),
                GSLayerZ);
        }
    GSLayerZ++;
    gfxClearPrimTag();
    }    


void gfxDrawPolygon(int x,int y,libVu0FVector *v,int n, int filled)
    {
    int i;

    if(filled)
         send2DPrimTag(LIB_PRIM_TFAN     ,0);
    else send2DPrimTag(LIB_PRIM_LINESTRIP,0);

    for(i=0;i<n;i++)
        addXYZ2(GSCENTERX + x + v[i][0],
                GSCENTERY + gfxNTSCy(y) + v[i][1],
                GSLayerZ);
        
    // si es un LineStrip, tenemos que envia de nuevo el primer punto
    // al final para que lo cierre
    if(!filled)
        addXYZ2(GSCENTERX + x + v[0][0],
                GSCENTERY + gfxNTSCy(y) + v[0][1],
                GSLayerZ);

    GSLayerZ++;
    gfxClearPrimTag();
    }


