/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/


#include <llibPlanar.h>
#include <string.h>

int fd;

IIF_HEADER *IIF_OpenFile(char *filename)
    {
    IIF_HEADER *iif_file;

    // Abrimos el Archivo
    fd = openFile(filename, O_RDONLY);

    // Preparamos la cabecera
    iif_file = (IIF_HEADER *)malloc(sizeof(IIF_HEADER));

    // y la leemos
    readFile(fd, iif_file, sizeof(IIF_HEADER));

    return iif_file;
    }
    
void IIF_CloseFile()
    {
    closeFile(fd);
    }

int IIF_isAnimated(IIF_HEADER *iif_file)
    {
    return (iif_file->psm == IIF_GIF256 || iif_file->psm == IIF_GIF16);
    }

int IIF_countFrames(IIF_HEADER *iif_file)
    {
    // si no es un iif animado, entonces solo tenemos 1 frame
    // pa que vamos a ponernos a contarlos
    if (!IIF_isAnimated(iif_file)) return 1;

    // vamos a contar cuantos frames tiene el archivo
    // para ello primero miramos cuantos bytes ocupa 1 frame
    int frame_size=iif_file->width*iif_file->height;
    if(iif_file->psm == IIF_GIF16 ) frame_size /=2; // en 16bits son menos
    
    // miramos cuanto ocupa el Header
    int header_size = sizeof(IIF_HEADER);
   	switch(iif_file->psm)
   		{
        case IIF_GIF256 : header_size += (256*4); break;
        case IIF_GIF16  : header_size += (16*4);  break;
        }

    // miramos cuanto ocupa el archivo
 	int file_size = seekFile(fd, 0, SEEK_END); // file len

 	// y ahora a calcular
 	int frames = (file_size - header_size) / frame_size;
 	printf("Header Size = %d \n", header_size);
 	printf("File size = %d \n", file_size);
 	printf("Frame size = %d \n", frame_size);
	printf("Frames = %d \n", frames);

 	// dejamos el cursor al inicio de los frames
	seekFile(fd, header_size, SEEK_SET);

    return frames;
    }

unsigned char *IIF_readClut(IIF_HEADER *iif_file)
    {
    int size=0;
    unsigned char *data;

    // miramos cuanto ocuparia el CLUT
   	switch(iif_file->psm)
   		{
   		case IIF_CLUT8_RGBA32: size = 256*4; break;
   		case IIF_CLUT8_RGBA16: size = 256*2; break;
   		case IIF_CLUT4_RGBA32: size = 16*4;  break;
   		case IIF_CLUT4_RGBA16: size = 16*2;  break;
        case IIF_GIF256      : size = 256*4; break;
        case IIF_GIF16       : size = 16*4;  break;
        }
    // Leemos el clut
    data = readFileBlock(fd,size);

    return data;
    }
    
unsigned char *IIF_readImage(IIF_HEADER *iif_file)
    {
    int size = 0;
    unsigned char *data;

    // de entrada el size es W*h
    size=iif_file->width*iif_file->height;

    // ahora vamos a ver cuando ocupa mas de 1 byte
    switch(iif_file->psm)
        {
        case IIF_RGBA32      : size *= 4; break;
        case IIF_RGB24       : size *= 3; break;
        case IIF_RGBA16      : size *= 2; break;
   		// case IIF_CLUT8_RGBA32: break;
   		// case IIF_CLUT8_RGBA16: break;
   		case IIF_CLUT4_RGBA32: size /= 2;  break;
   		case IIF_CLUT4_RGBA16: size /= 2;  break;
        // case IIF_GIF256      : break;
        case IIF_GIF16       : size /= 2;  break;
        }

    // y lo leemos
    data = readFileBlock(fd,size);

    return data;
    }

int IIF_hasMoreData(IIF_HEADER *iif_file)
	{
	return 0;
	}    


/////VIRTUAL

IIF_HEADER *IIF_vOpenFile(char *filename, int size)
    {
    IIF_HEADER *iif_file;
    // Abrimos el Archivo
    vopenFile(filename, size);

    // Preparamos la cabecera
    iif_file = (IIF_HEADER *)malloc(sizeof(IIF_HEADER));

    // y la leemos
	memcpy(iif_file,vreadFile(sizeof(IIF_HEADER)),sizeof(IIF_HEADER));

    return iif_file;
    }
    
void IIF_vCloseFile()
    {
    vcloseFile();
    }

int IIF_vcountFrames(IIF_HEADER *iif_file)
    {
    // si no es un iif animado, entonces solo tenemos 1 frame
    // pa que vamos a ponernos a contarlos
    if (!IIF_isAnimated(iif_file)) return 1;

    // vamos a contar cuantos frames tiene el archivo
    // para ello primero miramos cuantos bytes ocupa 1 frame
    int frame_size=iif_file->width*iif_file->height;
    if(iif_file->psm == IIF_GIF16 ) frame_size /=2; // en 16bits son menos
    
    // miramos cuanto ocupa el Header
    int header_size = sizeof(IIF_HEADER);
   	switch(iif_file->psm)
   		{
        case IIF_GIF256 : header_size += (256*4); break;
        case IIF_GIF16  : header_size += (16*4);  break;
        }

    // miramos cuanto ocupa el archivo
 	int file_size = vseekFile(0, SEEK_END); // file len

 	// y ahora a calcular
 	int frames = (file_size - header_size) / frame_size;
 	printf("Header Size = %d \n", header_size);
 	printf("File size = %d \n", file_size);
 	printf("Frame size = %d \n", frame_size);
	printf("Frames = %d \n", frames);

 	// dejamos el cursor al inicio de los frames
	vseekFile(header_size, SEEK_SET);

    return frames;
    }

unsigned char *IIF_vreadClut(IIF_HEADER *iif_file)
    {
    int size=0;
    unsigned char *data;

    // miramos cuanto ocuparia el CLUT
   	switch(iif_file->psm)
   		{
   		case IIF_CLUT8_RGBA32: size = 256*4; break;
   		case IIF_CLUT8_RGBA16: size = 256*2; break;
   		case IIF_CLUT4_RGBA32: size = 16*4;  break;
   		case IIF_CLUT4_RGBA16: size = 16*2;  break;
        case IIF_GIF256      : size = 256*4; break;
        case IIF_GIF16       : size = 16*4;  break;
        }
    // Leemos el clut
    data = vreadFileBlock(size);

    return data;
    }
    
unsigned char *IIF_vreadImage(IIF_HEADER *iif_file)
    {
    int size = 0;
    unsigned char *data;

    // de entrada el size es W*h
    size=iif_file->width*iif_file->height;

    // ahora vamos a ver cuando ocupa mas de 1 byte
    switch(iif_file->psm)
        {
        case IIF_RGBA32      : size *= 4; break;
        case IIF_RGB24       : size *= 3; break;
        case IIF_RGBA16      : size *= 2; break;
   		// case IIF_CLUT8_RGBA32: break;
   		// case IIF_CLUT8_RGBA16: break;
   		case IIF_CLUT4_RGBA32: size /= 2;  break;
   		case IIF_CLUT4_RGBA16: size /= 2;  break;
        // case IIF_GIF256      : break;
        case IIF_GIF16       : size /= 2;  break;
        }

    // y lo leemos
    data = vreadFileBlock(size);

    return data;
    }
