/***************************************************************
/  Unofficial LlibPlanar - Versin 0.9.1                       /
/                                                              /
/  Modificado por Ifcaro <taslokooo@gmail.com>                 /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Versin original:                                           /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include <libVu0.h>
#include <kernel.h>

void vu0NormalizaVector(libVu0FVector *v0, libVu0FVector *v1) {
    asm __volatile__("\n"
    "lqc2        vf16,0x0(%1)\n"
	"vmul.xyz    vf17,vf16,vf16\n"
    "vmulax.w    ACC,vf00,vf17x\n"
    "vmadday.w   ACC,vf00,vf17y\n"
    "vmaddz.w    vf17,vf00,vf17z\n"
    "vrsqrt      Q,vf00w,vf17w\n"
    "vwaitq\n"
    "vmulq.xyz   vf16,vf16,Q\n"
    "sqc2        vf16,0x0(%0)\n"
    ""
    : /* No Output */
    : "r"(v0), "r"(v1)
    : "memory"
    );
}

void vu0CalculaNormal(libVu0FVector *vn, libVu0FVector *vac, libVu0FVector *vbc, libVu0FVector *vcc) {
    asm __volatile__("\n"
    "lqc2              vf16,0x0(%1)\n"
    "lqc2              vf17,0x0(%2)\n"
    "lqc2              vf18,0x0(%3)\n"
    "\n"
    "\n"	
    "vsub.xyz          vf17,vf16,vf17;\n"
    "vsub.xyz          vf18,vf16,vf18;\n"
    "\n"
    "#producto cruz\n"
    "vopmula.xyz       ACC,vf17,vf18\n"
    "vopmsub.xyz       vf18,vf18,vf17\n"
    "vsub.w            vf18,vf00,vf00  # w = 0\n"
    "\n"
    "#normailizar\n"
    "vmul.xyz          vf17,vf18,vf18\n"
    "vmulax.w          ACC,vf00,vf17x\n"
    "vmadday.w         ACC,vf00,vf17y\n"
    "vmaddz.w          vf17,vf00,vf17z\n"
    "vrsqrt            Q,vf00w,vf17w\n"
    "vwaitq\n"
    "vmulq.xyz         vf18,vf18,Q\n"
    "\n"
    "\n"	
    "sqc2              vf18,0x0(%0)\n"
    ""
    : /* No Output */
    : "r"(vn), "r"(vac), "r"(vbc), "r"(vcc)
    : "memory"
    );
}

float vu0DotProduct(libVu0FVector *v0, libVu0FVector *v1) {
    float dot;
    asm __volatile__("\n"
    "lqc2        vf01,0x0(%1)\n"
    "lqc2        vf02,0x0(%2)\n"
	"\n"
    "vmul.xyz    vf01,vf01,vf02      # xx,yy,zz\n"
    "vmaxw.x     vf02,vf00,vf00      # 1,-,-,-\n"
	"\n"
    "vadday.x    ACC,vf01,vf01       # xx+yy\n"
    "vmaddz.x    vf01,vf02,vf01      # xx+yy+zz\n"
	"\n"
    ".set        noat\n"
    "qmfc2       $1,vf01\n"
    "mtc1        $1,%0\n"
    ".set        at\n"
    ""
    : "=f"(dot)
    : "r"(v0), "r"(v1)
    : "$1"
    );
return dot;
}

void vu0_getNormal(libVu0FVector *vn, libVu0FVector *vac, libVu0FVector *vbc, libVu0FVector *vcc)
	{
        asm __volatile__(
        "lqc2              vf16,0x0(%1)    \n\t"
        "lqc2              vf17,0x0(%2)    \n\t"
        "lqc2              vf18,0x0(%3)    \n\t"

        "vsub.xyz          vf17,vf16,vf17  \n\t"
        "vsub.xyz          vf18,vf16,vf18  \n\t"

        "vopmula.xyz       ACC,vf17,vf18   \n\t"
        "vopmsub.xyz       vf18,vf18,vf17  \n\t"
        "vsub.w            vf18,vf00,vf00  \n\t"

        "vmul.xyz          vf17,vf18,vf18  \n\t"
        "vmulax.w          ACC,vf00,vf17x  \n\t"
        "vmadday.w         ACC,vf00,vf17y  \n\t"
        "vmaddz.w          vf17,vf00,vf17z \n\t"
        "vrsqrt            Q,vf00w,vf17w   \n\t"
        "vwaitq                            \n\t"
        "vmulq.xyz         vf18,vf18,Q     \n\t"

        "sqc2              vf18,0x0(%0)    \n\t"

        : /* No Output */
        : "r"(vn), "r"(vac), "r"(vbc), "r"(vcc)
        : "memory"
    );
}


float vu0_getAngle(libVu0FVector *v0, libVu0FVector *v1)
	{
        float dot;
        asm __volatile__(
        "lqc2        vf01,0x0(%1)    \n\t"
        "lqc2        vf02,0x0(%2)    \n\t"

        "vmul.xyz    vf01,vf01,vf02  \n\t"
        "vmaxw.x     vf02,vf00,vf00  \n\t"

        "vadday.x    ACC,vf01,vf01   \n\t"
        "vmaddz.x    vf01,vf02,vf01  \n\t"

        ".set        noat            \n\t"
        "qmfc2       $1,vf01         \n\t"
        "mtc1        $1,%0           \n\t"
        ".set        at              \n\t"

        : "=f"(dot)
        : "r"(v0), "r"(v1)
        : "$1"
        );
        return dot;
    }

void vu0_normalice(libVu0FVector *v0, libVu0FVector *v1)
	{
        asm __volatile__(
        "lqc2        vf16,0x0(%1)    \n\t"
        "vmul.xyz    vf17,vf16,vf16  \n\t"
        "vmulax.w    ACC,vf00,vf17x  \n\t"
        "vmadday.w   ACC,vf00,vf17y  \n\t"
        "vmaddz.w    vf17,vf00,vf17z \n\t"
        "vrsqrt      Q,vf00w,vf17w   \n\t"
        "vwaitq                      \n\t"
        "vmulq.xyz   vf16,vf16,Q     \n\t"
        "sqc2        vf16,0x0(%0)    \n\t"

        : /* No Output */
        : "r"(v0), "r"(v1)
        : "memory"
        );
    }
