
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iif.h"
#include "loadClut.h"


#define IIF_RESET   0
#define IIF_PALETTE 1
#define IIF_IMAGE   2


#define RLUM    (0.3086f)
#define GLUM    (0.6094f)
#define BLUM    (0.0820f)
int getLuminance(unsigned char r, unsigned char g,unsigned char b)
    {
    return r*RLUM + g*GLUM + b*BLUM ;
    }


FILE *f;
IIF_HEADER *header;

int clutOpen(char *filename)
    {
    f = fopen(filename,"rb+");
	if(f==NULL)
        {	
		printf("Error: Could not open file \"%s\" for reading.\n", filename);
		return -1;
	    }
    header = (IIF_HEADER*)malloc(sizeof(IIF_HEADER));
    fread(header, 1, sizeof(IIF_HEADER), f);
    
    if(!clutIsValid())
        {
		printf("Error: Only CLUT files, thanks");
        clutClose();
		return -1;
        }
        
    printf("Open File : %s \n",filename);
    printf("Clut mode : %s \n",clutGetName());
            
        
        
    return 0;                    
    }
    
void clutClose()
    {
    free(header);
    fclose(f);
    }    
    
int clutIsValid()    
    {
    return (clutGetType()>IIF_RGBA16);
    }
     
int clutGetType()
    {
    return header->psm;
    }

void clutResetPointer(int pos)
    {
    int position = 0;
    switch(pos)
        {
        case IIF_RESET: 
                 position = 0; 
                 break;
        case IIF_PALETTE: 
                 position = sizeof(IIF_HEADER); 
                 break;
        case IIF_IMAGE:
                 position = sizeof(IIF_HEADER) + clutGetSizePalette();
                 break;
        }                   
                       
    fseek(f, position, SEEK_SET);                       
    }


char *clutGetName()
    {
    switch(clutGetType())         
        {
        case IIF_CLUT8_RGBA32: return "Clut8_RGBA32"; break;
        case IIF_CLUT8_RGBA16: return "Clut8_RGBA16"; break;

        case IIF_CLUT4_RGBA32: return "Clut4_RGBA32"; break;
        case IIF_CLUT4_RGBA16: return "Clut4_RGBA16"; break;

        case IIF_GIF256      : return "Clut8_RGBA32"; break;
        case IIF_GIF16       : return "Clut4_RGBA32"; break;
        }
    }
    
int clutGetSizePalette()
    {
    int size=0;
    switch(clutGetType())         
        {
        case IIF_CLUT8_RGBA32: size = 256 * 4; break;
        case IIF_CLUT8_RGBA16: size = 256 * 2; break;
        case IIF_CLUT4_RGBA32: size =  16 * 4; break;
        case IIF_CLUT4_RGBA16: size =  16 * 2; break;
        case IIF_GIF256      : size = 256 * 4; break;
        case IIF_GIF16       : size =  16 * 4; break;
        }
    return size;
    }     

long clutGetSizeImage()
    {
    return header->height * header->width;
    }     
    
unsigned char* clutLoadPalette()
    {
    unsigned char *data;
    int size = clutGetSizePalette();

    data = (unsigned char *) malloc(size);
    clutResetPointer(IIF_PALETTE);
    fread(data, size, 1, f);         
         
    return data;         
    }


void clutUpdatePalette(unsigned char *data)
    {
    int size = clutGetSizePalette();
    
    clutResetPointer(IIF_PALETTE);
    fwrite(data, size, 1, f);         
    }


unsigned char* clutLoadImage()
    {
    unsigned char *data;
    long size = clutGetSizeImage();
    
    data = (unsigned char *) malloc(size);
    clutResetPointer(IIF_IMAGE);
    fread(data, size, 1, f);         
         
    return data;         
    }


void clutUpdateImage(unsigned char *data)
    {
    long size = clutGetSizeImage();
    
    clutResetPointer(IIF_IMAGE);
    fwrite(data, size, 1, f);         
    }
